#include "asprestr.h"
#include "astroaspects.h"
#include "astroresources.h"
#include <qlabel.h>
#include "mylineedit.h"
#include "astroutils.h"
#include "astrographics.h"
#include <qpushbutton.h>
#include <qpixmap.h>
#include <cstdlib>

/* 
 *  Constructs a AspRest which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */

AspRest::AspRest(AspRestList* arl, int i)
    : _AspRestr(0, 0, 0 )
{
  AstroString as;
  Arl = arl;
  Item = i;
  char buf[6];
  const struct AspectRestrictions& asrt = Arl->Asrt[i];
  emit Name->setText(*Asr->AspectNames[i]);
  emit IsRestricted->setChecked(!asrt.Rest);
  sprintf(buf, "%.2f", asrt.Orb);
  as = buf;
  emit Orb->setText(as);
  Orb->setAlignment(Qt::AlignRight);
  emit Influence->setText(Sprintf("%d", &as, asrt.Influence));
  Influence->setAlignment(Qt::AlignRight);
  emit ColorButton->setPixmap(COL % asrt.Color);
}

AspRestList::AspRestList(QWidget* parent, const char *) : QScrollView(parent)
{
  int i;
  QString s;
  Qppm.setCheckable(TRUE); 
  connect(&Qppm, SIGNAL(activated(int)), this, SLOT(Activated(int)));
  for(i = 0; i < 16; i++)
    Qppm.insertItem(COL % i, i);
}

AspRestList::~AspRestList()
{
}

void AspRestList::Init(struct AspectRestrictions* asrt)
{
  int i, w;
  Asrt = asrt;
  Ar[Conjunction] = new AspRest(this, Conjunction);
  addChild(Ar[Conjunction], 0, 0);
  w = Ar[Conjunction]->width();
  Y = H = Ar[Conjunction]->height();
  FillList(Opposition, Unknown_Aspect);
  emit resizeContents(w, Y);
}

void AspRestList::GetLen(int* a, int* b, int* c, int *d, int *e) const
{
  const AspRest& arc = *Ar[Conjunction];
  *a = arc.IsRestricted->x();
  *b = arc.Orb->x() - *a;
  *c = arc.Influence->x() - arc.Orb->x();
  *d = arc.ColorButton->x()- arc.Influence->x();
  *e = width() - arc.ColorButton->x(); //width();
}

void AspRestList::Inhibit(int i, bool b)
{
  AspRest& ari = *Ar[i];
  emit ari.Name->setEnabled(!b);
  emit ari.IsRestricted->setEnabled(!b);
  emit ari.Influence->setEnabled(!b);
  emit ari.Orb->setEnabled(!b);
  emit ari.ColorButton->setEnabled(!b);
}

void AspRestList::Check(int i, bool b)
{
  emit Ar[i]->IsRestricted->setChecked(b);
  Asrt[i].Rest = b; 
}

void AspRestList::FillList(int first, int last)
{
  int i;
  for(i = first; i < last; i++)
  {
    Ar[i] = new AspRest(this, i);
    addChild(Ar[i], 0, Y);
    Y += H;
  }
}

void AspRestList::Activated(int i)
{
  Asrt[Item].Color = i;
  emit Ar[Item]->ColorButton->setPixmap(COL % i);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
AspRest::~AspRest()
{
    // no need to delete child widgets, Qt does it all for us
}

/* 
 * public slot
 */
void AspRest::InfluenceSlot(const QString& s)
{
  int i = atoi(s.data());
  if(i >= 0 && i <= 50)
    Arl->Asrt[Item].Influence = i;
  else
    AstroMessage(i18n("The influence value must be defined within the interval 0 ... 50"));
}
/* 
 * public slot
 */
void AspRest::OrbSlot(const QString& s)
{
    double d = atof(s.data());
  if(d >= -10.0 && d <= 10.0) 
    Arl->Asrt[Item].Orb = d;
  else
    AstroMessage(i18n("The orb value must be defined within the interval 0 ... 10.0"));
}

/* 
 * public slot
 */
void AspRest::RestrictSlot(bool b)
{ 
  Arl->Asrt[Item].Rest = !b;
}

/* 
 * public slot
 */
void AspRest::ColorButtonSlot()
{
  int i;
  Arl->Item = Item;
  for(i = 0; i < 16; i++)
    Arl->Qppm.setItemChecked(i, FALSE);
  Arl->Qppm.setItemChecked(Arl->Asrt[Item].Color, TRUE);
  Arl->Qppm.exec(QCursor::pos());
}

bool AspRestList::IsChecked(int i) const
{
  return Ar[i]->IsRestricted->isChecked();
}
