/***************************************************************************
                          astroapp.cpp  -  description
                             -------------------
    begin                : Wed Oct 18 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astroapp.h"
#include "astrodefs.h"
#include "astroparams.h"
#include "astrosigns.h"
#include "astroresources.h"
#include "astrodata.h"
#include "astrochartbase.h"
#include "astrowindow.h"
#include "stdwheelchart.h"
#include "astrowindow.h"
#include "astrolist.h"
#include "astroaspects.h"
#include "astroprocess.h"
#include "astrointerface.h"
#include "astromainwindow.h"
#include "astrorestrictions.h"
#include "astrosql.h"
#include <iostream.h>
#include <cstdlib>
#ifdef KDE
#include <kapp.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#endif

#ifdef KDE
#define ARG(i) args->arg(i - 1)
#endif

// The global AstroSubChartList pointer
AstroSubChartList* Asl;

// The global AstroSigns Signs definitions object pointer
AstroSigns* Asg;

// The global AstroResources object pointer;
AstroResources* Asr;

// The global AstroData list
AstroDataList* Adl;

// The global AstroChart list
AstroChartList* Ahl;

// The Chart icons list
AstroIconsList* Ail;

//The global colors list
AstroColorList* Acl;

// The global restrictions list
AstroRestrictionsList* Arl;

// The computing server object
AstroProcServer *Aps;

// The standard interface object for non-reentrant routines
AstroInterface* Asf;

// The main window pointer
AstroMainWindow* Amw;

#ifdef KDE
 KApplication *Kaptr;
#endif

// The global "Now" data
AstroData* Now;

// The global default restrictions set
AstroRestrictions* Drs;

// The SQL access point
AstroSQL* Asq;

AstroApp::AstroApp(int argc, char *argv[])
{
  int count;
  AstroWindow* Aw;
  Aps = new AstroProcServer;
#ifdef KDE
  KAboutData kad(PACKAGE, PACKAGE, VERSION, "A modern astrology program", KAboutData::License_GPL, 0, 0, "http://devel-home.kde.org/~kastrolog/skylendar.html", "skylendar@yahoo.com");
  kad.addAuthor("Christophe Gros", 0, "skylendar@yahoo.com");
  kad.addCredit("Alois Treindl", "swe library developer", "alois@astro.ch", "www.astro.ch");
  kad.addCredit("Dieter Koch", "swe library developer", "dieter@astro.ch", "www.astro.ch");
  KCmdLineOptions options[] = { { "+file", 0 , 0 }, { 0, 0, 0} };
  KCmdLineArgs::init(argc, argv, &kad);
  KCmdLineArgs::addCmdLineOptions(options);
  KApplication KasApp;
  Kaptr = &KasApp;
  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
  count = args->count();
  DCOPClient* dc = KasApp.dcopClient();
  if(dc->attach())
  dc->registerAs(ASTROINTERFACE);
#endif
  Asl = new AstroSubChartList;
  CHECK_PTR(Asl);
  Acl = new AstroColorList;
  CHECK_PTR(Acl);
  Asr = new AstroResources;
  CHECK_PTR(Asr);
  Asq = new AstroSQL(PACKAGE, PACKAGE);
  CHECK_PTR(Asq);
  if(!Asq->IfOk())
  {
    AstroMessage("Can't connect to SQL Database !");
    Aps->KillAll();
  }
  Asr->SetDefZFile();
  Asg = new AstroSigns;
  CHECK_PTR(Asg);
  Arl = new AstroRestrictionsList;
  CHECK_PTR(Arl);
  AstroComputing Ac;
  Asf = CreateInterface(FALSE);
  Adl = new AstroDataList;
  CHECK_PTR(Adl);
  Ahl = new AstroChartList;
  CHECK_PTR(Ahl);
  Ail = new AstroIconsList;
  CHECK_PTR(Ail);
  AstroData *test;
  if(count == 0)
    test = Now;
  else if(count == 7)
  {
    int day, month, year, hour, minute;
    day    = atoi(ARG(1));
    month  = atoi(ARG(2));
    year   = atoi(ARG(3));
    hour   = atoi(ARG(4));
    minute = atoi(ARG(5));
    AstroDate ad(day, month, year, hour, minute, "GMT");
    test = new AstroData(ad, atoi(ARG(6)));
  }
  else
  {
    cout << "Usage:" << argv[0] << " day month year hour minute place_index\n";
    exit(1);
  }
  Amw = new AstroMainWindow(test);
  CHECK_PTR(Amw);
#ifdef KDE
  KasApp.exec();
#endif
}

AstroApp::~AstroApp()
{
  delete Asq;
}

int main(int argc, char *argv[])
{
  AstroApp ap(argc, argv);
  return EXIT_SUCCESS;
}
