/***************************************************************************
                          astroaspects.cpp  -  description
                             -------------------
    begin                : Mon Apr 30 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astroaspects.h"
#include "astroresources.h"
#include "astroobjs.h"

AstroAspects::AstroAspects(AstroRestrictions* ar)
{
  Asprst = ar->Asprst;
  Rest   = ar->Rest;
  IsMinor = ar->IfMinor();
}

enum Aspects AstroAspects::IsAspect(enum Objs a, double angle_a, enum Objs b, double angle_b) const
{
  int i;
  double orb;
  double angle, aasp;
  for(i = Conjunction; i <= Last_Major; i++)
  {
    orb = Asprst[i].Orb + Rest[a].OrbAdd + Rest[b].OrbAdd;
    if(orb < 0.0) continue;
    angle = fabs(angle_a - angle_b);
    if(angle - orb > 180.0)
      angle = 360.0 - angle;
    aasp = (*this) * (enum Aspects)i;
    if((angle < aasp + orb) && (angle > aasp - orb) && (*this) == (enum Aspects)i)
      return (enum Aspects)i;
  }
  if(IsMinor)
    for(i = First_Minor; i < Unknown_Aspect; i++)
    {
      orb = Asprst[i].Orb + Rest[a].OrbAdd + Rest[b].OrbAdd;
      if(orb < 0.0) continue;
      angle = fabs(angle_a - angle_b);
      if(angle - orb > 180.0)
	angle = 360.0 - angle;
      aasp = (*this) * (enum Aspects)i;
      if((angle < aasp + orb) && (angle > aasp - orb) && (*this) == (enum Aspects)i)
	return (enum Aspects)i;
    }
  return Unknown_Aspect;
}

AstroFoundAspectsList::AstroFoundAspectsList(const AstroAspects& aa, const AstroObjs& a, const AstroObjs& b, bool ifsameplanet)
{
  GetList(aa, a, b, ifsameplanet);
}

void AstroFoundAspectsList::GetList(const AstroAspects& aa, const AstroObjs& a, const AstroObjs& b, bool ifsameplanet)
{
  enum Objs oa = Last_Planet, ob = Last_Planet;
  if(a.IfHouses())
    oa = House_12;
  if(b.IfHouses())
    ob = House_12;
  if(a.IfExotic())
    oa = Last_Exotic;
  if(b.IfExotic())
    ob = Last_Exotic;
  if(a.IfStars())
    oa = Last_Star;
  if(b.IfStars())
    ob = Last_Star;
  already = new bool[oa * ob];
  memset(already, FALSE, sizeof(bool) * oa * ob); 
  Clear();
  IfSamePlanet = ifsameplanet;
  TestAspects(aa, a, b, oa, ob);
  delete already;
}

void AstroFoundAspectsList::TestAspects(const AstroAspects& aa, const AstroObjs& a, const AstroObjs& b, int obj_a, int obj_b)
{
  int i, j;
  bool s = a.IfStars() &&b.IfStars();
  for(i = Sun; i <= obj_a; i++)
  {
    for(j = Sun; j <= obj_b; j++)
    {
      if(already[i, j])
	continue;
      already[j, i] = TRUE;
      if(!(a == i) || !(b == j) || (i == j && !IfSamePlanet)) continue;
      if(s && i >= First_Star && j >= First_Star) continue;
      enum Objs oa = (enum Objs)i, ob = (enum Objs)j;
      double angle_a = a * i, angle_b = b * j;
      enum Aspects asp = aa.IsAspect(oa, angle_a, ob, angle_b);
      if(asp != Unknown_Aspect)
	Append(new AspectFound(aa, asp, oa, angle_a, ob, angle_b));
    }
  }
}

AspectFound::AspectFound(const AstroAspects& aa, enum Aspects asp, enum Objs oa, double angle_a, enum Objs ob, double angle_b)
{
  const struct Rest* Rest = aa.GetRest();
  Aa = (AstroAspects *)&aa;
  Asp = asp;
  Oa = oa;
  Ob = ob;
  Angle_a = angle_a;
  Angle_b = angle_b;
  Orb = (*Aa) / asp + Rest[oa].OrbAdd + Rest[ob].OrbAdd;
}

enum AspectFound::AspectAccuracy AspectFound::GetAccuracy()
{
  double diff = fabs(Angle_a - Angle_b) - (*Aa) * Asp;
  int x = 100 * (int)(diff / Orb);
  if(x == 0)
    return Exact;
  else if(x <= 30)
    return Good;
  else if(x <= 60)
    return Medium;
  else
    return Poor;
}
