/***************************************************************************
                          astroaspects.h  -  description
                             -------------------
    begin                : Mon Feb 7 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROASPECTS_H
#define ASTROASPECTS_H

#include "../config.h"

#include "astrodefs.h"
#include "astrolist.h"
#include "astrostring.h"
#include "astroresources.h"

class AstroObjs;
class AstroRestrictions;

struct AspectsConstants
{
  double Angle;
  enum AspectKind Kind;
  int Harmonic;
};

struct AspectRestrictions
{
  bool Rest;
  double Orb;
  int Color, Influence;
};

extern const struct AspectsConstants*  AspConstVector;
#define I enum Aspects i

/** This class manages the aspects data */
class AstroAspects
{
 public:
  
  AstroAspects(AstroRestrictions* ar);

  inline double operator * (I)   const { return AspConstVector[i].Angle; }
  inline enum AspectKind Kind(I) const { return AspConstVector[i].Kind; }
  inline int Harmonic(I)         const { return AspConstVector[i].Harmonic; }
  inline bool operator == (I)    const { return !Asprst[i].Rest; }
  inline double operator / (I)   const { return Asprst[i].Orb; }
  inline int Color(I)            const { return Asprst[i].Color; }
  inline int Influence(I)        const { return Asprst[i].Influence; }
  inline const AstroString& operator << (I) const
    { return *Asr->AspectNames[i]; }
  inline const AstroString& operator  < (I) const
    { return *Asr->AspectShortNames[i]; }
  inline const struct Rest* GetRest()   const { return Rest; }

  /** Returns the Aspect if a and b are in the corresponding aspect.
  @param a Object a
  @param b Object b
  @param angle_a Angle (in degrees for obj a)
  @param same as for obj a
  */
  enum Aspects IsAspect(enum Objs a, double angle_a, enum Objs b, double angle_b) const ;
  
 private:

  struct Rest* Rest;
  struct AspectRestrictions* Asprst;
  bool IsMinor;
};

/** class provided by  AstroAspectsList
    It contains the values of a found aspect
    Angle_a, Angle_b: angles (in degrees) of the objs Oa and Ob.
    Strength: the power of the aspect (Positive number) */

class AspectFound
{
 public:

  /* The accuracy of an aspect
     Exact - Good: less than or equal 30% of the orb
     Medium      : less than of equal 60% of the orb
     Poor        : more than 60% of the orb
  */
  enum AspectAccuracy {Exact, Good, Medium, Poor };

  /** Constructor */
  AspectFound(const AstroAspects& aa, enum Aspects a, enum Objs oa, double angle_a, enum Objs ob, double angle_b);

  enum Aspects Asp; // Found aspect.
  enum Objs Oa, Ob; // Involved objects.
  double Angle_a, Angle_b, Orb; // Angles of involved objects + actual orb.
  inline double DiffAngle() { return fabs(Angle_b - Angle_a); }
  /** Returns the accuracy of the aspect
      @return AspectAccuracy
  */
  enum AspectAccuracy GetAccuracy();

 private:

  AstroAspects* Aa; // Aspects table pointer.
};

/** This class holds a list of found aspects between AstroObjs a and b */
class AstroFoundAspectsList: public AstroList(AspectFound)
{
 public:

  /** Constructor.
   @param a ref to first objects set
   @param b ref to second objects set
   @param ifsameplanet If TRUE regard the possible aspects between the same planet, (e.g. Venus in both a and b) */

  AstroFoundAspectsList(const AstroAspects& aa, const AstroObjs& a, const AstroObjs& b, bool ifsameplanet);

  /** Same as above, but not constructor method. */
  void GetList(const AstroAspects& aa, const AstroObjs& a, const AstroObjs& b, bool ifsameplanet);

 private:

  void TestAspects(const AstroAspects& aa, const AstroObjs& a, const AstroObjs& b, int obj_a, int obj_b);
  bool* already, IfSamePlanet;
};

#endif
  
