/***************************************************************************
                          astrochart.cpp  -  description
                             -------------------
    begin                : Mon Oct 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrochart.h"
#include "astroparams.h"
#include "rawtextchart.h"
#include "stdwheelchart.h"
#include "astroprocess.h"

AstroChart::AstroChart(bool owninterface)
{
  int i;
  for(i = 0; i < NBDATA; i++)
    Ov[i] = 0;
  Acb = 0;
  Cbp = new ChartBaseParams(SC(ChartParams *,Asr), owninterface ? CreateInterface() : Asf);
  CHECK_PTR(Cbp);
  Ahl->Append(this);
}

AstroChart::~AstroChart()
{
  int i;
  if(Acb) delete Acb;
  for(i = 0; i < NBDATA; i++)
    if(Ov[i])
      delete Ov[i];
  Cbp->Aif->Kill();
  Ahl->Remove(this);
}

/** This method displays the chart according to
the Chart Type */
void AstroChart::Display()
{
  if(Acb)
  {
    if(ct != Acb->ct)
    {
      delete Acb;
      CreateChart();
    }
    else
      Acb->Redisplay();
  }
  else
    CreateChart();
}

/** This method creates an Acb according to ct */
void AstroChart::CreateChart()
{
  switch(ct)
  {
    case RawText   : Acb = new RawTextChart (Cbp, &Ov); break;
    case StdWheels : Acb = new StdWheelChart(Cbp, &Ov); break;
  }
  Acb->Redisplay();
}

void AstroChart::SetData(AstroData *ad, int i, bool placeonly)
{
  if(i < 0 || i >= NBDATA)
    return;
  if(!Ov[i])
  {
    Ov[i] = new AstroObjs(*ad, this);
    CHECK_PTR(Ov[i]);
    Ov[i]->SetInterface(Cbp->Aif);
    if(Acb)
      Acb->PostSetData(i);
  }
  else
  {
    AstroData* adp = SC(AstroData *,Ov[i]);
    Ov[i]->Show = TRUE;
    if(placeonly)
    {
      
    }
    else
      *adp = *ad;
    adp->SetInterface(Cbp->Aif);
    adp->Recompute();
  }
}

int AstroChart::IfData(const AstroData* ad, bool placeonly) const
{
  int i;
  for(i = 0; i < NBDATA; i++)
    if(Ov[i] && (placeonly ? (Ov[i]->PlaceIdx == ad->PlaceIdx) : (Ov[i]->Idx == ad->Idx)))
      return i;
  return -1;
}

bool AstroChart::EditChartParams()
{
  int i;
  if(Cbp->Edit())
  {
    for(i = 0; i < NBDATA; i++)
      if(Ov[i])
      {
	*SC(ChartParams*, Ov[i]) = *SC(ChartParams*, Cbp);
	Ov[i]->Recompute();
      }
  }
  return TRUE;
}

int AstroChart::IfRestrict(const AstroRestrictions* ar) const
{
    int i;
    for(i = 0; i < NBDATA; i++)
     if(Ov[i] && ~*Ov[i] && (~*Ov[i])->Idx == ar->Idx)
       return i;
   return -1;
}

bool AstroChart::UpdateRestrict(AstroRestrictions *ar)
{
  int i;
  bool ret = FALSE;
  for(i = 0; i < NBDATA; i++)
    if(Ov[i] && ~*Ov[i] && (~*Ov[i])->Idx == ar->Idx)
    {
      SetRestrict(ar, i);
      ret = TRUE;
    }
  return ret;
}

bool AstroChart::UpdateData(AstroData *ad, bool placeonly)
{
  int i = IfData(ad, placeonly);
  if(i != -1)
  {
    SetData(ad, i);
    return TRUE;
  }
  return FALSE;
}

bool AstroChart::UpdateZif(Index idx, const char* zif)
{
  int i;
  bool ret = FALSE;
  for(i = 0; i < NBDATA; i++)
    if(Ov[i] && Ov[i]->CountryIdx == idx)
    {
      Ov[i]->SetTZ(zif);
      Ov[i]->Recompute();
      ret = TRUE;
    }
  return ret;
}

void AstroChart::SetRestrict(AstroRestrictions* ar, int i)
{
  if(i < 0 || i >= NBDATA) return;
  if(Ov[i])
  {
    Ov[i]->SetRestrict(ar, i);
    Cbp->Aif->SetRestrict(ar, i);
    Ov[i]->Recompute();
    Acb->IfRedrawChart = TRUE;
  }
  else
    AstroMessage(i18n("Can't set this restrictions set for an empty ring"));
}

int AstroChart::NbSingle()
{
  int i, s = 0;
  for(i = 0; i < NBDATA; i++)
    if(Ov[i] && Ov[i]->GetSubType() == Single)
      s++;
  return s;
}

bool AstroChart::Show(int i)
{
  int j, nb = 0;
  AstroObjs& ao = *Ov[i];
  if(ao.Show)
  {
    for(j = 0; j < NBDATA; j++)
      if(Ov[j] && Ov[j]->Show)
	nb++;
    if(nb == 1)
      return FALSE;
  }
  ao.Show = !ao.Show;
  Acb->IfRedrawChart = TRUE;
  return TRUE;
}

bool AstroChart::SetSubType(ChartSubType cst, int i)
{
  int j, s = 0, nbs = NbSingle();
  if(i < 0 || i >= NBDATA) return FALSE;
  if(!Ov[i])
    SetData(Adl->Get(0), i, TRUE);
  AstroObjs& ao = *Ov[i];
  //if(NbSingle() == 1 && Acb->Search(1, AstroChartBase::SingleOnly, FALSE) == i) return FALSE;
  switch(cst)
  {
    case NoSt:      break;
    case Show_Hide: return Show(i); break;
    case Single:    ao.SetSubChart((*Asl)[cst]); ao.Recompute(); break;
    case Transit:
    case Direction:
    case Progression:
      if(nbs == 0 || (nbs == 1 && cst != Single && cst != Show_Hide && ao.GetSubType() == Single))
	return FALSE;
      else
      {
	ao.SetSubChart((*Asl)[cst]);
	ao.Recompute();
      }
      break;
  }
  return TRUE;
}

void AstroChartList::UpdateData(AstroData* ad, bool placeonly)
{
  AstroChart* ac;
  for(ac = First(); ac; ac = Next())
  {
    if(ac->UpdateData(ad, placeonly))
	ac->Repaint();
  }
}

void AstroChartList::UpdateZif(Index idx, const char* zif)
{
  AstroChart* ac;
  for(ac = First(); ac; ac = Next())
  {
    if(ac->UpdateZif(idx, zif))
	ac->Repaint();
  }
}

void AstroChartList::UpdateRestrict(AstroRestrictions* ar)
{
  AstroChart* ac;
  for(ac = First(); ac; ac = Next())
  {
    if(ac->UpdateRestrict(ar))
	ac->Repaint();
  }
}

bool AstroChartList::CheckIdx(Index idx, bool country)
{
  AstroChart* ac;
  AstroObjs* ao;
  int i;
  for(ac = First(); ac; ac = Next())
  {
    for(i = 0; i < NBDATA; i++)
    {
      ao = (*ac)[i];
      if(ao && ((country ? ao->CountryIdx : ao->PlaceIdx) == idx))
	return TRUE;
    }
  }
  return FALSE;
}

void AstroChartList::ChangeIdx(Index oldidx, Index newidx)
{
  AstroChart* ac;
  AstroObjs* ao;
  int i;
  for(ac = First(); ac; ac = Next())
  {
    for(i = 0; i < NBDATA; i++)
    {
      ao = (*ac)[i];
      if(ao && ao->Idx == oldidx)
	ao->Idx = newidx;
    }
  }
}
