/***************************************************************************
                          astrochart.h  -  description
                             -------------------
    begin                : Mon Oct 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROCHART_H
#define ASTROCHART_H

#include "astrochartbase.h"
#include "astrodata.h"
#include "astrolist.h"

/**This class is the effective chart class.

  *@author Christophe Gros
  */

class AstroChart
{
public:

  friend class AstroChartList;
  friend class AstroObjs;

  /** Constructor
   @param owninterface if true this chart has its own computing process
  */
  AstroChart(bool owninterface = TRUE);

  /** Destructor */
  virtual ~AstroChart();
	
// Just returns a pointer to the wanted AstroObjs
  inline AstroObjs* operator[] (int i) const { return (Acb && i >= 0 && i < NBDATA) ? Ov[i] : 0; }
	
// The current chart type
  enum ChartType ct;
					
  /** This method displays the chart according to the Chart Type */
  virtual void Display();

  /** Sets the subtype of the ith ring */
  bool SetSubType(ChartSubType cst, int i);

  /** Sets the data of the ith AstroObjs */
  virtual void SetData(AstroData*, int i, bool placeonly = FALSE);

  /** Tests if ad is present in Ov. If true, return its index, otherwise -1 */
  int IfData(const AstroData* ad, bool placeonly = FALSE) const;

  /** Says if ad is present in the AstroObjs set */
  virtual bool UpdateData(AstroData* ad, bool placeonly = FALSE);

  /** update the zoneinfo file if CountryIdx = idx */
  bool UpdateZif(Index idx, const char* zif);

/** Tests if ar is present in Ov. If true, return its index, otherwise -1 */
  int IfRestrict(const AstroRestrictions* ar) const;

  /** Sets the restriction data for the ith AstroObjs */
  void SetRestrict(AstroRestrictions* ar, int i);

/** Says if ar is present in the AstroObjs set */
  bool UpdateRestrict(AstroRestrictions *ar);

  /** Edit the ChartParams and update all the  AstroObjs */
  bool EditChartParams();

  /** Returns computed chart's title */
  inline void GetTitle(AstroString& as) const { if(Acb) Acb->GetTitle(as); else as = ""; }

  AstroChartBase *Acb;

 protected:

  virtual void Repaint(bool b = FALSE) { if(b) Acb->IfRedrawChart = TRUE; }

  ChartBaseParams* Cbp;

 private:

  ObjsVector Ov;

  /** This method creates an acb according to ct */
 void CreateChart();

 /** Shows/Hides the ith ring of the chart. Returns FALSE if not successful */
 bool Show(int i);

 /** Gives the number of single rings */
 int NbSingle();

};

/** This class manages a linked list of AstroCharts */
class AstroChartList: public AstroList(AstroChart)
{
 public:

  /** Constructor */
  inline AstroChartList() { setAutoDelete(FALSE); }

  /** Changes the index for all the AstroObjs of all the AstroCharts
      if Index == oldidx
  */
  void ChangeIdx(Index oldidx, Index newidx);

  /** Modifies the data of all the AstroObjs of all the AstroCharts
      if index of ad == AstroObjs' inedx
      @param placeonly only modifies the place data
  */
  void UpdateData(AstroData* ad, bool placeonly = FALSE);

  /** Changes the AstroRestrictions pointer of all the AstroObjs */
  void UpdateRestrict(AstroRestrictions* ar);

  /** Changes the zoneinfo filename for all the AstroObjs with Index idx */
  void UpdateZif(Index idx, const char* zif);

  /** Tells if AstroObjs with index idx is present in the list
      @param country, AstroObjs with country index idx only */
  bool CheckIdx(Index idx, bool country = FALSE);
};
#endif
