/***************************************************************************
                          astrochartbase.cpp  -  description
                             -------------------
    begin                : Sat Oct 14 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrochartbase.h"

AstroChartBase::AstroChartBase(ChartBaseParams *cbp, ObjsVector* ob)
{
  IfRedrawChart = FALSE;
  Cbp = cbp;
  Ob = ob;
  DualChart = FALSE;
}

AstroChartBase::~AstroChartBase(){
}

int AstroChartBase::Count() const
{
  int i, nb = 0;
  const ObjsVector& ob = *Ob;
  for(i = 0; i < NBDATA; i++)
    if(ob[i] && ob[i]->Show)
      nb++;
  return nb;
}

int AstroChartBase::Search(int r, enum SearchType st, bool visible) const
{
  int i, j = 1;
  const ObjsVector& ob = *Ob;
  for(i = 0; i < NBDATA; i++)
    if(ob[i])
    {
      if(visible && !ob[i]->Show)
	continue;
      else
	switch(st)
	{
	  case SingleOnly:
	    if(ob[i]->GetSubType() == Single)
	    {
	      if(r == j)
		return i;
	      else
		j++;
	    }
	    break;
	  case NonSingle:
	    if(ob[i]->GetSubType() != Single)
	    {
	      if(r == j)
		return i;
	      else
		j++;
	    }
	    break;
	  case Any:
	    if(r == j)
	      return i;
	    else
	      j++;
	};
    }
  return -1;
}

bool AstroChartBase::operator == (int i) const
{
  const ObjsVector& ob = *Ob;
  if(i < 0 || i >= NBDATA) return FALSE;
  return ob[i] /*&& ob[i]->Show*/ ? TRUE : FALSE;
}
 
void AstroChartBase::PostSetData(int i)
{
  if(i > 0)
  {
    if(!DualChart) DualChart = TRUE;
    (*this)[i]->SetSubChart((*Asl)[Transit]);
  }
  IfRedrawChart = TRUE;
}

void AstroChartBase::GetTitle(AstroString &as) const
{
  int nb = Count(), f = First(), s = Search(2, Any, TRUE);
  const AstroObjs &ao = *(*this)[f], &ao1 = *(*this)[s];
  if(nb == 1)
    Sprintf(i18n("%a wheel chart for %a"), &as, &ao.GetSubChart().GetName(), &ao.Name);
  else
    Sprintf(i18n("%a wheel chart for %a and %a"), &as, &ao1.GetSubChart().GetName(), &ao.Name, &ao1.Name);
}
