/***************************************************************************
                          astrodata.cpp  -  description
                             -------------------
    begin                : Sat Sep 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrodata.h"
#include "astroresources.h"
#include "editastrodata.h"
#include <qtimer.h>
#include "astrosql.h"
#include <cstdlib>

AstroData::AstroData(AstroInterface* aif) : AstroDate(aif)
{
  Gender = Neutral;
  ChartType = 1;
  Idx = PlaceIdx = CountryIdx = 1;
  HMS_Format = FALSE;
  Latitude = Longitude = 0.0;
  Modified = FALSE;
}

AstroData::AstroData(const AstroDate& ad, Index countryidx, enum Gender gender = Neutral, unsigned int charttype, const AstroString& name) : AstroDate(ad)
{
  Gender = gender;
  ChartType = charttype;
  Idx = 1;
  HMS_Format = FALSE;
  Name = name;
  Modified = FALSE;
  DbPlaceFetch(countryidx); 
}

Index AstroData::DbSave(bool asdefault)
{
  char g;
  Index idx, saveidx;
  AstroString time, date, savename;
  AstroData* def;
  if(asdefault)
  {
  	def = new AstroData;
  	def->DbFetch(1);
  	savename = Name;
  	Day = def->Day;
  	Month = def->Month;
  	Year = def->Year;
  	Hm = def->Hm;
  	Name = i18n("Default Data");
  	saveidx = Idx;
  	Idx = 1;
  }
  bool update = Idx >= BOUND;
  date += Year; date += "-"; date += Month; date += "-"; date += Day;
  switch(Gender)
  {
    case Male:   g = 'M'; break;
    case Female: g = 'F'; break;
    default:     g = 'N';
  };
  AstroRequest arq;
  Get(time);
  if(!arq.Exec("SELECT Idx FROM Data WHERE Idx = %d", Idx) || arq.Number() == 0)
  {
    arq.Exec("INSERT INTO Data (Name, BirthDay, BirthTime, Gender, ChartType, Daylight, TimeZone, PlaceIdx) VALUES('%a', '%s', '%s', '%c', %d, %b, '%f', %d)", &Name, date.Data(), time.Data(), g, ChartType, GetDaylight(), GetOffset(), PlaceIdx);
    idx = arq.GetINSId();
    arq.Exec("SELECT Idx FROM Data WHERE Oid = %d", idx);
    Idx = arq.GetInt("Idx");
  }
  else
    arq.Exec("UPDATE Data SET Name = '%a', BirthDay = '%s', BirthTime = '%s', Gender = '%c', ChartType = %d, TimeZone = '%f', Daylight = %b, PlaceIdx = %d WHERE Idx = %d", &Name, date.Data(), time.Data(), g, ChartType, GetOffset(), GetDaylight(), PlaceIdx, Idx);
  arq.SetComment(Comment, Idx);
  Modified = FALSE;
 if(asdefault)
  {
  	delete def;
  	Name = savename;
  	Idx = saveidx;
  	Now();
  }
  return Idx;
}

/** Retrieves the chart data from the database
@param name The thing or person's name
@return true if found  */
bool AstroData::DbFetch(Index idx)
{
  char g;
  int h, m;
  bool daylight;
  double offset;
  AstroString as, *aptr;
  AstroStringList asl;
  AstroRequest arq;
  if(!arq.Exec("SELECT * FROM Data WHERE Idx = %d", idx) || arq.Number() == 0)
  {
    AstroMessage(i18n("Can't fetch data record !"));
    return FALSE;
  }
  Idx = arq.GetInt("Idx");
  Name = arq < "Name";
  as = arq < "BirthDay";
  Split(as, "-", &asl);
  aptr  = asl.First();
  Year  = atoi(aptr->Data());
  aptr  = asl.Next();
  Month = atoi(aptr->Data());
  aptr  = asl.Next();
  Day   = atoi(aptr->Data());
  as = arq < "BirthTime";
  Split(as, ":", &asl);
  aptr = asl.First();
  h = atoi(aptr->Data());
  aptr  = asl.Next();
  m = atoi(aptr->Data());
  Set(h, m);
  g = arq.GetChar("Gender");
  switch(g)
  {
    case 'M' : Gender = Male; break;
    case 'F' : Gender = Female; break;
    default: Gender = Neutral;
  };
  ChartType = arq.GetInt("ChartType");
  offset    = arq.GetDouble("TimeZone");
  daylight  = arq.GetBool("Daylight");
  SetOffDay(offset, daylight);
  Comment = arq < "Comment";
  PlaceIdx = arq.GetInt("PlaceIdx");
  if(!DbPlaceFetch(PlaceIdx))
    return FALSE;
  return Recompute();
}

bool AstroData::DbPlaceFetch(Index idx)
{
  AstroRequest arq;
  if(!arq.Exec("SELECT * FROM Place WHERE Idx = %d", idx) || arq.Number() == 0)
  {
    AstroMessage(i18n("Can't fetch place record !"));
    return FALSE;
  }
  PlaceName = arq < "Name";
  Latitude = arq.GetDouble("Latitude");
  Longitude = arq.GetDouble("Longitude");
  CountryIdx = arq.GetInt("CountryIdx");
  arq.Clear();
  if(!arq.Exec("SELECT TzFile FROM Country WHERE Idx = %d", CountryIdx) || arq.Number() == 0)
  {
    AstroMessage(i18n("Can't fetch database record !"));
    return FALSE;
  }
  SetTZ(arq.Get(arq.GetIndex("TzFile")));
  return TRUE;
}

bool AstroData::Edit()
{
  EditAstroData ead(*this);
  ead.show();
  return ead.Return;
}

bool AstroData::FetchZoneInfoFile(char *c, Index countryidx)
{
  AstroRequest arq;
  if(!arq.Exec("SELECT TzFile FROM Place WHERE Idx = %d", countryidx) || arq.Number() == 0)
  {
    AstroMessage(i18n("Can't fetch place record !"));
    return FALSE;
  }
  strncpy(c, arq.Get(arq.GetIndex("TZFile")), BUFSIZE);
  return TRUE;
}

Index AstroData::FetchCountryIdx(const char *c)
{
  AstroRequest arq;
  if(!arq.Exec("SELECT Idx FROM Country WHERE Name = '%s'", c) || arq.Number() == 0)
  {
    AstroMessage(i18n("Can't fetch country index !"));
    return 0;
  }
  return arq.GetInt("Idx");
}

AstroDataList::AstroDataList()
{
  Max = BOUND;
  AstroNow* an = new AstroNow();
  CHECK_PTR(an);
  Append(an);
  Now = an;
}

void AstroDataList::UpdatePlace(const AstroData& a)
{
  AstroData* ad;
  for(ad = First(); ad; ad = Next())
    if(ad->PlaceIdx == a.PlaceIdx)
    {
      ad->Latitude = a.Latitude;
      ad->Longitude = a.Longitude;
      ad->HMS_Format = a.HMS_Format;
      ad->CountryIdx = a.CountryIdx;
      ad->Recompute();
    }
}

void AstroDataList::AddData(AstroData* ad)
{
	if(ad->Idx == 1)
  		ad->Idx = ++Max;
  Append(ad);
}

AstroData* AstroDataList::Get(Index idx)
{
  AstroData* ad;
  for(ad = First(); ad; ad = Next())
    if(ad->Idx == idx)
      return ad;
  return 0;
}

bool AstroDataList::CheckIdx(Index idx, bool country)
{
   AstroData* ad;
  for(ad = First(); ad; ad = Next())
    if(country ? ad->CountryIdx : ad->PlaceIdx == idx)
      return TRUE;
  return FALSE;
}

bool AstroDataList::RemoveFromDb(Index idx)
{
  AstroRequest arq;
  Remove(Get(idx));
  if(idx >= BOUND)
    return TRUE;
  if(!arq.Exec("DELETE FROM Data WHERE Idx = %d", idx))
      AstroMessage(i18n("Problem in data deletion"));
  return TRUE;
}

AstroNow::AstroNow()
{
  DbFetch(1);
  Name = i18n("Now");
  Idx = 0;
  Gender = Neutral;
  ChartType = 0;
  Corrected = TRUE;
  Now();
#ifdef KDE
  Timer = new QTimer;
  connect(Timer, SIGNAL(timeout()), this, SLOT(Done()));
  Timer->start(60 * 1000);
#endif
}

void AstroNow::Done()
{
  Now();
}

AstroNow::~AstroNow()
{
#ifdef KDE
  Timer->stop();
  delete Timer;
#endif
}
