/***************************************************************************
                          astrodata.h  -  description
                             -------------------
    begin                : Sat Sep 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTRODATA_H
#define ASTRODATA_H

#include "astrodate.h"
#include "astrohm.h"
#include "astrostring.h"
#include "astrolist.h"
#include <qobject.h>

enum Gender { Male, Female, Neutral };
class QTimer;

/**This class is an extention of AstroDate, 
an contains explicit time, place and name data.
  *@author Christophe Gros
  */

class AstroData : public AstroDate
{

 public:
 
  /** Constructor */
  AstroData(AstroInterface* aif = Asf);

  /** Constructor */
  AstroData(const AstroDate& ad, Index countryidx = 1, enum Gender gender = Neutral, unsigned int charttype = 1, const AstroString& name = "");

  /** Destructor */
  virtual ~AstroData() {}

  /** The data and place names, + data comment */
  AstroString Name, PlaceName, Comment;

  /** If the Chart is for a male being, a female being or for a neutral thing or event, situation, etc ... */ 
  enum Gender Gender;

  /** Type of the chart: 0 = now, 1 = birth chart, 2 = death chart, etc...*/
  unsigned int ChartType;
 
  /** The current data, place and country indexes */
  Index Idx, PlaceIdx, CountryIdx;

  /** if true, expressses the longitude in H:M:S */
  bool HMS_Format;

  /** if the data has been modified without having been saved */
  bool Modified;

  /** Where the latitude and longitude are stored */
  AstroHM Latitude, Longitude;

      /** Retrieves the chart data + place + country from the database
   * @param idx the data database index 
   * @return true if found
   */
  bool DbFetch(Index idx);

 /** Access the database for retrieving the data place from the given name
* @param idx the unique idx of the database record containing the wanted 
* data place
* @return true if found
 */
  bool DbPlaceFetch(Index idx); 

  /** Opens a dialog box for the data edition */
  bool Edit();

  /** Saves into the database, automatic selection of INSERT or UPDATE
   @param asdefault if TRUE, save into Default Data
   @return the new or current row index
  */
  Index DbSave(bool asdefault = FALSE);

  /** Produces the zone info file for the given country index
      @param c the zone info filename buffer
      @param countryidx the country index
  */
  static bool FetchZoneInfoFile(char *c, Index countryidx);

  /** Returns the index of the given country name */
  static Index FetchCountryIdx(const char *c);

};

/** This class manages a linked list of AstroData */
class AstroDataList: public AstroList(AstroData)
{
 public:

  /** Constructor */
  AstroDataList();

  /** Adds am AstroData pointer */
  void AddData(AstroData*);

  /** Returns the pointer of an AstroData with a given Index */
  AstroData* Get(Index);

  /** Modifies all the place data whose the place index is the one from the
      given AstroData */
  void UpdatePlace(const AstroData&);

  /** Tells whether the AstroData of index idx is present into the list
      @param Country country or place
  */
  bool CheckIdx(Index idx, bool Country = FALSE);

  /** Removes from database the data with index idx */
  bool RemoveFromDb(Index idx);
  
 private:

  Index Max;
};

/** The specific AstroData giving the current time */
class AstroNow : public QObject, public AstroData
{
  Q_OBJECT

 public:

  AstroNow();
  ~AstroNow();

 private:

  QTimer* Timer;

private slots:

  void Done();
};
#endif
