/***************************************************************************
                          astrodefs.h  -  description
                             -------------------
    begin                : Sat Sep 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTRODEFS_H
#define ASTRODEFS_H
#include "../config.h"
#include "astrostring.h"

extern "C"
{
# include <math.h>
};

#ifndef PI
#define PI 3.14159265358979323846
#endif

class AstroSubChartList;
class AstroSigns;
class AstroResources;
class AstroColorList;
class AstroAspectsTable;
class AstroInterface;
class AstroRestrictionsList;
class AstroDataList;
class AstroChartList;
class AstroIconsList;
class AstroData;
class AstroRestrictions;
class AstroMainWindow;
class AstroSQL;

#ifndef TRUE
# define TRUE 1
#endif
#ifndef FALSE
# define FALSE 0
#endif

#ifndef FAR
# define FAR
#endif
#ifndef PASCAL_CONV
# define PASCAL_CONV
#endif

// For the Swiss Ephemeris 1.6 and above
#define SWE1_6

// C++ type conversion
#define SC(type, expr) static_cast<type>(expr)

#ifndef _KLOCALE_H
class QString;
QString i18n(const char *text);
#endif

// The translation standard macro
#ifndef KDE
# define i18n(c) (c)
#else
# define i18n(c) (AstroString)i18n(c)
#endif

#ifndef KDE
#define CHECK_PTR(ptr)
#endif

// The number of simultaneous data in a chart
#define NBDATA 4
// The number of stars
#define NBSTARS 74
// The out of scope or illegal zodiacal longitude value
#define NOVALUE 400.0

// Objects index
enum Objs
{
  Sun, Moon, Mercury, Venus, Mars, Jupiter, Saturn, Uranus,
  Neptune, Pluto, North_Node, South_Node, Black_Moon, The_Earth,
  Chiron, Pholus, Ceres, Pallas, Juno, Vesta, Fortune, Vertex, East_Point,
  Cupido = 40, Hades, Zeus, Kronos, Apollon, Admetos, Vulkanus, Poseidon,
  Ascendant = 100, MC, FC, Descendant,
  House_1, House_2, House_3, House_4, House_5, House_6, House_7, House_8,
  House_9, House_10, House_11, House_12,
  First_Star, Last_Star = (First_Star + NBSTARS -1),
  Undefined_Obj
};

#define Last_Planet  East_Point
#define First_Exotic Cupido
#define Last_Exotic  Poseidon
#define UO           Undefined_Obj
#define Last_Major   Sextile
#define First_Minor  Inconjunct

// Signs index
enum Signs
{
  Aries, Taurus, Gemini, Cancer, Leo, Virgo, Libra, Scorpio,
  Sagittarius, Capricorn, Aquarius, Pisces, Undefined_Sign
};

// Elements index
enum Elements { Fire, Air, Water, Earth, Undefined_Element };

// Triplicity index
enum Triplicity { Cardinal, Fixed, Mutable };

// Houses index
enum House
{
  Placidus, Koch, Porphyrius, Regiomontanus, Campanus, Equal, Vehlow, Meridian,
  Azimuthal, Topocentric, Alcabitus
};

// Sidereal charts systems index
enum Sidereal { Tropical = -1, Fagan_Bradley,  Lahiri, Deluce, Raman, Ushashashi,
 Krishnamurti, Djwhal_Khul, Yukteshwar, Jn_Bhasin, Babyl_Kugler_1,
 Babyl_Kugler_2, Babyl_Kugler_3, Babyl_Huber, Babyl_Etpsc, Aldebaran_15Tau,
 Hipparchos, Sassanian, Galcent_0Sag, J2000, J1900, B_1950 };

// Aspects index

enum Aspects { Conjunction, Opposition, Trine, Square, Sextile, Inconjunct,
 Semisextile, Semisquare, Sesquiquadrate, Quintile, Biquintile, Semiquintile,
 Septile, Novile, Binovile, Biseptile, Triseptile, Quatronovile, Unknown_Aspect };

// If an aspect is harmonic (trine), dissonant (square) or ambivalent (conjunction, oppos)
enum AspectKind { Harmonic, Dissonant, Ambivalent };

// Type of chart wheel, matrix, lists, etc...
enum ChartType { RawText, StdWheels };

//Subtype of chart. If chart have 2 independant rings, transiting or progressing charts, etc... 
enum ChartSubType { NoSt, Show_Hide, Single, Transit, Direction, Progression };

// If the chart is an ASCII text, a drawing or an HTML page 
enum ChartMode { TextMode, GraphicMode, HTMLMode };

// Some default values

#define DEF_DAY         1
#define DEF_MONTH       1
#define DEF_YEAR        2000
#define DEF_TIME        12.00
#define DEF_LATITUDE    0.0
#define DEF_LONGITUDE   0.0
#define DEF_TIMEOFFSET  1.0
#define DEF_NAME        "no name"
#define DEF_DYNAMICRESIZE FALSE
#define DEF_LEFTASCENDANT FALSE
#define DEF_SIDEBARS      TRUE
#define DEF_TRUENODE    FALSE
#define DEF_HOUSESYS    Placidus
#define DEF_TROPICAL_SIDEREAL Tropical
#define DEF_RESTRICTIONS  { \
 { /* Sun */ FALSE, 1.0, 30 }, \
 { /* Moon */ FALSE, 1.0, 25 }, \
 { /* Mercury */ FALSE, 0.0, 10 }, \
 { /* Venus */ FALSE, 0.0, 10 }, \
 { /* Mars */ FALSE, 0.0, 10 }, \
 { /* Jupiter */ FALSE, 0.0, 10 }, \
 { /* Saturn */ FALSE, 0.0, 10 }, \
 { /* Uranus */ FALSE, 0.0, 10 }, \
 { /* Neptune */ FALSE, 0.0, 10 }, \
 { /* Pluto */ FALSE, 0.0, 10 }, \
 { /* North Node */ FALSE, 0.0, 5 }, \
 { /* South Node */ FALSE, 0.0, 5 }, \
 { /* Black Moon */ FALSE, 0.0, 5 }, \
 { /* Earth */ TRUE, 0.0, 20 }, \
 { /* Chiron */ TRUE, 0.0, 5 }, \
 { /* Pholus */ TRUE, 0.0, 5 }, \
 { /* Ceres */ TRUE, 0.0, 5 }, \
 { /* Pallas */ TRUE, 0.0, 5 }, \
 { /* Juno */ TRUE, 0.0, 5 }, \
 { /* Vesta */ TRUE, 0.0, 5 }, \
 { /* Fortune */ FALSE, 0.0, 5 }, \
 { /* Vertex */ TRUE, 0.0, 5 }, \
 { /* East Point */ TRUE, 0.0, 5 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { /* Cupido */ FALSE, 0.0, 2 }, \
 { /* Hades */ FALSE, 0.0, 2 }, \
 { /* Zeus */ FALSE, 0.0, 2 }, \
 { /* Kronos */ FALSE, 0.0, 2 }, \
 { /* Apollon */ FALSE, 0.0, 2 }, \
 { /* Admetos */ FALSE, 0.0, 2 }, \
 { /* Vulkanus */ FALSE, 0.0, 2 }, \
 { /* Poseidon */ FALSE, 0.0, 2 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { TRUE, 0.0, 0 }, \
 { /* Ascendant */ FALSE, 0.0, 20 }, \
 { /* Midheaven */ FALSE, 0.0, 20 }, \
 { /* Nadir */ FALSE, 0.0, 10 }, \
 { /* Descendant */ FALSE, 0.0, 10 }, \
 { /* 1st House */ TRUE, 0.0, 20 }, \
 { /* 2nd House */ TRUE, 0.0, 5 }, \
 { /* 3rd House */ TRUE, 0.0, 5 }, \
 { /* 4th House */ TRUE, 0.0, 10 }, \
 { /* 5th House */ TRUE, 0.0, 5 }, \
 { /* 6th House */ TRUE, 0.0, 5 }, \
 { /* 7th House */ TRUE, 0.0, 10 }, \
 { /* 8th House */ TRUE, 0.0, 5 }, \
 { /* 9th House */ TRUE, 0.0, 5 }, \
 { /* 10th House */ TRUE, 0.0, 20 }, \
 { /* 11th House */ TRUE, 0.0, 5 }, \
 { /* 12th House */ TRUE, 0.0, 5 }, \
 { /* Achernar */ FALSE, 0.0, 0 }, \
 { /* Polaris */ FALSE, 0.0, 0 }, \
 { /* ZetaRetic.  */ FALSE, 0.0, 0 }, \
 { /* Alcyone */ FALSE, 0.0, 0 }, \
 { /* Aldebaran */ FALSE, 0.0, 0 }, \
 { /* Capella */ FALSE, 0.0, 0 }, \
 { /* Rigel */ FALSE, 0.0, 0 }, \
 { /* Bellatrix */ FALSE, 0.0, 0 }, \
 { /* Elnath */ FALSE, 0.0, 0 }, \
 { /* Alnilam */ FALSE, 0.0, 0 }, \
 { /* Betelgeuse */ FALSE, 0.0, 0 }, \
 { /* Menkalinan */ FALSE, 0.0, 0 }, \
 { /* Mirzam */ FALSE, 0.0, 0 }, \
 { /* Canopus */ FALSE, 0.0, 0 }, \
 { /* Alhena */ FALSE, 0.0, 0 }, \
 { /* Sirius */ FALSE, 0.0, 0 }, \
 { /* Adhara */ FALSE, 0.0, 0 }, \
 { /* Wezen */ FALSE, 0.0, 0 }, \
 { /* Castor */ FALSE, 0.0, 0 }, \
 { /* Procyon */ FALSE, 0.0, 0 }, \
 { /* Pollux */ FALSE, 0.0, 0 }, \
 { /* Suhail */ FALSE, 0.0, 0 }, \
 { /* Avior */ FALSE, 0.0, 0 }, \
 { /* Miaplacidus */ FALSE, 0.0, 0 }, \
 { /* Alphard */ FALSE, 0.0, 0 }, \
 { /* Regulus */ FALSE, 0.0, 0 }, \
 { /* Dubhe */ FALSE, 0.0, 0 }, \
 { /* Acrux */ FALSE, 0.0, 0 }, \
 { /* Gacrux */ FALSE, 0.0, 0 }, \
 { /* Mimosa */ FALSE, 0.0, 0 }, \
 { /* Alioth */ FALSE, 0.0, 0 }, \
 { /* Spica */ FALSE, 0.0, 0 }, \
 { /* Alkaid */ FALSE, 0.0, 0 }, \
 { /* Agena */ FALSE, 0.0, 0 }, \
 { /* Arcturus */ FALSE, 0.0, 0 }, \
 { /* Toliman */ FALSE, 0.0, 0 }, \
 { /* Antares */ FALSE, 0.0, 0 }, \
 { /* Shaula */ FALSE, 0.0, 0 }, \
 { /* Sargas */ FALSE, 0.0, 0 }, \
 { /* Kaus Australis */ FALSE, 0.0, 0 }, \
 { /* Vega */ FALSE, 0.0, 0 }, \
 { /* Altair */ FALSE, 0.0, 0 }, \
 { /* Peacock */ FALSE, 0.0, 0 }, \
 { /* Deneb */ FALSE, 0.0, 0 }, \
 { /* Alnair */ FALSE, 0.0, 0 }, \
 { /* Fomalhaut */ FALSE, 0.0, 0 }, \
 { /* Andromeda */ FALSE, 0.0, 0 }, \
 { /* Alpheratz */ FALSE, 0.0, 0 }, \
 { /* Algenib */ FALSE, 0.0, 0 }, \
 { /* Schedar */ FALSE, 0.0, 0 }, \
 { /* Mirach */ FALSE, 0.0, 0 }, \
 { /* Alrischa */ FALSE, 0.0, 0 }, \
 { /* Almac */ FALSE, 0.0, 0 }, \
 { /* Algol */ FALSE, 0.0, 0 }, \
 { /* Mintaka */ FALSE, 0.0, 0 }, \
 { /* Wasat */ FALSE, 0.0, 0 }, \
 { /* Acubens */ FALSE, 0.0, 0 }, \
 { /* Merak */ FALSE, 0.0, 0 }, \
 { /* Vindemiatrix  */ FALSE, 0.0, 0 }, \
 { /* Mizar */ FALSE, 0.0, 0 }, \
 { /* Kochab */ FALSE, 0.0, 0 }, \
 { /* Zubenelgenubi */ FALSE, 0.0, 0 }, \
 { /* Zubeneshamali */ FALSE, 0.0, 0 }, \
 { /* Alphecca */ FALSE, 0.0, 0 }, \
 { /* Unukalhai */ FALSE, 0.0, 0 }, \
 { /* Rasalhague */ FALSE, 0.0, 0 }, \
 { /* Albireo */ FALSE, 0.0, 0 }, \
 { /* Alderamin */ FALSE, 0.0, 0 }, \
 { /* Nashira */ FALSE, 0.0, 0 }, \
 { /* Skat */ FALSE, 0.0, 0 }, \
 { /* Scheat */ FALSE, 0.0, 0 }, \
 { /* Markab */ FALSE, 0.0, 0 }, \
 { /* Gal.Center */ FALSE, 0.0, 0 }, \
 { /* Apex */ FALSE, 0.0, 0 }, \
 } 
#define DEF_HOUSES_COMP TRUE
#define DEF_EXOTIC_COMP FALSE
#define DEF_STARS_COMP  FALSE
#define DEF_MINOR_COMP  FALSE
#define DEF_NOW_CHART   TRUE
#define DEF_RULERS      {\
  Mars, UO, Venus, UO, Sun, Saturn,\
  Venus, UO, Pluto, Mars, Moon, Uranus,\
  Mercury, UO, Jupiter, UO, UO, Neptune,\
  Moon, UO, Saturn, UO, Jupiter, Mars,\
  Sun, UO, Uranus, Saturn, Neptune, Mercury,\
  Mercury, Ceres, Neptune, Jupiter, Pluto, Venus,\
  Venus, UO, Mars, UO, Saturn, Sun,\
  Pluto, Mars, Venus, UO, Uranus, Moon,\
  Jupiter, Neptune, Mercury, UO, UO, UO,\
  Saturn, Uranus, Moon, UO, Mars, Jupiter,\
  Uranus, Saturn, Sun, UO, Mercury, UO,\
  Neptune, Jupiter, Pallas, Mercury, Venus, Pluto\
}

#define DEF_GRAPHIC_WIDTH  600
#define DEF_GRAPHIC_HEIGHT 450
#define DEF_COLORS      {\
   0,   0,   0,\
 176,  48,  96,\
   0, 100,   0,\
 255,  65,   0,\
   0,   0, 139,\
 139,   0,   0,\
   0, 139, 139,\
 211, 211, 211,\
   0, 100,   0,\
 255,   0,   0,\
   0, 255,   0,\
 255, 255,   0,\
   0,   0, 255,\
 255,   0, 255,\
   0, 255, 255,\
 255, 255, 255\
}

#define DEF_FIRE_COLOR    9
#define DEF_AIR_COLOR    14
#define DEF_WATER_COLOR   4
#define DEF_EARTH_COLOR   1
#define DEF_OTHER_COLOR   8
#define DEF_ASTERO_COLOR 13
#define DEF_EXOTIC_COLOR  1
#define DEF_ASPECTS_CONST {\
/* Angle, Kind, Harmonic */ \
{ /*Conjunction */   0.0, Ambivalent, 1 },\
{ /*Opposition */  180.0, Ambivalent,12 },\
{ /*Trine */       120.0, Harmonic,   6 },\
{ /*Square */       90.0, Dissonant,  4 },\
{ /*Sextile */      60.0, Harmonic,   6 },\
{ /*Inconjunct */  150.0, Ambivalent, 0 },\
{ /*Semisextile */  30.0, Harmonic,   6 },\
{ /*Semisquare */   45.0, Dissonant,  4 },\
{ /*Sesquiquadrat*/135.0, Dissonant,  4 },\
{ /*Quintile */     72.0, Harmonic,   5 },\
{ /*Biquintile */  144.0, Harmonic,   5 },\
{ /*Semiquintile */ 36.0, Harmonic,   5 },\
{ /*Septile */ 360.0/7.0, Harmonic,   7 },\
{ /*Novile */       40.0, Harmonic,   9 },\
{ /*Binovile */     80.0, Harmonic,   9 },\
{ /* Biseptil*/720.0/7.0, Harmonic,   7 },\
{ /*Trisepti*/1080.0/7.0, Harmonic,   7 },\
{ /*Quatronovile */160.0, Harmonic,   9 },\
}

#define DEF_ASPECTS_RESTRICTIONS {\
/* Rest, Orb, Color, Influence */ \
 { /* Conjunction */ FALSE, 10.0, 11, 30 },\
 { /*Opposition */   FALSE,  9.0, 12, 20 },\
 { /* Trine */       FALSE,  7.0, 10, 15 },\
 { /* Square */      FALSE,  7.0,  9, 15 },\
 { /* Sextile */     FALSE,  6.0, 13, 10 },\
 { /*Inconjunct */   FALSE,  4.0,  7,  5 },\
 { /*Semisextile */  FALSE,  3.0,  6,  5 },\
 { /*Semisquare */   FALSE,  3.0,  4,  5 },\
 { /*Sesquiquadrate*/FALSE,  3.0,  1,  5 },\
 { /*Quintile */     FALSE,  3.0, 14,  4 },\
 { /*Biquintile */   FALSE,  2.0, 14,  2 },\
 { /*Semiquintile */ FALSE,  2.0, 14,  2 },\
 { /*Septile */      FALSE,  3.0, 13,  4 },\
 { /*Novile */       FALSE,  3.0,  9,  4 },\
 { /*Binovile */     FALSE,  2.0,  9,  2 },\
 { /* Biseptile*/    FALSE,  2.0, 13,  2 },\
 { /*Triseptile */   FALSE,  2.0, 13,  2 },\
 { /*Quatronovile */ FALSE,  2.0,  9,  2 },\
}

// Standard path to the zone info files
#define ZPATH "/usr/share/zoneinfo/"
#define STDZFILE "GMT"
#define SWEPATH "../"

// Font names
#define ASTROFONT "astrolog"
#define TEXTFONT "Times"

// Print factors
#define PRINTZOOMFACT 5
#define BOLDLINE 4

// The characters in ASTROFONT corresponding to the signs and planets glyphs
#define SIGNCHARS "ABCDEFGHIJKL"
#define PLACHARS "QRSTUVWXYZjie [ \\]^_adh                          "
#define HOUSESCODES "PKORCAVXHTB"

// The julian day value of 1 minute;
#define ONEMINUTE  0.000694444444
// Number of minutes in a day
#define MINSINDAY   ((23 * 60) + 56)
// Number of hours in a year
#define HOURSINYEAR ((365 * 24) + 6)

// Returns an hour/minute value as double
#define MILLIROUND 0.005
#define RHM(h, m)((double)(h) + ((double)(m) / 100.0))

#define HMR(r, h, m) { double f = fabs(r), x = floor(f); h = (r) < 0.0 ? \
-(int)x : (int)x; m = (int)((f - x + MILLIROUND) * 100.0); }

#define ROUND 0.5
#define FRACT(r) ((r) - floor(r))
#define SIGN(r) ((r) < 0.0 ? -1 : ((r) > 0.0 ? 1 : 0))

// Degrees -> radians and radians -> degrees tranformation
#define DEGRAD(x) (TWOPI * (x) / 360.0)
#define RADDEG(x) (360.0 * (x) / TWOPI)

// The year and month of the gregorian calendar introduction
#define yeaJ2G 1582
#define monJ2G   10
// The julian day of the julian->gregorian calendar shift
#define J2G 2299161.000000

// database index type
typedef unsigned int Index;

// Default sizes
#define ASTRORINGSDEFSIZE 400
#define BOUND 2147483646

// Various extern objects

extern AstroSubChartList* Asl;
extern AstroSigns  *Asg;
extern AstroResources *Asr;
extern AstroColorList *Acl;
extern AstroAspectsTable *Aat;
extern AstroInterface* Asf;
extern AstroIconsList* Ail;
extern AstroRestrictionsList* Arl;
extern AstroDataList* Adl;
extern AstroChartList* Ahl;
extern AstroData* Now;
extern AstroRestrictions* Drs;
extern AstroMainWindow* Amw;
extern AstroSQL* Asq;

#define COL (*Acl)

#endif





