/***************************************************************************
                          astrographics.cpp  -  description
                             -------------------
    begin                : Sat Dec 30 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrographics.h"
#include "astroutils.h"
#include "astrohm.h"

#ifdef KDE
#include "qfont.h"
#include "qpen.h"
#include "qpainter.h"
#endif

AstroPixmap::AstroPixmap(const char *c, const char* path)
{
#ifdef KDE
  QString qp(path);
  load(qp + c);
#endif
}

AstroGraphics::AstroGraphics(AstroGraphics* parent = 0, int x, int y, int w, int h)
{
  charbuf[1] = 0;
#ifdef KDE
  if(parent)
  {
    Qp    = parent->Qp;
    Pen   = parent->Pen;
    astro = parent->astro;
    text  = parent->text;
    Size  = parent->Size;
    Base  = parent->Base;
  }
  Width  = w;
  Height = h;
  PosX   = x;
  PosY   = y;
  X =  Y = 0;
#endif
}

void AstroGraphics::Move(int x, int y)
{
#ifdef KDE
  X = x + PosX;
  Y = y + PosY;
  Qp->moveTo(X, Y);  
#endif
}

void AstroGraphics::Line(int x, int y)
{
#ifdef KDE
  X = x + PosX;
  Y = y + PosY;
  Qp->lineTo(X, Y);
#endif
}

void AstroGraphics::Dot()
{
  const int r = Size / 200, r2 = r * 2;
#ifdef KDE
  const QBrush b(Qt::SolidPattern), &bref = Qp->brush();
  Qp->setBrush(b);
  Qp->drawEllipse(X - r, Y - r, r2, r2);
  Qp->setBrush(bref);
#endif
}

void AstroGraphics::Circle(int x, int y, int r)
{
#ifdef KDE
    const int r2 = r * 2;
    Qp->drawEllipse(x - r + PosX, y - r + PosY, r2, r2); 
#endif
}

void AstroGraphics::Color(const AstroColor& ac)
{
#ifdef KDE
  Pen->setColor(ac);
  Qp->setPen(*Pen);
#endif
}

void AstroGraphics::Background(const AstroColor& ac)
{
#ifdef KDE
  Qp->setBackgroundColor(ac);
#endif
}

void AstroGraphics::Dash(int d)
{
#ifdef KDE
  Qt::PenStyle ps;
  switch(d)
  {
    case 0: ps = Qt::SolidLine; break;
    case 1: ps = Qt::DashLine;  break;
    case 2: ps = Qt::DashDotLine; break;
    case 3: ps = Qt::DotLine; break;
    default: ps = Qt::SolidLine;
  };
  Pen->setStyle(ps);
  Qp->setPen(*Pen);
#endif
}

void AstroGraphics::FontSize(enum AstroGraphics::SizeChars sc, bool isastro = FALSE)
{
  int x;
  switch(sc)
  {
    case Very_Tiny : x = (Size * 7) / 10; break;
    case Tiny   : x = (Size * 9) / 10; break;
    case Medium : x = (Size * 14) / 10; break;
    case Large  : x = (Size * 20) / 10; break;
    case Huge   : x = (Size * 28) / 10; break;
  }
#ifdef KDE
  if(isastro)
    astro->setPointSize(x);
  else
    text->setPointSize(x);
#endif
}

void AstroGraphics::Text(const AstroString& as, bool movex)
{
#ifdef KDE
  Qp->setFont(*text);
  Qp->drawText(X, Y, as);
#endif
  if(movex)
  {
    int w, h;
    TextMetrics(as, &w, &h);
    Move(X - PosX + w, Y - PosY);
  }
}

void AstroGraphics::JustifText(const AstroString& as, enum Justif j, int len)
{
  int w, h, i1, i2;
  TextMetrics(as, &w, &h);
  i1 = (len - w) / 2;
  i2 = len - w - i1;
  switch(j)
  {
  case Left:   Text(as); Move(X - PosX + len, Y - PosY); break;
  case Right:  Move(X - PosX + (len - w), Y - PosY); Text(as, TRUE); break;
  case Center: Move(X - PosX + i1, Y - PosY); Text(as); Move(X - PosX + i2 + w, Y - PosY);
  };
}

void AstroGraphics::Sign(enum Signs s)
{
  Glyph(Asg->GetChar(s));
}

void AstroGraphics::Glyph(char c)
{
  charbuf[0] = c;
#ifdef KDE
  Qp->setFont(*astro);
  Qp->drawText(X, Y, charbuf);
#endif
}

void AstroGraphics::JustifGlyph(char c, enum Justif justif, int len)
{
  int w, h, i1, i2;
  GlyphMetrics(c, &w, &h);
  i1 = (len - w) / 2;
  i2 = len - w - i1;
  switch(justif)
  {
  case Left:   Glyph(c); Move(X - PosX + len, Y - PosY); break;
  case Right:  Move(X - PosX + (len - w), Y - PosY); Glyph(c); Move(X - PosX + w, Y - PosY); break;
  case Center: Move(X - PosX + i1, Y - PosY); Glyph(c); Move(X - PosX + i2 + w, Y - PosY);
  };
}

void AstroGraphics::TextMetrics(const AstroString& as, int* width, int* height)
{
#ifdef KDE
  Qp->setFont(*text);
  *width  = Qp->fontMetrics().width(as);
  *height = Qp->fontMetrics().height();
#endif
}

void AstroGraphics::GlyphMetrics(char c, int* width, int* height)
{
#ifdef KDE
  Qp->setFont(*astro);
  *width  = Qp->fontMetrics().width(c);
  *height = Qp->fontMetrics().height();
#endif
}

void AstroGraphics::DegSignMin(double d, bool name, bool retro)
{
  int i, w, w1, h;
  double x, flx;
  char buf[3];
  AstroString as = "000";
  TextMetrics(as, &w, &h);
  i = (int)d / 30;
  x = d - (double)(i * 30);
  flx = floor(x);
  as = (int)flx;
  const enum Signs s = (enum Signs)(Aries + i);
  JustifText(as, Right, w);
  GlyphMetrics(Asg->GetChar(Scorpio), &w1, &h);
  JustifGlyph(Asg->GetChar(s), Center, w1);
  sprintf(buf, "%02d", (int)((x - flx) * 100.0));
  as = buf;
  w = (w / 3) * 2;
  JustifText(as, Right, w);
  JustifText(retro ? " R" : "  ", Left, w);
}

void AstroGraphics::Hms(double d, const AstroString& sep, bool second = FALSE)
{
  int h, m, s, w;
  char buf[3];
  AstroString as;
  TextMetrics("0", &w, &h);
  HMR(d, h, m);
  as = h;
  JustifText(as, Right, 3 * w);
  Text(sep, FALSE);
  Move(X - PosX + w, Y - PosY);
  sprintf(buf, "%02d", m);
  JustifText(buf, Left, 2 * w);
  /*if(second)
  {
    Text(":", FALSE);
    sprintf(buf, "%02d", s);
    JustifText(buf, Left, w);
    }*/
}

void AstroGraphics::TextAngle(AstroString& a, double angle)
{
  int i, w, h;
  if(angle >= PI)
    angle -= PI;
  if(angle <= (PI / 4.0))
  {
    for(i = 0; i < a.Len(); i++)
    {
      const AstroString& b = a[i];
      TextMetrics(b, &w, &h);
      Text(b);
      Move(X - PosX + w, Y - PosY - (int)(sin(angle) * (double)w));
    }
  } else if(angle <= (PI / 2.0))
  {
    for(i = 0; i < a.Len(); i++)
    {
      const AstroString& b = a[i];
      TextMetrics(b, &w, &h);
      Text(b);
      h /= 2;
      Move(X - PosX + (int)(cos(angle) * (double)h), Y - PosY - h);
    }
  } else if(angle <= (3.0 * PI / 4.0))
  {
    for(i = a.Len() -1; i >= 0 ; i--)
    {
      const AstroString& b = a[i];
      TextMetrics(b, &w, &h);
      Text(b);
      h /= 2;
      Move(X - PosX + (int)(cos(angle) * (double)h), Y - PosY - h);
    }
  } else
  {
    for(i = a.Len() - 1; i >=  0; i--)
    {
      const AstroString& b = a[i];
      TextMetrics(b, &w, &h);
      Text(b);
      Move(X - PosX - w, Y - PosY - (int)(sin(angle) * (double)w));
    }
  }   
}

AstroColorList::AstroColorList()
{
  int i;
  Fore = &Ac[0];
  Back = &Ac[15];
  Grey = new AstroColor(100, 100, 100);
  for(i = 0; i < 16; i++)
  {
 		ColorPixmaps[i] = new AstroPixmap(40, 15);
		//ColorPixmaps[i]->fill(Ac[i]);
  }
}

AstroColorList::AstroColorList(unsigned const char *s)
{
  int i;
  for(i = 0; i < 16; i++)
    Ac[i].SetRgb(*s++, *s++, *s++);
  Fore = &Ac[0];
  Back = &Ac[15];
  Grey = new AstroColor(100, 100, 100);
  for(i = 0; i < 16; i++)
	{
 		ColorPixmaps[i] = new AstroPixmap(40, 15);
		ColorPixmaps[i]->fill(Ac[i]);
  }
}

AstroColorList::~AstroColorList()
{
  int i;
  for(i = 0; i < 16; i++)
    delete ColorPixmaps[i];
}

void AstroColorList::SetColor(AstroColor& ac, int i)
{
  if(i >= 0 && i <= 15)
  {
    Ac[i] = ac;
    /*if(ColorPixmaps[i])
      delete ColorPixmaps[i];
    ColorPixmaps[i] = new AstroPixmap(40, 15);*/
    ColorPixmaps[i]->fill(ac);
  }
}

void AstroColorList::SetColor(unsigned char red, unsigned char green, unsigned char blue, int i)
{
  if(i >= 0 && i <= 15)
  {
    Ac[i].SetRgb(red, green, blue);
    /*if(ColorPixmaps[i])
      delete ColorPixmaps[i];
    ColorPixmaps[i] = new AstroPixmap(40, 15);*/
    ColorPixmaps[i]->fill(Ac[i]);
  }
}

void AstroGraphics::SetWidth(int i)
{
  if(Base->GetWidth() < PosX + i)
    Base->Width = PosX + i;
  Width = i;
}

void AstroGraphics::SetHeight(int i)
{
  if(Base->GetHeight() < PosY + i)
    Base->Height = PosY + i;
  Height = i;
}




