/***************************************************************************
                          astrohm.cpp  -  description
                             -------------------
    begin                : Fri Sep 15 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrodefs.h"
#include "astrohm.h"
#include <cstdlib>

#ifdef KDE
#undef i18n
#include <kglobal.h>
#include <klocale.h>
#include <qdatetime.h>
#endif

int AstroHM::Set(int hour, int minute, HM_Mode m)
{
  if(m == HM_Time && (hour < 0 || hour > 23))
    return ASTROHM_WRONG_HOUR;
  if(m == HM_Latitude && (hour < -89 || hour > 89))
    return ASTROHM_WRONG_HOUR;
  if(m == HM_Longitude && (hour < -179 || hour > 179))
    return ASTROHM_WRONG_HOUR;
  if(minute < 0 || minute > 59)
    return ASTROHM_WRONG_MINUTE;
  Hm = DegToDec(RHM(hour, minute));
  return 0;
}

int AstroHM::Set(AstroString as, HM_Mode m)
{
  int h, min, sec, i2;
  char *sep, sp = 0, *c = new char[as.Len() + 1];
  strcpy(c, as.Data());
  bool post = FALSE, neg = FALSE;
  PackString(c);
  switch(m)
  {
    case HM_Time:      i2 = 23; sep = ""; break;
    case HM_Latitude:  i2 = 89; sep = "ns"; break;
    case HM_Longitude:
    case HM_HMS:       i2 = 179; sep = "ew"; break;
  }
  h = GetNum(&c, 0, i2, FALSE, sep);
  if(h == UTILS_NUMERROR)
    return ASTROHM_WRONG_HOUR;
// If no delim and no minute
  if(*c == 0)
  {
    Hm = (double)h;
    return 0;
  }
// If followed by am or pm
  if(m == HM_Time && (!strcmp(c, "am") || !strcmp(c, "pm")))
  {
    h += !strcmp(c, "pm") ? 12 : 0;
    Hm = (double)h;
    return 0;
  }
// If not separated by a delimiter or by the separator
  if(!in(*c, DELIM))
  {
    if(!in(*c, sep))
      return ASTROHM_WRONG_SEPARATOR;
    else
      sp = *c;
  }
  else
    post = TRUE;
  c++;
  min = GetNum(&c, 0, 59, TRUE, 0);
  if(min == UTILS_NUMERROR)
    return ASTROHM_WRONG_MINUTE;
// If followed by am or pm
  if(m == HM_Time && (!strcmp(c, "am") || !strcmp(c, "pm")))
  {
    h += !strcmp(c, "pm") ? 12 : 0;
    Hm = DegToDec(RHM(h, min));
    return 0;
  }
  if(post && m != HM_Time)
  {
    if(!in(*c, sep))
      return ASTROHM_WRONG_SEPARATOR;
    else
      sp = *c;
  }
  if(sp && sp != *sep)
    neg = TRUE;
// If Hour:minute:second longitude format
  if(m == HM_HMS)
  {
 // The total number of second
    int s;
    double x, y;
    c++;
    sec = GetNum(&c, 0, 59, TRUE, 0);
    if(min == UTILS_NUMERROR)
      return ASTROHM_WRONG_SECOND;
    s = sec + (60 * min) + (3600 * abs(h));
//  86400 = the number of seconds in a day
    x = (double)s * 360.0 / 86400.0;
//  Conversion from decimal remainder to sexagesimal
    y = floor((x - floor(x) + 0.00005) * 10000.0);
    y *= 60.0 / 100.0;
    x = floor(x) + (y / 10000.0);
    Hm = DegToDec(neg ? -x : x);		
  }
  else
    Hm = DegToDec(RHM(h, min) * (neg ? -1.0 : 1.0));
  return 0;
}

AstroHM& AstroHM::operator = (double d)
{
  Hm = DegToDec(d);
  return *this;
}

/** This method returns an AstroString string containing the formatted hm value */
const AstroString& AstroHM::Get(AstroString& result, HM_Mode m, bool locale) const
{
  int h, min, sec;
  char buf[12];
  result = (char)0;
  bool neg = Hm < 0.0 ? TRUE : FALSE;
  if(locale && m == HM_Time)
  {
  	HMR(DecToDeg(Hm), h, min);
  	const QTime qt(h, min);
  	result = KGlobal::locale()->formatTime(qt);
	return result;
  }
  switch(m)
  {
    case HM_Time     : HMR(DecToDeg(Hm), h, min); sprintf(buf, "%d%c%.2d", h, ':', min); break;
    case HM_Latitude : HMR(DecToDeg(Hm), h, min); sprintf(buf, "%d%c%.2d%c", abs(h), ':', min, neg ? 'S' : 'N'); break;
    case HM_Longitude: HMR(DecToDeg(Hm), h, min); sprintf(buf, "%d%c%.2d%c", abs(h), ':', min, neg ? 'W' : 'E'); break;
    case HM_HMS:       HMConvert(&h, &min, &sec); sprintf(buf, "%d%c%.2d:%.2d", abs(h), neg ? 'W' : 'E', min, sec);
  }
  result = buf;
  return result;
}

/** This method convert the hm value in hour
minute, second */
void AstroHM::HMConvert(int* h, int* m, int* s) const
{
  int i, j, k;
  double sign, x, y, angle, p = DecToDeg(Hm);
  sign = p < 0.0 ? -1.0 : 1.0;
  x = fabs(p);
  y = floor((x - floor(x) +0.00005) * 10000.0) * 100.0 / 60.0;
  x = (floor(x) + (y / 10000.0)) * 86400.0 / 360.0;
  i = (int)x;
  j = i / 3600;
  angle = (double)j + 0.01;
  *h = (int)(floor(angle) * sign);
  k = (i - (j * 3600));
  *m = k / 60;
  *s = k - (*m * 60);
}




