/***************************************************************************
                          astrointerface.h  -  description
                             -------------------
    begin                : Fri May 18 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROINTERFACE_H
#define ASTROINTERFACE_H

#include "astroresources.h"

#ifdef KDE
#include "astroprocess_stub.h"
#endif

#include "astrodefs.h"
#include "astrostring.h"

class AstroRestrictions;

/* structure that contains the 6 values returned by the swe computation routines */  
struct DataValues
{
  double V[6];
  bool Retro;
  inline double operator[](int i) const { return V[i]; }
};

/** A class that manages all the values for the computed objects */
class Values
{
 public:

  bool Ok;

  /** returns the ith data */
  inline const DataValues& operator[](int i) const { return Data[i]; }

  /** Returns the first (longitude) value for the ith object */
  inline double Get(int i) const { return Data[i].V[0]; }

  /** If the ith object is retro */
  inline bool Retro(int i) const { return Data[i].Retro; }

  /** Gets the jth value of the ith object */
  inline double Get(int i, int j) const { return Data[i].V[j]; }

  /** Sets all the values for the ith object */
  void Set(int i, double d, bool r = FALSE, double d1 = 0.0, double d2 = 0.0, double d3 = 0.0, double d4 = 0.0, double d5 = 0.0);

  /** Just sets the first value of object i */
  void CSet(int i, double d, bool r = FALSE);

 private:

  DataValues Data[Last_Star + 1];
};

/** Experimental. Don't use it by now */
template <class T> class AstroByteArray: public QByteArray
{
 public:

  inline void operator=(const T& t) { assign(&t, sizeof(T)); }
  inline const T& operator*(void) { return (T&) *data(); }
};

typedef AstroByteArray<Values> ValuesArray;
typedef AstroByteArray<AstroRestrictions> RestrictionsArray;

/** This class provides an interface to the astro calculation process */
class AstroInterface:
#ifdef KDE
 public AstroComputing_stub
#endif
{

 public:

  /** Constructor
      @param app the server name
      @param id the served class id
  */
  AstroInterface(AstroString app, AstroString id);

  /** Sets timezone file */
  void SetTZ(const char* file = 0, bool set = TRUE);

  /** Gives the path for the swe positions files */
  void SetSweDataPath(const char* path);

  /** Recomputes the objects with the date and position params */
  bool Recompute(Values* vals, double julday, double latitude, double longitude, int extraflags, bool gethouses = TRUE, int restr = 0);

  /** Sets the restriction group */
  void SetRestrict(const AstroRestrictions* ar, int restr = 0);

  /** Returns the error text line if any */ 
  void GetError(AstroString&);

  /** Returns zoneinfo buffer (internal use only */
  inline const char *GetZib() const { return ZoneInfoBuf; }

 private:

  char ZoneInfoBuf[BUFSIZE];
#ifdef KDE
  QByteArray QbaInput, QbaOutput;
#endif
};

#endif


