/***************************************************************************
                          astromainwindow.cpp  -  description
                             -------------------
    begin                : Sat May 26 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astromainwindow.h"
#include <qworkspace.h>
#include <qsplitter.h>
#include <kmenubar.h>
#include <kglobal.h>
#include <kstddirs.h>
#include <qvaluelist.h>
#undef forward
#include <kstdaccel.h>
#include <kaction.h> 
#include <kstdaction.h>
#include "astrodefs.h"
#include "astroprocess.h"
#include "astroutils.h"
#include "astrodata.h"
#include "searchdata.h"
#include "searchrest.h"
#include "editcountry.h"

#ifdef KDE
#define SAC(s) KStdAccel::key(KStdAccel::s)
#define MENU(text, acc, slot, id) (void)new KAction(text, acc, this, SLOT(slot), actionCollection(), id)
#define IMENU(text, icon, acc, slot, id) (void)new KAction(text, *Asr->Pixmaps[icon + 2], acc, this, SLOT(slot), actionCollection(), id)
#define ITMENU(text, icon, acc, slot, id) new KToggleAction(text, *Asr->Pixmaps[icon + 2], acc, this, SLOT(slot), actionCollection(), id)
#define IMMENU(text, icon, id) new KActionMenu(text, *Asr->Pixmaps[icon + 3], this, id)

#endif
#define INSBTN(i, s) Ktb->insertButton(*Asr->Pixmaps[i + 2], i, SIGNAL(clicked(int)), this, SLOT(IconClicked(int)), TRUE, s, i)
#define NEWDATA 0
#define SEARCHDATA 1
#define ONR1 2
#define ONR2 3
#define ONR3 4
#define ONR4 5
#define ANIMOF 7
#define INVANIM 8
#define JBASE 9
#define JRATE 9

AstroMainWindow::AstroMainWindow(const AstroData* ad)
{
  int i;
  AstroString as;
  AstroWindow* aw;
  Ring = 1;
#ifdef KDE
  emit resize(780, 590);
  Qsp = new QSplitter(this);
  QValueList<int> qvl;
  qvl.append(119);
  setCentralWidget(Qsp);
  Aiw = new AstroIconsWindow(this);
  CHECK_PTR(Aiw);
  Qsp->setResizeMode(Aiw, QSplitter::Stretch);
  Qsp->setSizes(qvl);
  Qws = new QWorkspace(Qsp);
  CHECK_PTR(Qws);
  Backgr = new KPixmap(KGlobal::dirs()->findResource("wallpaper", "Paper01.jpg"));
  if(Backgr)
  {
    Qws->setBackgroundMode(FixedPixmap);
    Qws->setBackgroundPixmap(*Backgr);
  }
  SetupActions();
  Ktb = toolBar("mainToolBar");
  for(i = 0; i < 7; i++)
    JumpBase.insertItem(*Asr->JumpBaseNames[i], i);
  connect(&JumpBase, SIGNAL(activated(int)), this, SLOT(JumpBaseSlot(int)));
  JumpBase.setCheckable(TRUE);
  JumpBase.setItemChecked(0, TRUE);
  for(i = 1; i <= 9; i++)
  {
    as = i;
    JumpRate.insertItem(as, i);
  }
  connect(&JumpRate, SIGNAL(activated(int)), this, SLOT(JumpRateSlot(int)));
  connect(Qws, SIGNAL(windowActivated( QWidget*)), this, SLOT(WindowActivated(QWidget*)));
  JumpRate.setCheckable(TRUE);
  JumpRate.setItemChecked(1, TRUE);
  Ktb->insertButton(*Asr->Pixmaps[JBASE + 2], JBASE, &JumpBase, TRUE, i18n("Jump base"), JBASE);
  Ktb->insertButton(*Asr->Pixmaps[JRATE + 2], JRATE, &JumpRate, TRUE, i18n("Jump rate"), JRATE);
#endif
  if(ad)
  {
    aw = new AstroWindow(this, (AstroData*)ad);
    CHECK_PTR(aw);
    Append(aw);
  }
#ifdef KDE
  show();
#endif
}

#ifdef KDE
void AstroMainWindow::SetupActions()
{
  KStdAction::quit(this, SLOT(Quit()), actionCollection());
  KStdAction::print(this, SLOT(Print()), actionCollection());
  KStdAction::saveOptions(this, SLOT(SaveParms()), actionCollection());
  KStdAction::preferences(this, SLOT(EditDefParms()), actionCollection());
  MENU(i18n("&New Data"), SAC(New), NewData(), "new data");
  MENU(i18n("Search for &Data"), SAC(Find), SearchForData(), "search for data");
  MENU(i18n("Search for &Restrictions Sets"), 0, SearchForRest(), "search for restrictions");
  MENU(i18n("Edit &Countries"), 0, EditCountries(), "edit countries");
  MENU(i18n("New &Restrictions Set"), CTRL+Key_N, NewRestrictions(), "new restrictions set");
  MENU(i18n("&Edit"), 0, Edit(), "edit");
  MENU(i18n("&Delete"), 0, Delete(), "delete");
  MENU(i18n("&Save"), 0, Save(), "save");
  MENU(i18n("D&uplicate"), SAC(Insert), Duplicate(), "duplicate");
  MENU(i18n("S&tandard Chart"), 0, StandardChart(), "standard chart");
  MENU(i18n("&Edit Parameters"), 0, EditParams(), "edit params");
  MENU(i18n("&Tile Windows"), 0, Tile(), "tile");
  MENU(i18n("&Cascade Windows"), 0, Cascade(), "cascade");
  IMENU(i18n("New Data"), NEWDATA, Key_N, NewData(), "new data");
  IMENU(i18n("Search for Data"), SEARCHDATA, Key_S, SearchForData(), "search for data");
  Onr1 = ITMENU(i18n("Onto ring &1"), ONR1, Key_1, Ring1(), "onto ring 1");
  Onr1->setChecked(TRUE);
  Onr2 = ITMENU(i18n("Onto ring &2"), ONR2, Key_2, Ring2(), "onto ring 2");
  Onr3 = ITMENU(i18n("Onto ring &3"), ONR3, Key_3, Ring3(), "onto ring 3");
  Onr4 = ITMENU(i18n("Onto Ring &4"), ONR4, Key_4, Ring4(), "onto ring 4");
  OnOffAnim = ITMENU(i18n("Animation On/Off"),  ANIMOF, 0, AnimOnOff(), "anim on off");
  AnimInv = ITMENU(i18n("Inverse animation"), INVANIM, 0, InvAnim(), "inv anim");
 //IMMENU(i18n("Jump base"), 11, "jump base");
  //IMMENU(i18n("Jump rate"), 11, "jump rate");

  createGUI();
}
#endif

void AstroMainWindow::IconClicked(int j)
{
}

void AstroMainWindow::Ring1()
{
	Ring = ONR1 - 1;
	emit Onr2->setChecked(FALSE);
	emit Onr3->setChecked(FALSE);
	emit Onr4->setChecked(FALSE);
}

void AstroMainWindow::Ring2()
{
	Ring = ONR2 - 1;
	emit Onr1->setChecked(FALSE);
	emit Onr3->setChecked(FALSE);
	emit Onr4->setChecked(FALSE);
}


void AstroMainWindow::Ring3()
{
	Ring = ONR3 - 1;
	emit Onr1->setChecked(FALSE);
	emit Onr2->setChecked(FALSE);
	emit Onr4->setChecked(FALSE);
}

void AstroMainWindow::Ring4()
{
	Ring = ONR4 - 1;
	emit Onr1->setChecked(FALSE);
	emit Onr2->setChecked(FALSE);
	emit Onr3->setChecked(FALSE);
}

void AstroMainWindow::AnimOnOff()
{
	ActiveWindow()->Anim(OnOffAnim->isChecked());
}

void AstroMainWindow::InvAnim()
{
	ActiveWindow()->Backward = !ActiveWindow()->Backward;
}

AstroWindow* AstroMainWindow::ActiveWindow()
{
#ifdef KDE
  return (AstroWindow *)Qws->activeWindow();
#endif
}

void AstroMainWindow::StandardChart()
{
  AstroWindow* aw;
  const AstroData* ad = Aiw->GetData();
  if(ad)
  {
    aw = new AstroWindow(this, (AstroData*)ad);
    CHECK_PTR(aw);
    Append(aw);
  }
}

void AstroMainWindow::EditParams()
{
  AstroWindow* aw = ActiveWindow();
  if(aw->EditParms())
    aw->Repaint(TRUE);
}

 void AstroMainWindow::EditDefParms()
{
  Asr->Edit();
}

void AstroMainWindow::SaveParms()
{
  Asr->Save();
}

void AstroMainWindow::EditCountries()
{
  EditCountry ec;
  ec.show();
}

void AstroMainWindow::Quit()
{
  extern AstroProcServer *Aps;
  AstroString as;
  if(AstroQuestion(Sprintf(i18n("Do you really want to quit %s ?"), &as, PACKAGE)));
    Aps->KillAll();
}

void AstroMainWindow::Tile()
{
#ifdef KDE
  emit Qws->tile();
#endif
}

void AstroMainWindow::Cascade()
{
#ifdef KDE
  emit Qws->cascade();
#endif
}

void AstroMainWindow::NewData()
{
  AstroData* ada;
  ada = new AstroData;
  ada->Now();
  if(ada->Edit())
    Aiw->AddData(ada);
  else
    delete ada;
}

void AstroMainWindow::SearchForData()
{
  SearchData sd;
  sd.show();
}

void AstroMainWindow::SearchForRest()
{
  SearchRest sr;
  sr.show();
}

void AstroMainWindow::NewRestrictions()
{
  AstroRestrictions* ar = new AstroRestrictions;
  CHECK_PTR(ar);
  ar->Name = i18n("New restrictions set");
  if(ar->Edit())
    Aiw->AddRestrict(ar);
  else
    delete ar;
}
 
void AstroMainWindow::Edit()
{
  if(Aiw->Aib)
    switch(Aiw->Aib->Bt)
    {
      case DataButton: EditData(); break;
      case RestrictButton: EditRestr(); break;
      default: AstroMessage(i18n("Can't edit this icon."));
    }
}

void AstroMainWindow::Save()
{
  AstroData* ad;
  AstroRestrictions* ar;
  Index oldidx;
  if(Aiw->Aib)
    switch(Aiw->Aib->Bt)
    {
      case DataButton:
	ad = Aiw->GetData();
	oldidx = ad->Idx;
	Aiw->Idx = Aiw->Aib->Idx = ad->DbSave(oldidx == 0 ? TRUE : FALSE);
	if(Aiw->Idx != oldidx)
	  Ahl->ChangeIdx(oldidx, Aiw->Idx);
	ad->Modified = FALSE;
	break;
      case RestrictButton:
	ar = Aiw->GetRestrictions();
	oldidx = ad->Idx;
	Aiw->Idx = Aiw->Aib->Idx = ar->DbSave();
	/*if(Aiw->Idx != oldidx)
	  Ahl->ChangeIdx(oldidx, Aiw->Idx);*/
	ar->Modified = FALSE;
	break;
      default: AstroMessage(i18n("Can't save this icon."));
    }
}

void AstroMainWindow::Print()
{
  AstroWindow* aw = ActiveWindow();
  aw->SetDisplayMode(AstroGraphicChart::PrintMode);
  aw->Repaint();
  aw->SetDisplayMode(AstroGraphicChart::GraphicMode);
}

void AstroMainWindow::EditRestr()
{
  AstroRestrictions* ar = Aiw->GetRestrictions();
  if(ar->Edit())
  {
    Aiw->ChangeCurrent(ar);
    Ahl->UpdateRestrict(ar);
  }
}

void AstroMainWindow::EditData()
{
  AstroData* ad = Aiw->GetData();
  if(ad->Edit())
  {
    Aiw->ChangeCurrent(ad);
    Ahl->UpdateData(ad);
  }
}

void AstroMainWindow::Delete()
{
  if(Aiw->Aib)
    switch(Aiw->Aib->Bt)
    {
      case DataButton: DeleteData(); break;
      case RestrictButton: DeleteRestr(); break;
      default: AstroMessage(i18n("Can't delete this icon."));
    }
}

void AstroMainWindow::Duplicate()
{
  AstroData* ad;
  AstroRestrictions* ar;
  AstroString as;
  if(Aiw->Aib)
    switch(Aiw->Aib->Bt)
    {
      case DataButton:
	ad = new AstroData(*Aiw->GetData());
	ad->Name = Sprintf(i18n("Copy of %a"), &as, &ad->Name);
        Aiw->AddData(ad);
	break;
      case RestrictButton:
	ar = new AstroRestrictions(*Aiw->GetRestrictions());
	ar->Name = Sprintf(i18n("Copy of %a"), &as, &ar->Name);
	Aiw->AddRestrict(ar);
	break;
      default: AstroMessage(i18n("Can't duplicate this icon."));
    }
}

void AstroMainWindow::DeleteData()
{
  AstroChart* ac;
  const AstroData* ad = Aiw->GetData();
  for(ac = Ahl->First(); ac; ac = Ahl->Next())
  {
    if(!Aiw->Aib)
    {
      AstroMessage(i18n("Can't delete this item !"));
      return;
    };
    if(ac->IfData(ad) != -1)
    {
      AstroMessage(i18n("Data in use. Can't delete it !"));
      return;
    }
  }
  Aiw->DeleteCurrent();
}

void AstroMainWindow::DeleteRestr()
{
  AstroChart* ac;
  const AstroRestrictions* ar = Aiw->GetRestrictions();
  for(ac = Ahl->First(); ac; ac = Ahl->Next())
  {
    if(!Aiw->Aib)
    {
      AstroMessage(i18n("Can't delete this item !"));
      return;
    };
    if(ac->IfRestrict(ar) != -1)
    {
      AstroMessage(i18n("Restrictions set in use. Can't delete it !"));
      return;
    }
  }
  Aiw->DeleteCurrent();
}

void AstroMainWindow::Close(AstroWindow* aw)
{
  Remove(aw);
  //delete aw;
}

void AstroMainWindow::WindowActivated(QWidget* w)
{
	AstroWindow* aw = SC(AstroWindow*, w);
	emit OnOffAnim->setChecked(aw->QTim ? TRUE : FALSE);
	emit AnimInv->setChecked(aw->Backward);
	JumpBaseSlot(aw->Jbt);
	JumpRateSlot(aw->Rate);
}

void AstroMainWindow::JumpBaseSlot(int i)
{
  int j;
  for(j = 0; j < 7; j++)
    JumpBase.setItemChecked(j, FALSE );
  JumpBase.setItemChecked(i, TRUE);
  ActiveWindow()->SetJbt((enum AstroWindow::JumpBaseType)i);
}

void AstroMainWindow::JumpRateSlot(int i)
{
  int j;
  for(j = 1; j <= 9; j++)
    JumpRate.setItemChecked(j, FALSE);
  JumpRate.setItemChecked(i, TRUE);
  ActiveWindow()->SetRate(i);
}
