/***************************************************************************
                          astromainwindow.h  -  description
                             -------------------
    begin                : Sat May 26 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROMAINWINDOW_H
#define  ASTROMAINWINDOW_H

#include "../config.h"
#include <kmainwindow.h>
#include "astrostring.h"
#include "astrolist.h"
#include "astrowindow.h"

#ifdef KDE
class QWorkspace;
class QSplitter;
class KPixmap;
class KToggleAction;
#endif

class AstroData;
class AstroIconsWindow;

/** The application's main window */

class AstroMainWindow

: public KMainWindow,
public AstroList(AstroWindow)
{
  Q_OBJECT
    
  public:

  /** Constructor
      @param ad the first data displayed (if any) in an AstroWindow
  */
  AstroMainWindow(const AstroData* ad = 0);

  /** Returns the active window */
  AstroWindow* ActiveWindow();

  friend class AstroWindow;
  friend class SearchData;
  friend class SearchRest;

#ifdef KDE
 public slots:

 /** New chart */
 void NewData();

 /** Search for data */
 void SearchForData();

 /** Search for restrictions sets */
 void SearchForRest();

 /** New restrictions set */
 void NewRestrictions();

 /** Creates a standard chart */
 void StandardChart();

 /** Edit current chart parameters */
 void EditParams();

 /** Edit the default parameters */
 void EditDefParms();

 /** Save parameters */
 void SaveParms();

 /** Edits the countries database */
 void EditCountries();

 /** Quits the application */
 void Quit();

 /** Reorganize the chart windows as tiles */
 void Tile();

 /** /** Reorganize the chart windows in cascade */
 void Cascade();

 /** Edits the current data or restrictions set */
 void Edit();

 /** Deletes the current data or restriction set*/
 void Delete();

 /** Saves the current data or restrictions set into the db */
 void Save();

 /** Duplicate the current window */
 void Duplicate();

 /** Print the current window */
 void Print();

 /** Closes the chart window */ 
 void Close(AstroWindow*);

 /** This slot is invoked when icon i clicked */
 void IconClicked(int i);

 /** This slot is invoked when day onth year... pulldown menu activated */
 void JumpBaseSlot(int);

 /** This slot is invoked when 1, 2, 3, 4... pulldown menu activated */
 void JumpRateSlot(int);

 /** Slots for rings selection */
 void Ring1();
 void Ring2();
 void Ring3();
 void Ring4();

/** Slots for animation s */
 void AnimOnOff();
 void InvAnim();

 /**
 void Animate();

 void ChangeDir();*/

 protected:

  void SetupActions();

  private slots:

  	void WindowActivated(QWidget*);

#endif

 private:

  int Ring;
  AstroIconsWindow* Aiw;
  KToggleAction *Onr1, *Onr2, *Onr3, *Onr4, *OnOffAnim, *AnimInv;
  /** Edit a data */
 void EditData();

 void EditRestr();

 void DeleteData();
 
 void DeleteRestr();

#ifdef KDE 
 QPopupMenu JumpBase, JumpRate;
 QSplitter* Qsp;
 QWorkspace* Qws;
 KPixmap* Backgr;
 KToolBar* Ktb;
#endif
};

#endif
