/***************************************************************************
                          astroparams.cpp  -  description
                             -------------------
    begin                : Sat Sep 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "astroutils.h"
#include "astroparams.h"
#include "astrographics.h"
#include "astrorestrictions.h"
#include "editchartparms.h"
#include "editdefparms.h"
#include "astromainwindow.h"
#include "astrowindow.h"

#ifdef KDE
#include <kapp.h>
#include <kconfig.h>
#endif

extern "C"
{
#include "sweodef.h"
#include "swephexp.h"
#include "sweph.h"
}

// Various const param tables
static const unsigned char ColorVector[16 * 3] = DEF_COLORS;
static const int    RulersVector[12 * 6]       = DEF_RULERS;
static const struct Rest Rv[]                  = DEF_RESTRICTIONS;
static const struct AspectsConstants Acv[]     = DEF_ASPECTS_CONST;
static const struct AspectRestrictions Arv[]   = DEF_ASPECTS_RESTRICTIONS;
const struct Rest* RestVector                  = Rv;
const struct AspectsConstants* AspConstVector  = Acv;
const struct AspectRestrictions* AspRestVector = Arv;
const char* HousesCodes                        = HOUSESCODES;

bool AstroParams::Load()
{
#ifdef KDE
  extern KApplication *Kaptr;
  ConfFile = TRUE;
  QString group = "General";
  KConf = Kaptr->config();
  if(KConf->hasGroup(group))
    KConf->setGroup(group);
  else
    ConfFile = FALSE;
#endif

  DynamicResize = GetArg("Dynamic_Resize", DEF_DYNAMICRESIZE);
  LeftAscendant = GetArg("Left_Ascendant", DEF_LEFTASCENDANT);
  Sidebars      = GetArg("Sidebars", DEF_SIDEBARS);	
  GetArg("Zone_Info_Path", ZPATH, ZPath);
  ExtraTime   = GetArg("Time_Offset", DEF_TIMEOFFSET);
  FirstNow    = GetArg("Now_Chart", DEF_NOW_CHART);
  GetArg("Rulers_Table", Rulers, RulersVector, 12 * 6);
  Width       = GetArg("Graphic_Width", DEF_GRAPHIC_WIDTH);
  Height      = GetArg("Graphic_Height", DEF_GRAPHIC_HEIGHT);
#ifdef KDE
	group = "Colors";
	if(ConfFile)
   	 KConf->setGroup(group);
#endif
  FireColor   = GetArg("Fire_Color", DEF_FIRE_COLOR);
  AirColor    = GetArg("Air_Color", DEF_AIR_COLOR);
  WaterColor  = GetArg("Water_Color", DEF_WATER_COLOR);
  EarthColor  = GetArg("Earth_Color", DEF_EARTH_COLOR);
  OtherColor  = GetArg("Other_Color", DEF_OTHER_COLOR);
  AsteroColor = GetArg("Astero_Color", DEF_ASTERO_COLOR);
  ExoticColor = GetArg("Exotic_Color", DEF_EXOTIC_COLOR);
  GetArg("Palette", *Acl, ColorVector);
  return TRUE;
 }

bool AstroParams::Save()
{
  QString group = "General";
  KConf->setGroup(group);
  SetArg("Dynamic_Resize", DynamicResize);
  SetArg("Left_Ascendant", LeftAscendant);
  SetArg("Sidebars", Sidebars);
  SetArg("Zone_Info_Path", ZPath);
  SetArg("Time_Offset", ExtraTime);
  SetArg("Graphic_Width", Width);
  SetArg("Graphic_Height", Height);
#ifdef KDE
	group = "Colors";
	if(ConfFile)
   	KConf->setGroup(group);
#endif
  SetArg("Fire_Color", FireColor);
  SetArg("Air_Color", AirColor);
  SetArg("Water_Color", WaterColor);
  SetArg("Earth_Color", EarthColor);
  SetArg("Other_Color", OtherColor);
  SetArg("Astero_Color",  AsteroColor);
  SetArg("Exotic_Color", ExoticColor);
  SetArg("Palette", *Acl);
  KConf->sync();
  return TRUE;
}

bool AstroParams::Edit()
{
  EditDefParms edp(this);
  edp.show();
  if(edp.Return && Amw)
  {
    AstroWindow* aw;
    for(aw = Amw->First(); aw; aw = Amw->Next())
      aw->Repaint();
  }
  return edp.Return;
}

void AstroParams::SetArg(const char* ident, int i)
{
#ifdef KDE
  KConf->writeEntry(ident, i);
#endif
}

void AstroParams::SetArg(const char* ident, const AstroString& as)
{
#ifdef KDE
  KConf->writeEntry(ident, as);
#endif
}

void AstroParams::SetArg(const char* ident, const char* s)
{
#ifdef KDE
  KConf->writeEntry(ident, s);
#endif
}

void AstroParams::SetArg(const char* ident, double d)
{
#ifdef KDE
  KConf->writeEntry(ident, d);
#endif
}

void AstroParams::SetArg(const char* ident, bool b)
{
#ifdef KDE
  KConf->writeEntry(ident, b);
#endif
}

void AstroParams::SetArg(const char* ident, const int* array, int nb)
{
#ifdef KDE
  int i;
  QValueList<int> v;
  for(i = 0; i < nb; i++)
    v += array[i];
  KConf->writeEntry(ident, v);
#endif
}

int AstroParams::GetArg(const char* ident, int Default)
{
#ifdef KDE
   return KConf ? KConf->readNumEntry(ident, Default) : Default;
#else
  return Default;
#endif
}

void AstroParams::GetArg(const char* ident, AstroString Default, char* result, int size = BUFSIZE)
{
#ifdef KDE
  if(ConfFile)
  {
    QString s = KConf->readEntry(ident, Default);
    strncpy(result, s.data(), size);
  }
  else
    strncpy(result, Default.Data(), size);
#else
  strncpy(result, Default.Data(), size);
#endif
}

double AstroParams::GetArg(const char* ident, double Default)
{
#ifdef KDE
  return KConf ? KConf->readDoubleNumEntry(ident, Default) : Default;
#else
  return Default;
#endif
}

void AstroParams::GetArg(const char* ident, AstroColorList& acl, const unsigned char* Default)
{
#ifdef KDE
  int i, r, g, b;
  const unsigned char *j = Default;
  QString group = ident;
  AstroString id = "Color";
  QColor col;
  if(ConfFile)
    KConf->setGroup(group);
  for(i = 0; i <= 15; i++)
  {
    r = *j++; g = *j++; b = *j++;
    col.setRgb(r, g, b);
    if(ConfFile)
    {
      col = KConf->readColorEntry(id + i, &col);
    }
    Acl->SetColor(col.red(), col.green(), col.blue(), i);
  }
#endif
}

void AstroParams::SetArg(const char* ident, const AstroColorList& acl)
{
#ifdef KDE
	int i;
	QString group = ident;
	AstroString id = "Color";
	KConf->setGroup(group);
	for(i = 0; i < 16; i++)
	{
		KConf->writeEntry(id + i, acl[i]);
	}
#endif
}

/** Gets a bool argument from init file. If not defined return default) */
 bool AstroParams::GetArg(const char* ident, bool Default)
 {
#ifdef KDE
   return KConf ? KConf->readBoolEntry(ident, Default) : Default;
#else
   return Default;
#endif
 }

void AstroParams::GetArg(const char* ident, int* result, const int* Default, int nb)
{
  int i;
#ifdef KDE
  QValueList<int> qvi;
  QValueList<int>::Iterator it;
  if(ConfFile)
  {
    qvi = KConf->readIntListEntry(ident);
    for(it = qvi.begin(), i = 0; it != qvi.end() && i < nb; it++, i++)
	result[i] = (*it);
    while(i < nb)
      result[i++] = 0;
  }
  else
    for(i = 0; i < nb; i++)
      result[i] = Default[i];
#else
  for(i = 0; i < nb; i++)
      result[i] = Default[i]
#endif
}

void AstroParams::GetArg(const char* ident, double* result, const double* Default, int nb)
{
  int i;
#ifdef KDE
  QStringList qsl;
  QStringList::Iterator it;
  if(ConfFile)
  {
    qsl = KConf->readListEntry(ident);
    for(it = qsl.begin(), i = 0; it != qsl.end() && i < nb; it++, i++)
	result[i] = atof((*it).data());
    while(i < nb)
      result[i++] = 0.0;
  }
  else
    for(i = 0; i < nb; i++)
      result[i] = Default[i];
#else
  for(i = 0; i < nb; i++)
      result[i] = Default[i];
#endif
}

void AstroParams::GetArg(const char* ident, bool* result, const bool* Default, int nb)
{
  int i;
#ifdef KDE
  QValueList<int> qvi;
  QValueList<int>::Iterator it;
  if(ConfFile)
  {
    qvi = KConf->readIntListEntry(ident);
    for(it = qvi.begin(), i = 0; it != qvi.end() && i < nb; it++, i++)
	result[i] = (*it) ? TRUE : FALSE;
    while(i < nb)
      result[i++] = FALSE;
  }
  else
    for(i = 0; i < nb; i++)
      result[i] = Default[i];
#else
  for(i = 0; i < nb; i++)
      result[i] = Default[i];
#endif
}

void AstroParams::GetArg(const char* ident, struct AspectData *result, const struct AspectData * Default)
{
  int i;
#ifdef KDE
  QString s, l, group = ident;
  QStringList qsl;
  QStringList::Iterator it;
  if(ConfFile)
  {
    KConf->setGroup(group);
    for(i = Conjunction; i < Unknown_Aspect; i++)
    {
      l = "Aspect " + i;
      qsl = KConf->readListEntry(l);
      if(qsl.count() == 4)
      {
	it = qsl.begin();
	result[i].Angle = atof((*it).data());
	it++;
	result[i].Orb   = atof((*it).data());
	it++;
	result[i].Ak    = (enum AspectKind)atol((*it).data());
	it++;
	result[i].Color = atol((*it).data());
      }
      else
	result[i] = Default[i];
    }
   group = "General";
   KConf->setGroup(group); 
  }
  else
  {
    for(i = Conjunction; i < Unknown_Aspect; i++)
      result[i] = Default[i];
  }
#else
for(i = Conjunction; i < Unknown_Aspect; i++)
  result[i] = Default[i];
#endif
}

bool ChartParams::Edit()
{
  EditChartParms ecp(this);
  ecp.show();
  return ecp.Return;
}
/** Destructor */
 AstroParams::~AstroParams()
{
}
