/***************************************************************************
                          astroparams.h  -  description
                             -------------------
    begin                : Sat Sep 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef ASTROPARAMS_H
#define ASTROPARAMS_H

#include "astrodefs.h"
#include "astrostring.h"

#ifdef KDE
class KConfig;
#endif 

// The char[] standard size
#define BUFSIZE 80

class AstroColor;
class AstroColorList;
class AstroRestrictions;

/** This class contains the parameters which can be directely used by AstroObjs */
class ChartParams
{
public:

  // General
  bool DynamicResize;	 // if the chart is dynamically resized
  // Wheels
  bool LeftAscendant;	 // If ascendant on the leftmost point
  bool Sidebars;	 // If sidebars displayed

  /** Edits the current parameters */
  bool Edit();
};

/**
*  This class manages the standard kastrolog parameters, such as the default
* latitude
*/
 class AstroParams : public ChartParams
 {
  public:

  struct AspectData
  {
    double Angle, Orb;
    enum AspectKind Ak;
    int Color;
  };

  AstroParams() {}

  char ZPath[BUFSIZE];          // The zone info files path.
  double ExtraTime;             // Extra time to add to the "now" time.	
  bool FirstNow;								// If the first displayed chart is the "now" chart
  int  Rulers[12 * 6];         // Contains the rulers, falls, etc... for all the signs

  int Width, Height;           // The standard graphic width and height

  // The color indexes for the fire, air, water and earth elements, plus other for non specified, and color indexes for the asteroids + Chiron, and the exotic objects.
  int FireColor, AirColor, WaterColor, EarthColor, OtherColor, AsteroColor, ExoticColor; 

  /** Load the parameters form the config file */
  bool Load();

  /** Saves these parameters into the config file */
  bool Save();

  /** Edit the parameters */
  bool Edit();
  /** Destructor */
   ~AstroParams();

  private:

  bool ConfFile;

/**
*  Gets a numerical argument from init file. If not defined, return default
*/
  int GetArg(const char* ident, int Default);
/**
*  Gets a string argument from init file. If not defined, result = default
*/
  void GetArg(const char* ident, AstroString Default, char* result, int size = BUFSIZE);
/**
*  Gets a double argument from init file. If not defined, return default
*/
  double GetArg(const char* ident, double Default);

/** Gets a bool argument from init file. If not defined return result */
  bool GetArg(const char* ident, bool Default);

  /** Initializes the color list from an array of 16 rgb triplets */
  void GetArg(const char* ident, AstroColorList&, const unsigned char *);

  /** Initializes an array of int with nb elems */
  void GetArg(const char* ident, int* result, const int* Default, int nb);

  /** Initializes an array of bool with nb elems */
  void GetArg(const char* ident, bool *result, const bool* Default, int nb);

  /** Initializes an array of double with nb elems */
  void GetArg(const char* ident, double *result, const double* Default, int nb);
  /** Initializes the AspData table */
  void GetArg(const char* ident, struct AspectData *result, const struct AspectData * Default);

  void SetArg(const char* ident, int);

  void SetArg(const char* ident, const AstroString&);

  void SetArg(const char* ident, const char*);

  void SetArg(const char* ident, double);

  void SetArg(const char* ident, bool);

  void SetArg(const char* ident, const int*, int nb);

	void SetArg(const char* ident, const AstroColorList&);
	
#ifdef KDE
  KConfig* KConf;
#endif
 };

#endif
