/***************************************************************************
                          astroprocess.cpp  -  description
                             -------------------
    begin                : Fri May 11 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astroprocess.h"
#include "astrostring.h"
#include "astroresources.h"
#include "astrointerface.h"
#include "astroutils.h"
#include "astrorestrictions.h"
#include "astrochartbase.h"

#ifdef KDE
#include "kapp.h"
#endif
#include <cstdio>

extern const char* HousesCodes;

extern "C"
{
# include <sys/types.h>
# include <sys/wait.h>
# include <unistd.h>
# include <signal.h>
# include <stdlib.h>
# include "swephexp.h"
};

#define APP app.local8Bit()
#define ID  id.local8Bit()

struct GetHousesStruct
{
  double cusps[13], asc, mc, fc, desc, vertex, east_point;
};

static int piddes[2], sigdes[2];
 
static void WritePid(pid_t pid)
{
  write(piddes[1], &pid, sizeof(pid_t));
}

static pid_t ReadPid()
{
  pid_t i;
  read(piddes[0], &i, sizeof(pid_t));
  return i;
}

static void WriteSig(char c)
{
  write(sigdes[1], &c, 1);
}

static char ReadSig()
{
  char c;
  read(sigdes[0], &c, 1);
  return c;
}

void Values::Set(int i, double d, bool r, double d1, double d2, double d3, double d4, double d5)
{
  if(i < 0 || i > Last_Star + 1) return;
  double *v = Data[i].V;
  v[0] = d;
  v[1] = d1;
  v[2] = d2;
  v[3] = d3;
  v[4] = d4;
  v[5] = d5;
  Data[i].Retro = r;
}

void Values::CSet(int i, double d, bool r)
{
  if(i < 0 || i > Last_Star + 1) return;
  double *v = Data[i].V;
  v[0] = d;
  Data[i].Retro = r;
}

AstroProcServer::AstroProcServer()
{
  int status, pgid;
  static char *argv[] = { ASTROPROCSESSERVER };
  setpgrp();
  pgid = getpgrp();
  pipe(piddes);
  pipe(sigdes);
  pid = fork();
  if(pid > 0)
  { // Proc server parent
    WritePid(getpid());
    while(1)
    {
      char c = ReadSig();
      int argc = 1;
      if(c == 'N')
      {
	pid_t p = fork();
	if(!p)
	  { // Comp server child
#ifdef KDE
	  AstroString s = ASTROCOMPUTING;
	  s += getpid();
	  KApplication CmpServer(argc, argv, s.Data(), FALSE, FALSE);
	  DCOPClient* dc = CmpServer.dcopClient();
	  if(dc->attach())
	  {
	    dc->registerAs(s.Data());
	    AstroComputing Ac;
	    CmpServer.exec();
	  }
#endif
	}
	else // Proc server parent
	  WritePid(p);
      }
      else if(c == 'K')
	killpg(getpgrp(), 15);
    }
  }
  else // Proc server child
  {
    setpgid(0, pgid);
    pid = ReadPid();
  }
}

int AstroProcServer::NewCmpServer()
{
  pid_t p;
  WriteSig('N');
  p = ReadPid();
  return p;
}

void AstroProcServer::KillAll()
{
  WriteSig('K');
}

AstroInterface::AstroInterface(AstroString app, AstroString id) :
#ifdef KDE
    DCOPStub(APP, ID), AstroComputing_stub(APP, ID)
#endif
{
  ZoneInfoBuf[0] = 0;
}

AstroInterface* CreateInterface(bool cmpserver)
{
  int i;
  AstroString s = ASTROCOMPUTING;
  pid_t p = 1;
  if(cmpserver)
    p = AstroProcServer::NewCmpServer();
  if(p <= 0) return 0;
  s += p;
  AstroInterface* asi = new AstroInterface(cmpserver ? s : ASTROINTERFACE, COMPUTEOBJECT);
  if(!asi) return 0;
  for(i = 0; i < 10000; i++)
  {
    asi->Sync();
    if(asi->ok())
      break;
  }
  if(i == 10000)
  {
    AstroMessage(i18n("Can't connect to computing server."));
    AstroProcServer::KillAll();
  }
  QByteArray qba;
  qba.setRawData((char *)Drs, sizeof(_AstroRestrictions));
  //qba.assign((char *)Drs, sizeof(AstroRestrictions));
  asi->KDESetSweDataPath(Asr->SWEphems);
  asi->KDEInitRestricts(qba);
  qba.resetRawData((char *)Drs, sizeof(_AstroRestrictions));
  return asi;
}

AstroComputing::AstroComputing(): QObject(), DCOPObject(COMPUTEOBJECT)
{
  int i;
  Vals = new Values;
  QbaReturn.assign((char *)Vals, sizeof(Values));
}

void AstroInterface::SetSweDataPath(const char* path)
{
  KDESetSweDataPath(path);
}

void AstroComputing::KDESetSweDataPath(QString path)
{
  swe_set_ephe_path((char *)path.data());
}

AstroComputing::~AstroComputing()
{
  int i;
  if(tzsave[0])
    setenv("TZ", tzsave, 1);
 else
    unsetenv("TZ");
  delete Vals;
  for(i = 0; i < NBDATA; i++)
    delete Ar[i];
}

void AstroInterface::SetTZ(const char *file, bool set)
{
  char buf[BUFSIZE];
  if(strcmp(file, ZoneInfoBuf))
  {
    strncpy(buf, Asr->ZPath, BUFSIZE);
    strncat(buf, file, BUFSIZE);
    KDESetTZ(buf, set);
    if(file)
      strncpy(ZoneInfoBuf, file, BUFSIZE);
  }
}

void AstroComputing::KDESetTZ(QString file, bool set)
{
  char *c, varbuf[80];
  if(file.data())
  {
    c = getenv("TZ");
    if(c)
      strncpy(tzsave, c, 80);
    varbuf[0] = ':';
    varbuf[1] = 0;
    //strncat(varbuf, Asr->ZPath, 80);
    strncat(varbuf, file.data(), 80);
    setenv("TZ", varbuf, 1);
    if(set)
      tzset();
  }
}

void AstroComputing::SetTZTime(double time, int day, int month, int year)
{
  struct tm tm;
  int h, m;
  h = (int)floor(time);
  m =  (int)(FRACT(fabs(time)) * 100.0 + ROUND / 600.0);
  tm.tm_sec = 0;
  tm.tm_min = m;
  tm.tm_hour = h;
  tm.tm_mday = day;
  tm.tm_mon = month - 1;
  tm.tm_year = year-1900;
  tt = mktime(&tm);
}

double AstroComputing::TimeOffset()
{
  struct tm *tmptrg;
  double dt, zheure, zminute;
  tmptrg =  gmtime(&tt);
  dt = difftime(mktime(tmptrg), tt);
  zminute = floor(dt/60.0);
  zheure = ceil(zminute / 60.0);
  zminute = floor(zminute - (zheure * 60.0));
  return -(zheure + zminute / 100.0);
}

bool AstroComputing::Isdst()
{
  struct tm *tmptrl;
  tmptrl = localtime(&tt);
  return tmptrl->tm_isdst ? TRUE : FALSE;
}

/** Computes the houses, vertex and East Point */
static void GetHouses(double julday, double latitude, double longitude, char housecode, int flags, struct GetHousesStruct *ghs)
{
  double ascmc[10];
  (void)swe_houses_ex(julday, flags, latitude, longitude, (int)housecode, ghs->cusps, ascmc);
  ghs->asc        = ascmc[0];
  ghs->mc         = ascmc[1];
  ghs->fc         = Normalize(ascmc[1] + 180.0);
  ghs->desc       = Normalize(ascmc[0] + 180.0);
  ghs->vertex     = ascmc[3];
  ghs->east_point = ascmc[4];
}

/** Gets a planet values
@param array a double[6] array
@param bool if not 0, returns true if retro */
static bool GetPlanet(int index, double julday, int flags, struct GetHousesStruct* ghs, double *array, char *serr, const _AstroRestrictions& arr, bool *retro = 0)
{
  double Su, Mo;
  int i = index;
  long l;

  if(index == North_Node || index == South_Node)
    index = (arr.TrueNode) ? SE_TRUE_NODE : SE_MEAN_NODE;
  if(index >= Fortune && index <= East_Point)
  {
    switch(index)
    {
      case Fortune: swe_calc_ut(julday, Sun, flags, array, serr);
	Su = *array;
	swe_calc_ut(julday, Moon, flags, array, serr);
	Mo = *array;
	*array = Normalize(ghs->asc - Su + Mo); break;
      case Vertex: *array = ghs->vertex; break;
      case East_Point: *array = ghs->east_point; break;
    }
    *(++array) = *(++array) = *(++array) = *(++array) = *(++array) = 0.0;
    if(retro)
      *retro = FALSE;
    return TRUE;
  }
  if(index > Black_Moon && index < Fortune)
    index++;
  l = swe_calc_ut(julday, index, flags, array, serr);
  if(l < 0)
    return FALSE;
  if(i == South_Node)
    *array = Normalize(*array + 180.0);
  if(retro)
  {
      double array2[6];
      l = swe_calc_ut(julday + ONEMINUTE, index, flags, array2, serr);
      *retro = (*array > array2[0]) ? TRUE : FALSE;
  }
  return TRUE;
}

QByteArray AstroComputing::KDERecompute(double julday, double latitude, double longitude, int extraflags, bool gethouses, int restr)
{
  int i, flags = 0;
  double array[6];
  bool retro;
  char buf[4];
  long l;
  struct GetHousesStruct ghs;
  const _AstroRestrictions& Arr = *Ar[restr];
  flags |= extraflags;
   if(Arr.Sidereal > Tropical)
  {
    flags |= SEFLG_SIDEREAL;
    swe_set_sid_mode(Arr.Sidereal, 0, 0);
  }
  Vals->Ok = FALSE;
  if(gethouses)
    GetHouses(julday, latitude, longitude, HousesCodes[Arr.GetHouse()], flags, &ghs);
  Vals->Set(Ascendant, ghs.asc);
  Vals->Set(MC, ghs.mc);
  Vals->Set(FC, ghs.fc);
  Vals->Set(Descendant, ghs.desc);
  for(i = House_1; i <= House_12; i++)
    Vals->Set(i, ghs.cusps[i - House_1 +1]);
  if(Arr == Vertex)
    Vals->Set(Vertex, ghs.vertex);
  if(Arr == East_Point)
    Vals->Set(East_Point, ghs.east_point);
  for(i = Sun; i <= Fortune; i++)
  {
    if(Arr == i)
    {
      if(GetPlanet(i, julday, flags, &ghs, array, serr, Arr, &retro))
			Vals->Set(i, array[0], retro, array[1], array[2], array[3], array[4], array[5]);
      else
			return QbaReturn;
    }
  }
  if(Arr.IfExotic())
  {
    for(i = First_Exotic; i <= Last_Exotic; i++)
      if(Arr == i)
      {
	if(GetPlanet(i, julday, flags, &ghs, array, serr, Arr, &retro))
	Vals->Set(i, array[0], retro, array[1], array[2], array[3], array[4], array[5]);
	else
	  return QbaReturn;
      }
  }
  if(Arr.IfStars())
  {
    for(i = First_Star; i <= Last_Star; i++)
      if(Arr == i)
      {
	sprintf(buf, "%d", i - First_Star + 1);
	l = swe_fixstar_ut(buf, julday, flags, array, serr);
	if(l)
	  return QbaReturn;
	Vals->Set(i, array[0], FALSE, array[1], array[2], array[3], array[4], array[5]);
      }
  }
  Vals->Ok = TRUE;
  return QbaReturn;
}

void AstroComputing::KDESetRestrict(QByteArray qbrest, int i)
{
  const _AstroRestrictions* ptr = (const _AstroRestrictions*)qbrest.data();
  *Ar[i] = *ptr;
}

QString AstroComputing::KDEGetError()
{
  return serr;
}

bool AstroInterface::Recompute(Values* vals, double julday, double latitude, double longitude, int flags, bool gethouses, int rest)
{
  QbaOutput = KDERecompute(julday, latitude, longitude, flags, gethouses, rest);
  const Values* ptr = (const Values*)QbaOutput.data();
  *vals = *ptr;
  return vals->Ok;
}

void AstroInterface::SetRestrict(const AstroRestrictions* ar, int i)
{
  if(i < 0 || i >= NBDATA) return;
  const _AstroRestrictions* _ar = SC(const _AstroRestrictions*, ar);
  //QbaInput.assign((const char *)ar, sizeof(AstroRestrictions));
  // If assign, get a crash after 2 or 3 calls
  QbaInput.setRawData((const char *)_ar, sizeof(_AstroRestrictions));
  KDESetRestrict(QbaInput, i);
  QbaInput.resetRawData((const char *)_ar, sizeof(_AstroRestrictions));
}

void AstroInterface::GetError(AstroString& as)
{
  as = KDEGetError();
}

void AstroComputing::KDEInitRestricts(QByteArray qba)
{
  int i;
  const _AstroRestrictions* ptr = (const _AstroRestrictions*) qba.data(); 
  for(i = 0; i < NBDATA; i++)
  {
    Ar[i] = new _AstroRestrictions;
    *Ar[i] = *ptr;
    //memcpy(Ar[i], ptr, sizeof(_AstroRestrictions));
  }
}

void AstroComputing::Kill()
{
  exit(0);
}

void AstroComputing::Sync()
{
  // Does nothing !
}
