/***************************************************************************
                          astroprocess.h  -  description
                             -------------------
    begin                : Fri May 11 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROPROCESS_H
#define ASTROPROCESS_H

#define ASTROPROCSESSERVER "AstroProcessesServer"
#define ASTROCOMPUTING     "AstroComputing "
#define ASTROINTERFACE     "Interface"
#define COMPUTEOBJECT      "Computing Object"

#include "../config.h"
#include "astrodefs.h"

#ifdef KDE
#include <dcopclient.h>
#include <dcopobject.h>
#include <qobject.h>
#include <qstring.h>
#endif

extern "C"
{
# include <time.h>
}

class AstroInterface;
class _AstroRestrictions;
class Values;

#ifdef KDE

/** The computing class itself */
class AstroComputing: public QObject, virtual public DCOPObject
{
  Q_OBJECT
  K_DCOP

 public:

  /** Constructor */
  AstroComputing();

  /** Destructor */
  ~AstroComputing();

 k_dcop:

  /**
   *  Sets the time and hour values. Time must NOT be expressed in decimal
   */
  void SetTZTime(double time, int day, int month, int year);

  /*
   * Returns the time offset (hour.minute)
   */
  double TimeOffset();

  /**
   * Says if the daylight time is active or not
   */
  bool Isdst();

  /** Sets (using a QString) the timezone file. if set == TRUE,
      the time conversion is set */
  void KDESetTZ(QString file, bool set = TRUE);

  /**
   * Sets (usign a QString) the swe data path
   */
  void KDESetSweDataPath(QString path);

  QByteArray KDERecompute(double julday, double latitude, double longitude, int extraflags, bool gethouses, int restr);

  void KDEInitRestricts(QByteArray qba);

  void KDESetRestrict(QByteArray qba, int i);

  QString KDEGetError();

  /** Kills the current process */
  void Kill();

  /** Does nothing. Just for client/server syncro */
  void Sync();
#endif

 private:

  time_t tt;
  char tzsave[80];
  char serr[256];
  _AstroRestrictions* Ar[NBDATA];
  QByteArray QbaReturn;
  Values* Vals;
};

/** This class manages the creation of the computing servers */
class AstroProcServer
{
 public:

  /** Creates a computing server and return its interface */
  friend AstroInterface* CreateInterface(bool cmpserver = TRUE);

  /** Constructor */
  AstroProcServer();

  /** Ends the application */
  static void KillAll();

 private:

  static int NewCmpServer(); 
  int pid;
};

#endif




