/***************************************************************************
                          astroresources.cpp  -  description
                             -------------------
    begin                : Mon Oct 9 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astroresources.h"
#include "astroutils.h"
#include "astrowindow.h"
#include "astrosql.h"

#ifdef KDE
#include <kglobal.h>
#include <kstddirs.h>
//#include <klocale.h>
#endif

extern "C"
{
#include "sweodef.h"
#include "swephexp.h"
#include "sweph.h"
};

#undef i18n

#ifndef KDE
#define i18n(c) new AstroString(c)
#else
extern QString i18n(const char *text);
#define i18n(c) new AstroString(i18n(c))
#endif

AstroResources::AstroResources()
{
	Load();
// Initialisation of the various used constants

  fill(DayNames, i18n("Monday"), i18n("Tuesday"), i18n("Wednesday"), i18n("Thusrday"),
       i18n("Friday"), i18n("Saturday"), i18n("Sunday"), 0);

  fill(DayShortNames, i18n("Mon"), i18n("Tue"), i18n("Wed"), i18n("Thu"), i18n("Fri"),
       i18n("Sat"), i18n("Sun"), 0);

  fill(MonthNames, i18n("January"),  i18n("February"),  i18n("March"),  i18n("April"),
       i18n("May"),  i18n("June"),  i18n("July"),  i18n("August"),  i18n("September"),
       i18n("October"),  i18n("November"),  i18n("December"), 0);
  
  fill(MonthShortNames, i18n("Jan"), i18n("Feb"), i18n("Mar"), i18n("Apr"), i18n("May"),
       i18n("Jun"), i18n("Jul"), i18n("Aug"), i18n("Sep"), i18n("Oct"), i18n("Nov"),
       i18n("Dec"), 0);

  fill(HousesSystemNames, i18n("Placidus"),  i18n("Koch"), i18n("Porphyrius"),
        i18n("Regiomontanus"), i18n("Campanus"), i18n("Equal"), i18n("Vehlow"),
        i18n( "Meridian"), i18n("Azimuthal"), i18n("Topocentric"), i18n( "Alcabitus"), 0);

  fill(SiderealSystemNames, i18n("Tropical"), i18n("Fagan Bradley"),  i18n("Lahiri"),
       i18n("Deluce"), i18n("Raman"), i18n("Ushashashi"), i18n("Krishnamurti"),
       i18n("Djwhal Khul"), i18n("Yukteshwar"), i18n("Jn Bhasin"),
       i18n("Babyl Kugler 1"), i18n("Babyl Kugler 2"), i18n("Babyl Kugler 3"),
       i18n("Babyl Huber"), i18n("Babyl Etpsc"), i18n("Aldebaran 15Tau"),
       i18n("Hipparchos"), i18n("Sassanian"), i18n("Galcent 0Sag"), i18n("J2000"),
       i18n("J1900"), i18n("B_1950"), 0 );

  fill(ObjNames, i18n(SE_NAME_SUN), i18n(SE_NAME_MOON), i18n(SE_NAME_MERCURY),
  i18n(SE_NAME_VENUS), i18n(SE_NAME_MARS), i18n(SE_NAME_JUPITER), i18n(SE_NAME_SATURN),
  i18n(SE_NAME_URANUS), i18n(SE_NAME_NEPTUNE), i18n(SE_NAME_PLUTO), i18n("North Node"),
  i18n("South Node"), i18n("Black Moon"), i18n(SE_NAME_EARTH), i18n(SE_NAME_CHIRON),
  i18n(SE_NAME_PHOLUS), i18n(SE_NAME_CERES), i18n(SE_NAME_PALLAS), i18n(SE_NAME_JUNO),
  i18n(SE_NAME_VESTA) , i18n("Fortune"), i18n("Vertex"), i18n("East Point"), i18n(SE_NAME_CUPIDO), 
  i18n(SE_NAME_HADES), i18n(SE_NAME_ZEUS), i18n(SE_NAME_KRONOS), i18n(SE_NAME_APOLLON),
  i18n(SE_NAME_ADMETOS), i18n(SE_NAME_VULKANUS), i18n(SE_NAME_POSEIDON),  i18n("Ascendant"),
  i18n("Midheaven"), i18n("Nadir"), i18n("Descendant"), i18n("1st House"), i18n("2nd House"),
  i18n("3rd House"), i18n("4th House"), i18n("5th House"), i18n("6th House"), i18n("7th House"),
  i18n("8th House"), i18n("9th House"), i18n("10th House"), i18n("11th House"), i18n("12th House"), 0);

  fill(ObjShortNames, i18n("Sun"), i18n("Moo"), i18n("Mer"), i18n("Ven"), i18n("Mar"),
  i18n("Jup"), i18n("Sat"), i18n("Ura"), i18n("Nep"), i18n("Plu"), i18n("Nnd"), i18n("Snd"),
  i18n("Blm"), i18n("Ear"), i18n("Chi"), i18n("Pho"), i18n("Cer"), i18n("Pal"), i18n("Jun"),
  i18n("Ves"), i18n("For"), i18n("Ver"), i18n("EP "), i18n("Cup"), i18n("Had"), i18n("Zeu"),
  i18n("Kro"), i18n("Apo"), i18n("Adm"), i18n("Vul"), i18n("Pos"), i18n("Asc"), i18n("MC "),
  i18n("Nad"), i18n("Dsc"), i18n("Hs1"), i18n("Hs2"), i18n("Hs3"), i18n("Hs4"), i18n("Hs5"),
  i18n("Hs6"),
  i18n("Hs7"), i18n("Hs8"), i18n("Hs9"), i18n("H10"), i18n("H11"), i18n("H12"), 0);
#ifdef KDE
  ObjError = i18n("Unknown object");
#else
  ObjError = new AstroString(i18n("Unknown object"));
#endif

  fill(SignsNames, i18n("Aries"), i18n("Taurus"), i18n("Gemini"), i18n("Cancer"), i18n("Leo"),
  i18n("Virgo"), i18n("Libra"), i18n("Scorpio"), i18n("Sagittarius"), i18n("Capricorn"),
  i18n("Aquarius"), i18n("Pisces"), 0);

#ifdef KDE
  SignsError = i18n("Unknown sign");
#else
  SignsError = new AstroString(i18n("Unknown sign"));
#endif 

  fill(SignsShortNames, i18n("Ari"), i18n("Tau"), i18n("Gem"), i18n("Can"), i18n("Leo"),
  i18n("Vir"), i18n("Lib"), i18n("Sco"), i18n("Sag"), i18n("Cap"), i18n("Aqu"), i18n("Pis"), 0);

  fill(StarsNames, i18n("Achernar"), i18n("Polaris"), i18n("ZetaRetic. "), i18n("Alcyone"),
  i18n("Aldebaran"), i18n("Capella"), i18n("Rigel"), i18n("Bellatrix"), i18n("Elnath"),
  i18n("Alnilam"), i18n("Betelgeuse"), i18n("Menkalinan"), i18n("Mirzam"), i18n("Canopus"),
  i18n("Alhena"), i18n("Sirius"), i18n("Adhara"), i18n("Wezen"), i18n("Castor"), i18n("Procyon"),
  i18n("Pollux"), i18n("Suhail"), i18n("Avior"), i18n("Miaplacidus"), i18n("Alphard"),
  i18n("Regulus"), i18n("Dubhe"), i18n("Acrux"), i18n("Gacrux"), i18n("Mimosa"), i18n("Alioth"),
  i18n("Spica"), i18n("Alkaid"), i18n("Agena"), i18n("Arcturus"), i18n("Toliman"), i18n("Antares"),
  i18n("Shaula"), i18n("Sargas"), i18n("Kaus Australis"), i18n("Vega"), i18n("Altair"),
  i18n("Peacock"), i18n("Deneb"), i18n("Alnair"), i18n("Fomalhaut"), i18n("Andromeda"),
  i18n("Alpheratz"), i18n("Algenib"), i18n("Schedar"), i18n("Mirach"), i18n("Alrischa"),
  i18n("Almac"), i18n("Algol"), i18n("Mintaka"), i18n("Wasat"), i18n("Acubens"), i18n("Merak"),
  i18n("Vindemiatrix "), i18n("Mizar"), i18n("Kochab"), i18n("Zubenelgenubi"), i18n("Zubeneshamali"),
  i18n("Alphecca"), i18n("Unukalhai"), i18n("Rasalhague"), i18n("Albireo"), i18n("Alderamin"),
  i18n("Nashira"), i18n("Skat"), i18n("Scheat"), i18n("Markab"), i18n("Gal.Center"), i18n("Apex"), 0);

   fill(StarsShortNames, i18n("Ach"), i18n("Pol"), i18n("Zet"), i18n("Alc"), i18n("Ald"),
   i18n("Cap"), i18n("Rig"), i18n("Bel"), i18n("Eln"), i18n("Aln"), i18n("Bet"), i18n("Men"),
   i18n("Mir"), i18n("Can"), i18n("Alh"), i18n("Sir"), i18n("Adh"), i18n("Wez"), i18n("Cas"),
   i18n("Pro"), i18n("Pol"), i18n("Suh"), i18n("Avi"), i18n("Mia"), i18n("Alp"), i18n("Reg"),
   i18n("Dub"), i18n("Acr"), i18n("Gac"), i18n("Mim"), i18n("Ali"), i18n("Spi"), i18n("Alk"),
   i18n("Age"), i18n("Arc"), i18n("Tol"), i18n("Ant"), i18n("Sha"), i18n("Sar"), i18n("Kau"),
   i18n("Veg"), i18n("Alt"), i18n("Pea"), i18n("Den"), i18n("Aln"), i18n("Fom"), i18n("And"),
   i18n("Alp"), i18n("Alg"), i18n("Sch"), i18n("Mir"), i18n("Alr"), i18n("Alm"), i18n("Alg"),
   i18n("Min"), i18n("Was"), i18n("Acu"), i18n("Mer"), i18n("Vin"), i18n("Miz"), i18n("Koc"),
   i18n("Zub"), i18n("Zub"), i18n("Alp"), i18n("Unu"), i18n("Ras"), i18n("Alb"), i18n("Ald"),
   i18n("Nas"), i18n("Ska"), i18n("Sch"), i18n("Mar"), i18n("Gal"), i18n("Ape"), 0);

#ifdef KDE
   StarsError = i18n("Unknown star");
#else
   StarsError = new AstroString(i18n("Unknown star"));
#endif

   fill(NthAbbrev, i18n("st"), i18n("nd"), i18n("rd"), i18n("th"), i18n("th"), i18n("th"), 
	i18n("th"), i18n("th"), i18n("th"), i18n("th"), i18n("th"), i18n("th"), 0);

   fill(AspectNames, i18n("Conjunction"), i18n("Opposition"), i18n("Trine"), i18n("Square"), i18n("Sextile"), i18n("Inconjunct"), i18n("Semisextile"), i18n("Semisquare"), i18n("Sesquiquadrate"), i18n("Quintile"), i18n("Biquintile"), i18n("Semiquintile"), i18n("Septile"), i18n("Novile"), i18n("Binovile"), i18n("Biseptile"), i18n("Triseptile"), i18n(" Quatronovile"), NULL);
   
   fill(AspectShortNames, i18n("Con"), i18n("Opp"), i18n("Tri"), i18n("Squ"), i18n("Sex"), i18n("Inc"), i18n("Ssx"), i18n("Ssq"), i18n("Ses"), i18n("Qui"), i18n("BQn"), i18n("Sqn"), i18n("Sep"), i18n("Nov"), i18n("Bnv"), i18n("Bsp"), i18n("Tsp"), i18n("Qnv"), NULL);

   fill(ElemPla, Fire, Water, Air, Earth, Fire, Fire, Earth, Air, Water, Water, Undefined_Element, Undefined_Element, Undefined_Element, Earth, Undefined_Element, Undefined_Element, Undefined_Element, Undefined_Element, Undefined_Element, Undefined_Element, Undefined_Element, Undefined_Element,Undefined_Element, Undefined_Element + 1);

   fill(ElemHouses, Fire, Earth, Water, Air, Fire, Earth, Air, Water, Fire, Earth, Air, Water, Fire, Earth, Air, Water, Undefined_Element + 1);
 
   fill(SubTypesNames, i18n(""), i18n("Show/Hide"), i18n("Single"), i18n("Transit"), i18n("Directions"), i18n("Progressions"), NULL);

   fill(JumpBaseNames, i18n("Minutes"), i18n("Hours"), i18n("Days"), i18n("Months"), i18n("Years"), i18n("Centuries"), i18n("Millenia"), 0);

#ifdef KDE
	QString p(PACKAGE);
	p = KGlobal::dirs()->findResourceDir("data", p + '/') + PACKAGE + '/';
	strncpy(SWEphems, p.data(), BUFSIZE);
	p += "pics/";
   strncpy(IconsPath, p.data(), BUFSIZE);
#endif
   SetPixmaps("group.png", "subtype.png", "newdata.png", "searchdata.png", "1.png", "2.png", "3.png", "4.png", 0, "animation.png", "direction.png", "anim2.png", -1);
}

AstroResources::~AstroResources()
{
}

void AstroResources::SetDefZFile()
{
  Index idx;
  char buf[BUFSIZE];
  AstroRequest arq;
  arq.Exec("SELECT countryidx FROM place WHERE idx = 1");
  idx = arq.GetInt("CountryIdx");
  arq.Clear();
  sprintf(buf, "SELECT TZFile FROM Country WHERE Idx = %d", idx);
  arq.Exec(buf);
  strncpy(Asr->DefZFile, arq.Get(arq.GetIndex("TZFile")), BUFSIZE);
}

#ifdef KDE
void AstroResources::fill(AstroString *t[], AstroString *c, ...)
{
  AstroString **ptr = &c;
  while(*ptr)
      *t++ = *ptr++;
}
#else
void AstroResources::fill(AstroString *t[], char *c, ...)
{
  char **ptr = &c;
  while(*ptr)
      *t++ = new AstroString(*ptr++);
}
#endif

void AstroResources::fill(enum Elements *t, enum Elements e, ...)
{
  enum Elements *ptr = &e;
  while(*ptr <= Undefined_Element)
    *t++ = *ptr++;
}

void AstroResources::SetPixmaps(const char* c, ...)
{
  char** ptr = (char **)&c;
  int i;
  for(i = 0; *ptr != (char *)-1; i++, ptr++)
    if(*ptr)
      Pixmaps[i] = new AstroPixmap(*ptr, IconsPath);
}
