/***************************************************************************
                          astroresources.h  -  description
                             -------------------
    begin                : Mon Oct 9 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTRORESOURCES_H
#define ASTRORESOURCES_H

#include "astrostring.h"
#include "astrodefs.h"
#include "astroparams.h"
#include "astrolist.h"

class AstroPixmap;

// The number of objects 
#define OBJNB 48

/**This class provides constants resources such as planets names, signs names , etc...
  *@author Christophe Gros
  */

class AstroResources: public AstroParams
{
public:

  // The days and months names 
  AstroString *DayNames[7], *DayShortNames[7], *MonthNames[12], *MonthShortNames[12];
  AstroString *HousesSystemNames[11], *SiderealSystemNames[22]; 

  // The names and short names (3 characters) of the objects
  AstroString *ObjNames[OBJNB], *ObjShortNames[OBJNB], *ObjError;

  // The names and short names (3 characters) of the astrological signs
  AstroString *SignsNames[12], *SignsShortNames[12], *SignsError;

  // The Stars names and short names
  AstroString *StarsNames[NBSTARS], *StarsShortNames[NBSTARS], *StarsError;

  // First, Second, Third abbreviations
  AstroString *NthAbbrev[12];

  // Aspects names and short names */
  AstroString *AspectNames[Unknown_Aspect], *AspectShortNames[Unknown_Aspect];

  // The tables of Elements for the given planets and houses
  enum Elements ElemPla[Last_Planet], ElemHouses[House_12 - Ascendant];

  // Names of the rings subtypes
  AstroString *SubTypesNames[6];

  // Names of Jump bases
  AstroString* JumpBaseNames[7];

  // Different available pixmaps for icons
  AstroPixmap *Pixmaps[15];

  // The default zone info file
  char DefZFile[BUFSIZE];

 char SWEphems[BUFSIZE]; // The swiss ephemeris path

  char IconsPath[BUFSIZE]; // The icons path

  void SetDefZFile();

  AstroResources();
  ~AstroResources();

 private:
	
#ifdef KDE
	void fill(AstroString *t[], AstroString *c, ...);
#else
	void fill(AstroString *t[], char *c, ...);
#endif
	void fill(enum Elements *t, enum Elements e, ...);

	void SetPixmaps(const char* c, ...);
};

#endif
