/***************************************************************************
                          astrorestrictions.cpp  -  description
                             -------------------
    begin                : Wed Apr 18 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrorestrictions.h"
#include "astroresources.h"
#include "editrestrictions.h"
#include "astrosql.h"
#include "astroutils.h"

#define C(a, b) for(i = a; i <= b; i++) { if(!Rest[i].Rest) c++; }

extern const struct AspectRestrictions* AspRestVector;
extern const struct Rest* RestVector;

AstroRestrictions::AstroRestrictions()
{
   memcpy(Rest, RestVector, sizeof(struct Rest[ Undefined_Obj]));
   memcpy(Asprst, AspRestVector, sizeof(struct AspectRestrictions[Unknown_Aspect]));
   Houses = DEF_HOUSES_COMP;
   Exotic = DEF_EXOTIC_COMP;
   Stars  = DEF_STARS_COMP;
   Minor  = DEF_MINOR_COMP;
   TrueNode = DEF_TRUENODE;
   House    = DEF_HOUSESYS;
   Sidereal = DEF_TROPICAL_SIDEREAL;
   CountObjects();
  	Idx = 0;
  	Modified = FALSE;
}

void _AstroRestrictions::CountObjects()
{
  int i, c = 0;
  C(Sun, Last_Planet);
  if(Houses)
    C(Ascendant, House_12);
  if(Exotic)
    C(First_Exotic, Last_Exotic);
  if(Stars)
    C(First_Star, Last_Star);
  Count = c;
}

bool _AstroRestrictions::operator == (int index) const
{
  if(index < 0 && index > Last_Star)
    return FALSE;
  if((index > Last_Planet && index < First_Exotic) ||
     (index > Last_Exotic && index < Ascendant))
    return FALSE;
  if(!Houses && index >= Ascendant && index <= House_12)
    return FALSE;
  if(!Exotic && index >= First_Exotic && index <= Last_Exotic)
    return FALSE;
  if(!Stars && index >= First_Star && index <= Last_Star)
    return FALSE;
  return !Rest[index].Rest;
}

void _AstroRestrictions::Restrict(int index, bool b)
{
  if(index < 0 && index > Last_Star)
    return;
  if((index > Last_Planet && index < First_Exotic) ||
     (index > Last_Exotic && index < Ascendant))
    return;
  Rest[index].Rest = b;
  if(b)
    Count--;
  else
    Count++;
}

void _AstroRestrictions::AspRestrict(int index, bool b)
{
  if(index < 0 && index > Unknown_Aspect)
    return;
  Asprst[index].Rest = b;
}

bool AstroRestrictions::Edit()
{
  EditRestrictions er(this);
  CountObjects();
  return er.Return;
}

AstroRestrictionsList::AstroRestrictionsList()
{
  Max = BOUND;
  AstroRequest arq;
  AstroString s(i18n("Default restrictions set"));
  Drs = new AstroRestrictions;
  CHECK_PTR(Drs);
  Drs->Idx = 1;
  Drs->Name = s;
  if(arq.Exec("SELECT Idx FROM Restrictions WHERE Idx = 1") && arq.Number() > 0)
  	Drs->DbFetch(1);
  else
  	Drs->DbSave();
  Append(Drs);
}

void AstroRestrictionsList::AddRestrictions(AstroRestrictions* ar)
{
	if(ar->Idx == 0)
  		ar->Idx = ++Max;
  Append(ar);
}

AstroRestrictionsList::~AstroRestrictionsList()
{}

AstroRestrictions* AstroRestrictionsList::Get(Index idx)
{
  AstroRestrictions* ar;
  for(ar = First(); ar; ar = Next())
    if(ar->Idx == idx)
      return ar;
  return 0;
}

bool AstroRestrictions::DbFetch(Index idx)
{
  int i, g, n;
  AstroRequest arq;
  if(!arq.Exec("SELECT Name, Houses, Exotic, Stars, Minor, HousesSys, TrueNode, Sidereal FROM Restrictions WHERE Idx = %d", idx) || arq.Number() == 0)
  {
    AstroMessage(i18n("Can't fetch restrictions set"));
    return FALSE;
  }
  Idx = idx;
  Name   = arq < "Name";
  Houses = arq.GetBool("Houses");
  Exotic = arq.GetBool("Exotic");
  Stars  = arq.GetBool("Stars");
  Minor  = arq.GetBool("Minor");
  House  = (enum House)arq.GetInt("HousesSys");
  TrueNode = arq.GetBool("TrueNode");
  Sidereal = (enum Sidereal)arq.GetInt("Sidereal");
  if(!arq.Exec("SELECT ObjIndex, Rest, OrbAdd, Influence FROM ObjRest WHERE Index = %d", idx))
  {
    AstroMessage(i18n("Can't fetch restrictions set"));
    return FALSE;
  }
  n = arq.Number();
  for(i = 0; i < n; i++, arq.Next())
  {
    g                 = arq.GetInt("ObjIndex");
    if(g < 1 || g >= Last_Star) continue;
    Rest[g].Rest      = arq.GetBool("Rest");
    Rest[g].OrbAdd    = arq.GetDouble("OrbAdd");
    Rest[g].Influence = arq.GetInt("Influence");
  }
  if(!arq.Exec("SELECT AspIndex, Rest, Orb, Color, Influence FROM AspRest WHERE Index = %d", idx))
  {
    AstroMessage(i18n("Can't fetch restrictions set"));
    return FALSE;
  }
  n = arq.Number();
  for(i = 0; i < n; i++, arq.Next())
  {
    g                   = arq.GetInt("AspIndex");
    if(g < 0 || g >= Unknown_Aspect) continue;
    Asprst[g].Rest      = arq.GetBool("Rest");
    Asprst[g].Orb       = arq.GetDouble("Orb");
    Asprst[g].Color     = arq.GetInt("Color");
    Asprst[g].Influence = arq.GetInt("Influence");
  }
 CountObjects();
  return TRUE;
}

void AstroRestrictions::ObjIU(int f, int l, bool update)
{
  int i;
  bool ret;
  AstroRequest arq;
  for(i = f; i <= l; i++)
  {
  		const struct Rest& rest = Rest[i];
  		if(update)
      	ret = arq.Exec("UPDATE ObjRest SET Rest = %b, OrbAdd = %f, Influence = %d WHERE Index = %d AND ObjIndex = %d", rest.Rest, rest.OrbAdd, rest.Influence, Idx, i);
		else
      	ret = arq.Exec("INSERT INTO ObjRest (ObjIndex, Rest, OrbAdd, Influence, Index) VALUES (%d, %b, %f, %d, %d)", i, rest.Rest, rest.OrbAdd, rest.Influence, Idx);
      if(!ret)
      {
      	AstroMessage(i18n("Can't INSERT/UPDATE the restrictions data"));
      	return;
      }
   }
}

Index AstroRestrictions::DbSave()
{
  int i;
  bool update = FALSE;
  Index idx;
  AstroRequest arq;
  if(arq.Exec("SELECT Name, Idx FROM Restrictions WHERE Name = '%a'", &Name) && arq.Number() > 0 && arq.GetInt("Idx") != Idx)
  {
  	AstroMessage(i18n("Sorry, a restrictions set with the same name already exists."));
  	return Idx;
  }
  if(!arq.Exec("SELECT Idx FROM Restrictions WHERE Idx = %d", Idx) || arq.Number() == 0)
  {
    if(!arq.Exec("INSERT INTO Restrictions (Name, Houses, Exotic, Stars, Minor, HousesSys, TrueNode, Sidereal) VALUES ('%a', %b, %b, %b, %b, %d, %b, %d)", &Name, Houses, Exotic, Stars, Minor, House, TrueNode, Sidereal))
    {
      AstroMessage(i18n("Can't save restrictions set"));
      return Idx;
    }
    idx = arq.GetINSId();
    arq.Exec("Select Idx FROM Restrictions WHERE Oid = %d", idx);
    Idx = arq.GetInt("Idx");
  }
  else
  {
    if(!arq.Exec("UPDATE Restrictions SET Name = '%a', Houses = %b, Exotic = %b, Stars = %b, Minor = %b, HousesSys = %d, TrueNode = %b, Sidereal = %d WHERE Idx = %d", &Name, Houses, Exotic, Stars, Minor, House, TrueNode, Sidereal, Idx))
    {
      AstroMessage(i18n("Can't save restrictions set"));
      return Idx;
    }
    update = TRUE;
  }
  ObjIU(Sun, Last_Planet, update); 
  ObjIU(Ascendant, House_12, update);
  ObjIU(First_Exotic, Last_Exotic, update);
  ObjIU(First_Star, Last_Star, update);
  for(i = Conjunction; i < Unknown_Aspect; i++)
  {
  	if(update)
    	arq.Exec("UPDATE AspRest SET Rest = %b, Orb = %f, Color = %d, Influence = %d WHERE Index = %d AND AspIndex = %d",Asprst[i].Rest, Asprst[i].Orb, Asprst[i].Color, Asprst[i].Influence, Idx, i);
   else
   	arq.Exec("INSERT INTO AspRest (AspIndex, Rest, Orb, Color, Influence, Index) VALUES (%d, %b, %f, %d, %d, %d)", i, Asprst[i].Rest, Asprst[i].Orb, Asprst[i].Color, Asprst[i].Influence, Idx);
  	}
  return Idx;
}

void _AstroRestrictions::SetHousesSys(enum House house, bool hide)
{
  House = house;
  if(hide)
    Rest[House_1].Rest  = Rest[House_4].Rest  = Rest[House_7].Rest  = Rest[House_10].Rest = (house == Vehlow || house == Azimuthal) ? FALSE : TRUE;  
}

bool AstroRestrictionsList::RemoveFromDb(Index idx)
{
  AstroRequest arq;
  Remove(Get(idx));
  if(idx >= BOUND)
    return TRUE;
  if(!arq.Exec("DELETE FROM Restrictions WHERE Idx = %d", idx))
    AstroMessage(i18n("Problem in restrictions set deletion"));
  if(!arq.Exec("DELETE FROM AspRest WHERE Index = %d", idx))
    AstroMessage(i18n("Problem in restrictions set deletion"));
  if(!arq.Exec("DELETE FROM ObjRest WHERE Index = %d", idx))
    AstroMessage(i18n("Problem in restrictions set deletion"));
  return TRUE;
}

bool AstroRestrictionsList::CheckName(const AstroString& as)
{
  AstroRestrictions* ar;
  for(ar = First(); ar; ar = Next())
    if(ar->Name == as)
      return TRUE;
  return FALSE;
}