/***************************************************************************
                          astrorestrictions.h  -  description
                             -------------------
    begin                : Wed Apr 18 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTRORESTRICTIONS_H
#define ASTRORESTRICTIONS_H

#include "astrodefs.h"
#include "astrolist.h"
#include "astroaspects.h"

struct Rest
{
  bool Rest;
  double OrbAdd;
  int Influence;
};

/** Restrictions set class
 (whether a planet or aspect is computed or not, orbs, etc...)
*/
class _AstroRestrictions
{
 public:

  friend class AstroAspects;
  friend class EditRestrictions;

  /** Returns true if index is NOT restricted, false otherwise */
  bool operator == (int index) const;

   /** Sets the restriction flag of the index object to b */
  void Restrict(int index, bool b);

  /** Count the number of non-restricted objects */
  inline int CountObjs() const { return Count; }

  /** If the houses are restricted */
  inline bool IfHouses() const { return Houses; }

  /** If the exotic object are restricted */
  inline bool IfExotic() const { return Exotic; }

  /** If the Stars are restricted */
  inline bool IfStars() const { return Stars; }

  inline bool IfMinor() const { return Minor; }

  /** (Un)restrict all the houses */
  inline void RestHouses(bool b) { Houses = b; CountObjs(); }

  /** (Un)restrict all the exotic objects */
  inline void RestExotic(bool b) { Exotic = b; CountObjs(); }

  /** (Un)restrict all the stars */
  inline void RestStars(bool b) { Stars = b; CountObjs(); }

  inline void RestMinor(bool b) { Minor = b; }

  /** If aspect index is NOT restricted */
  inline bool Asp(int index) const { return (index < 0 || index > Unknown_Aspect) ? FALSE : Asprst[index].Rest; }

  /** Sets the aspects restriction flag of the index object to b */
  void AspRestrict(int index, bool b);

  void SetHousesSys(enum House, bool hide = TRUE);
  inline enum House GetHouse() const { return House; }
 
  Index Idx;             // The current database index

  bool Modified;         // If modified

  bool TrueNode;	 // If the Nodes are true instead of mean.
  enum Sidereal Sidereal; // Sidereal charts system.

 protected:

  // Objects restrictions
  struct Rest Rest[Last_Star + 1];
  bool Houses, Exotic, Stars, Minor;
  struct AspectRestrictions Asprst[Unknown_Aspect];
  enum House House;	 // The default house system.
  void CountObjects();
  int Count;
};

class AstroRestrictions: public _AstroRestrictions
{

 public:

 /** Constructor */
  AstroRestrictions();

   bool Edit();
   bool DbFetch(Index idx);
   Index DbSave();
   AstroString Name;

 private:

  void ObjIU(int f, int l, bool update = FALSE);
};

class AstroRestrictionsList: public AstroList(AstroRestrictions)
{
 public:

 /** Constructor */
  AstroRestrictionsList();

 /** Destructor */
  ~AstroRestrictionsList();

  /** Adds a restrictions set */
  void AddRestrictions(AstroRestrictions* ar);

  /** Gets a restrictions set from ites index */
  AstroRestrictions* Get(Index);

/** Removes a restrictions set from database, knowing its index */
  bool RemoveFromDb(Index idx);

  /** Checks if the name already exists in the list */
  bool CheckName(const AstroString&);

 private:

  Index Max;
};

#endif

