/***************************************************************************
                          astrosql.cpp  -  description
                             -------------------
    begin                : Wed Nov 7 2001
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "astrodefs.h"
#include "astrosql.h"
#include <cstdlib>
#include <cstring>

#define BUFSIZE 80

AstroSQL::AstroSQL(const char* dbname, const char* user)
{
  char buf[80];
  Ok = TRUE;
  sprintf(buf, "dbname = %s user = %s", dbname, user);
  conn = PQconnectdb(buf);
  if(!conn)
  {
    Ok = FALSE;
    return;
  }
  enum ConnStatusType cst = PQstatus(conn);
  if(cst != CONNECTION_OK)
  Ok = FALSE;
}

int AstroRequest::Number()
{
  return Ok ? PQntuples(Result) : 0;
}

int AstroRequest::GetIndex(const char *c)
{
  return Ok ? PQfnumber(Result, c) : -1; 
}

bool AstroRequest::SetComment(const AstroString& as, Index idx)
{
  enum ExecStatusType est; 
  AstroString r("UPDATE Data SET Comment = '");
  r += as;
  r += "' WHERE Idx = ";
  r += (int)idx;
  Ok = TRUE;
  Clear();
  Result = PQexec(Asql->conn, r.Utf8().Data());
  if(!Result)
  {
    Ok = FALSE;
    return FALSE;
  }
  est = PQresultStatus(Result);
  if(est == PGRES_FATAL_ERROR || est == PGRES_NONFATAL_ERROR || est == PGRES_BAD_RESPONSE)
  {
    Ok = FALSE;
    return FALSE;
  }
  tuple = 0;
  return TRUE;
}

char* AstroRequest::Get(Index idx)
{
  if(idx == -1)
  {
    printf("Invalid index\n");
    return 0;
  }
  return Ok ? PQgetvalue(Result, tuple, idx) : 0;
}

AstroString AstroRequest::operator < (const char* c)
{
  AstroString as;
  as.Utf8(Get(GetIndex(c)));
  return as;
}

int AstroRequest::GetInt(const char *c)
{
  char buf[20];
  strncpy(buf, Get(GetIndex(c)), 20);
  return atoi(buf);
}

double AstroRequest::GetDouble(const char *c)
{
   char buf[20];
  strncpy(buf, Get(GetIndex(c)), 20);
  return atof(buf);
}

char AstroRequest::GetChar(const char *c)
{
  char buf;
  strncpy(&buf, Get(GetIndex(c)), 1);
  return buf;
}

bool AstroRequest::GetBool(const char *c)
{
  char buf;
  strncpy(&buf, Get(GetIndex(c)), 1);
  return buf == 't' ? TRUE : FALSE;
}

AstroSQL::~AstroSQL()
{
  PQfinish(conn);
}

bool AstroRequest::Exec(const char *c, ...)
{
  enum ExecStatusType est;
  AstroString v, *aptr;
  char buf[2048], buf1[40], *ptr, *ptr1, **x = (char **)&c, *y;
  int i;
  double *fptr;
  x++;
  for(ptr = (char *)c, ptr1 = buf; *ptr; ptr++)
    if(*ptr != '%')
    {
    	if(*ptr == '\\')
    		ptr++;
      *ptr1++ = *ptr;
     }
    else
    {
      ptr++;
      switch(*ptr)
      {
        case 'c' : *ptr1++ = (int)*x;
	  x++;
	  break;
        case 's' : y = (char *)*x;
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
        case 'd' : i = (int)*x;
	  y = buf1;
	  sprintf(buf1, "%d", i);
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
        case 'b': i = (int)*x;
	  y = buf1;
	  sprintf(buf1, "%s", i == 0 ? "\'f\'" : "\'t\'");
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
        case 'f' : fptr = (double *)x;
	y = buf1;
	sprintf(buf1, "%f", *fptr);
	  while(*y)
	    *ptr1++ = *y++;
	  x += 2;
	  break;
	  case 'a': aptr = (AstroString *)*x;
	  v = aptr->Utf8();
	  y = (char *)v.Data();
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
      };
    }
  *ptr1 = 0;
  Ok = TRUE;
  Clear();
  //printf("%s\n", buf);
  Result = PQexec(Asql->conn, buf);
  if(!Result)
  {
    Ok = FALSE;
    return FALSE;
  }
  est = PQresultStatus(Result);
  if(est == PGRES_FATAL_ERROR || est == PGRES_NONFATAL_ERROR || est == PGRES_BAD_RESPONSE)
  {
    Ok = FALSE;
    return FALSE;
  }
  tuple = 0;
  return TRUE;
}

Index AstroRequest::GetINSId()
{
  return PQoidValue(Result);
}
