/***************************************************************************
                          astrosql.h  -  description
                             -------------------
    begin                : Wed Nov 7 2001
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROSQL_H
#define ASTROSQL_H

#include "../config.h"
#include "astrostring.h"
#include "astrodefs.h"
extern "C"
{
#include <libpq-fe.h>
};


class AstroSQL
{

  friend class AstroRequest;

public:

  AstroSQL(const char* dbname, const char* user);
  ~AstroSQL();
  inline bool IfOk() const { return Ok; };

private:

  PGconn* conn;
  bool Ok;  
};

extern AstroSQL* Asq;

class AstroRequest
{
public:

  /** Constructor */
  inline AstroRequest(AstroSQL* asql = Asq) { Asql = asql; Result = 0; }

  /** Destructor */
  ~AstroRequest() { Clear(); };

  /** Returns true if good request */
  inline bool IfOk() const { return Ok; }

  /** Clears the current request buffer */
  inline void Clear() { if(Result) PQclear(Result); Result = 0; }

  /** Executes a SQL request
      @param c the sql request with a printf like syntax,
      where only the %c %s %d and %f are processed.
      It also exists a %b for boolean, which admits 0 or 1 as input and 
      produces 'f' or 't' into the final sql string
  */  
  bool Exec(const char *c, ...);

  /** Returns the number of tuples in the result of request */
  int Number();

  /** Returns the index of the given column
      @param c the column's name
  */
  int GetIndex(const char *c);

  /* Informs the request to pass to the next row */
  inline void Next() { tuple++; }

  /** Updates the comment field of a Data table of index idx
      @param comment the comment itself (Saved as Utf8)
      @param idx the row Idx
  */
  bool SetComment(const AstroString& comment, Index idx);

  /** Returns the content of the column of index idx
      @return the value as string, even for the numbers
  */
  char* Get(Index idx);

  /** Returns the AstroString value of the field whose the name is c */
  AstroString operator < (const char* c);

  /** Returns the integer value of the field whose the name is c */
  int GetInt(const char *c);

  /** Returns the double value of the field whose the name is c */
  double GetDouble(const char *);

  /** Returns the character value of the field whose the name is c */
  char GetChar(const char *);

  /** Returns the boolean value of the field whose the name is c */
  bool GetBool(const char *);

  /** Returns the unique row id of the last INSERTed row */
  Index GetINSId();

private:

  AstroSQL* Asql;
  PGresult* Result;
  int tuple;
  bool Ok;
};
#endif
