/***************************************************************************
                          astrostring.h  -  description
                             -------------------
    begin                : Sat Sep 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROSTRING_H
#define ASTROSTRING_H

#include "../config.h"

#ifdef KDE
#define QT
#endif

#ifdef QT
# include <qstring.h>
#else
# include <g++/String.h>
#endif

extern "C"
{
# include <stdio.h>
};

// Just a convenience, since = (int) and += (int) don't work directly
const char *rebuf(int i);

/** AstroString is a portable, unicode and templated string class */

 template <class S>  class _AstroString : public S
 {
 public:
  inline _AstroString() { *this = ""; }
  inline _AstroString(const char *c)
  {
    S& s = *this;
    s = c;
  }
  inline _AstroString& operator = (const char* c)
  {
    S& s = *this;
    s = c;
    return *this;
  }

  /** returns the string length */
  inline int Len() const
  {
#   ifdef QT
    return length();
#   else
    return length();
#   endif
  }
  /** returns the content of the string
   * @return the const char* pointer to the string
   */
  inline const char* Data() const
  {
# ifdef QT
    const S& s = *this;
    return (const char*)s.data();
# else
    return (const char*)*this;
# endif
  }

  /** puts an integer converted into a string into an _AstroString */
  inline _AstroString<S>& operator = (int i)
  {
    S& s = *this;
    s = rebuf(i);
    return *this;
  }

  /** adds an integer converted into a string to an _AstroString */
  inline _AstroString<S>& operator += (int i)
  {
    *this += rebuf(i);
    return *this;
  }
  /** adds a string to an _AstroString */
  inline _AstroString<S>& operator += (const char* c)
  {
    S& s = *this;
    s += c;
    return *this;
  }

  /** adds a character to an _AstroString */
  inline _AstroString<S>& operator += (char c)
  {
    S& s = *this;
    s += c;
    return *this;
  }

  /** adds an _AstroString to an _AstroString */
  inline friend _AstroString<S> operator + (const _AstroString& as1, const _AstroString<S>& as2)
  {
    _AstroString<S> s = as1;
    s += as2;
    return s;
  }

  inline friend _AstroString<S> operator + (const _AstroString<S>& as1, const char *s1)
  {
    _AstroString<S> s = as1;
    s += s1;
    return s;
  }

  inline friend _AstroString<S> operator + (const char* s1, const _AstroString<S>& s2)
  {
    _AstroString<S> s = s1;
    s += s2;
    return s;
  }

  inline friend _AstroString<S> operator + (const _AstroString<S>& s1, int i)
  {
    _AstroString<S> s = s1;
    s += i;
    return s;
  }
  
  inline friend _AstroString<S> operator + (int i, const _AstroString<S>& s2)
  {
    _AstroString<S> s = i;
    s += s2;
    return s;
  }

  /** adds an AstroString to an AstroString */
  inline _AstroString<S>& operator += (const _AstroString& as)
  {
    S& s = *this;
    s += (S)as;
    return *this;
  }

  inline const _AstroString<S> Char(int i)
  {
# ifdef QT
    return (const _AstroString<S>)S::mid(i, 1);
# endif
  }

  inline const _AstroString<S> Left(int i) const
  {
# ifdef QT
    return (const _AstroString<S>)S::left(i);
# endif
  }

  /** Returns the Unicode utf8 coded string */
  inline _AstroString<S> Utf8() const
  {
# ifdef QT
    return (const char *)S::utf8();
# endif
  }

  /** Initializes with an unicode utf8 codes string */
  inline void Utf8(const char *c)
  {
# ifdef QT
    *this = QString::fromUtf8(c);
# endif
  }

  /** Returns the ith character into an _AstroString */
  inline const _AstroString<S> operator[](int i)
  {
# ifdef QT
    return (const _AstroString<S>)S::mid(i, 1);
# endif
  }
 };

/** The real AstroString string class */
#ifdef QT
  typedef _AstroString<QString> AstroString;
#else
  typedef _AstroString<String>  AstroString;
#endif
 
#endif

