/***************************************************************************
                          astroutils.cpp  -  description
                             -------------------
    begin                : Fri Sep 15 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astroutils.h"

#ifdef KDE
#undef i18n
#include <klocale.h>
#include <qasciidict.h>
#include <qregexp.h>
#include <kmessagebox.h> 
#endif

extern "C"
{
# include <stdlib.h>
# include <ctype.h>
# include "sweodef.h"
# include "swephexp.h"
# include "sweph.h"
# include "swephlib.h"
};

#ifdef KDE
AstroString *Translate(const char *c)
 {
   static QAsciiDict<AstroString> Dcs;
   AstroString *acs = Dcs[c];
   if(!acs)
   {
     acs = new AstroString(i18n(c));
     Dcs.insert(c, acs);
   }
   return acs;
}
#endif

void PackString(char *s)
{
  char *c;
  for(c = s; *s; s++)
  {
    if(*s != ' ')
      *c++ = tolower(*s);
  }
  *c = 0;
}

bool in(char c, const char *s)
{
  const char *p;
  if(!s)
    return FALSE;
  for(p = s; *p; p++)
    if(*p == c)
      return TRUE;
  return FALSE;
}

int GetNum(char **s, int i1, int i2, bool more, const char *seps = 0)
{
  char buf[4], *ptr = *s, *s1 = *s, c;
  int i = 0, j = 0, result;
  if(more && seps)
    while(c = *(s1++))
      if(!isdigit(c) && !in(c, seps) && j++ < 2)
        return UTILS_NUMERROR;
  while(*ptr && isdigit(*ptr) && i < 3)
    buf[i++] = *ptr++;
  buf[i] = 0;
  result = atoi(buf);
  if(result < i1 || result > i2)
    return UTILS_NUMERROR;
  else
  {
    *s = ptr;
    return result;
  }
}

double DegToDec(double d)
{
  return SIGN(d)*(floor(fabs(d))+FRACT(fabs(d))*100.0/60.0);
}

double DecToDeg(double d)
{
  return SIGN(d)*(floor(fabs(d))+FRACT(fabs(d))*60.0/100.0);
}

double Normalize(double x)
{
  /*double d;
    if(x > 359.59)
    {
    d = floor(x / 360.0);
    x -= (d * 360.0);
    }
    return x;*/
  if(x < 0.0)
    x = 360.0 + floor(x) + FRACT(x);
  return swe_degnorm(x);
}

void Split(const AstroString& str, const AstroString& pat, AstroStringList* asl)
{
#ifdef KDE
  asl->Clear();
  int i = 0, j;
  while((j = str.find(pat, i)) != -1)
  {    
    asl->Append(new AstroString(str.mid(i, j - i)));
    i = j + pat.length();
  }
  asl->Append(new AstroString(str.mid(i, str.length() - i)));
#endif
}

const char *rebuf(int i)
{
  static char buf[12];
  (void)sprintf(buf, "%d", i);
  return buf;
}

bool AstroQuestion(const AstroString& quest, AstroString* yes = 0, AstroString* no = 0)
{
#ifdef KDE
  return (KMessageBox::questionYesNo(0, quest, QString::null) == KMessageBox::Yes) ? TRUE : FALSE;
#endif
}

void AstroMessage(const AstroString& s)
{
#ifdef KDE
  KMessageBox::information(0, (QString)s);
#endif
}

#define REST *__restrict__

AstroString& Sprintf(const AstroString& format, AstroString* abuf, ...)
{
  int i;
  double *fptr;
  AstroString u = format.Utf8(), v, *aptr;
  char buf[2048], buf1[40], *y, **x = (char **)&abuf, *ptr, *ptr1, *c = (char *)u.Data();
  x++;
  for(ptr = c, ptr1 = buf; *ptr; ptr++)
    if(*ptr != '%')
      *ptr1++ = *ptr;
    else
    {
      ptr++;
      switch(*ptr)
      {
        case 'c' : *ptr1++ = (int)*x;
	  x++;
	  break;
        case 's' : y = (char *)*x;
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
        case 'd' : i = (int)*x;
	  y = buf1;
	  sprintf(buf1, "%d", i);
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
        case 'f' : fptr = (double *)x;
	y = buf1;
	sprintf(buf1, "%f", *fptr);
	  while(*y)
	    *ptr1++ = *y++;
	  x += 2;
	  break;
        case 'a': aptr = (AstroString *)*x;
	  v = aptr->Utf8();
	  y = (char *)v.Data();
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
      };
    }
  *ptr1 = 0;
  abuf->Utf8(buf);
  return *abuf;
}

double IncTime(double julday, int minutes, int hours, int days, int months, int years)
{
  double d = floor(julday), r = julday - d, k, kd;
  bool b = FALSE;
  if(r >= 0.0 && r < 0.5)
  {
    b = TRUE;
    r += 0.5;
  }
  else
    r -= 0.5;
  k = minutes * ONEMINUTE + (hours + (6 * years)) * (60 * ONEMINUTE);
  k += r;
  kd = floor(k);
  r = k - kd + (b ? -0.5 : 0.5);
  d += kd;
  return d + r + days + (30 * months) + (365 * years) + (ONEMINUTE / 65);
}

double Progress(double julday, int years, int months, int days)
{
  int m = (months * HOURSINYEAR) / 12, k = MINSINDAY * (HOURSINYEAR * years + m + days * 24) / HOURSINYEAR;
  m = years * 12 + months;
  return IncTime(julday, k + m / 3, 0);
}

double Direct(double julday, int years, int months)
{
  int m = (months * HOURSINYEAR) / 12, k = (4 * 60) * (HOURSINYEAR * years + m) / HOURSINYEAR;
  return IncTime(julday, k / 60, 0);
}

void Diffs(double j, double j1, int& days, int& hours, int& minutes)
{
  int nb;
  bool b = FALSE, b1 = FALSE;
  double d = floor(j), d1 = floor(j1), r = j - d, r1 = j1 - d1, dr;
  if(r >= 0.0 && r < 0.5)
  {
    b = TRUE;
    r += 0.5;
  }
  else
    r -= 0.5;
  if(r1 >= 0.0 && r1 < 0.5)
  {
    b1 = TRUE;
    r1 += 0.5;
  }
  else
    r1 -= 0.5;
  days = (int)fabs(d - d1);
  dr = fabs(r - r1);
  nb = (int)(dr / ONEMINUTE);
  hours = nb / 60;
  minutes = nb % 60;
}
