/***************************************************************************
                          astroutils.h  -  description
                             -------------------
    begin                : Fri Sep 15 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrodefs.h"
#include "astroparams.h"
#include "astrostring.h"
#include "astrolist.h"

#define UTILS_NUMERROR 10000

/** Returns a AstroString translation of a given char *.
    The given translation are kept in a QAsciiDict
*/

AstroString *Translate(const char *);

/**
*  Suppress white chars from string and turn all the upper to lower cases
*  @param s The string pointer
**/

void PackString(char*);

/**
* return true if char c is in string s
*/
bool in(char c, const char *s);

/**
* Get a number from pointer **s, check if between i1 and i2
* strict check if more = TRUE, otherwise, an empty value is tolerated
* **s is incremented to the next potential digit character
* @return the number or UTILS_NUMEROR
* @param s pointer to pointer to string
* @param i1 lower bound
* @param i2 upper bound
* @param more thorough check
* @param seps the separators list
*/
int GetNum(char **s, int i1, int i2, bool more, const char *seps = 0);

/** Convert an inputed fractional degrees/minutes value to a true decimal
 degree quantity. For example, the user enters the decimal value "10.30"
 to mean 10 degrees and 30 minutes; this will return 10.5, i.e. 10
 degrees and 30 minutes expressed as a floating point degree value.      */

double DegToDec(double);

/** This is the inverse of the above function. Given a true decimal value
 for a zodiac degree, adjust it so the degrees are in the integer part
 and the minute expressed as hundredths, e.g. 10.5 degrees -> 10.30    */

double DecToDeg(double);

/** Returns a normalized angle. iF the given angle is greater than 359.59
	returns the corresponding value within the range 0.0 .. 359.59 */

double Normalize(double x);

/** This function splits the string str according to the pattern pat, and fills up the given AstroStringList with the plitted str parts.
@param str the source string
@param pat the split pattern
@param asl the AstroStringList
*/
void Split(const AstroString& str, const AstroString& pat, AstroStringList* asl);

void AstroMessage(const AstroString&);

bool AstroQuestion(const AstroString& quest, AstroString* yes = 0, AstroString* no = 0);

AstroString& Sprintf(const AstroString& format, AstroString* buf, ...);

void Diffs(double j, double j1, int& days, int& hours, int& minutes);

double IncTime(double julday, int minutes, int hours, int days = 0, int months = 0, int years = 0);

double Progress(double julday, int years, int months, int days);

double Direct(double julday, int years, int months);
