/***************************************************************************
                          astrowindow.cpp  -  description
                             -------------------
    begin                : Sat Dec 30 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrowindow.h"
#include "stdwheelchart.h"
#include "astroresources.h"
#include "astroprocess.h"
#include "astrointerface.h"
#include "astromainwindow.h"
#include "astrorestrictions.h"
#include "astrosql.h"

#ifdef KDE
#include <qpainter.h>
#include <qpen.h>
#include <qfont.h>
#include <kprinter.h>
#include <qpicture.h>
#include <qworkspace.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qdragobject.h>
#include <qpaintdevicemetrics.h>
#endif

extern AstroMainWindow* Amw;

static char *BtnLabel[] = {" Data:", " Subtype:" };


AstroWindow::AstroWindow(const AstroMainWindow* amw, AstroData* ad1, AstroData* ad2 = 0, AstroData* ad3 = 0, AstroData* ad4 = 0): QScrollView(amw->Qws), AstroGraphicChart(Asr->Width, Asr->Height)
{
  int vsbw, hsbh;
  AstroString as;
  setAcceptDrops(TRUE); 
  QTim = 0;
  Day = Month = Year = Hour = 0;
  Minute = Rate = 1;
  MilliSec = 1000;
  Backward = FALSE;
  Jbt = Minutes;
  SetData(ad1, 0);
  if(ad2)
    SetData(ad2, 1);
  if(ad3)
    SetData(ad3, 2);
  if(ad4)
    SetData(ad4, 3);
  Width = GetWidth();
  Height = GetHeight();
#ifdef KDE
  resizeContents(Width, Height);
  Qw = viewport();
  Qw->setMinimumSize(100, 100);
  Qw->setSizeIncrement(Width / 10, Height / 10); 
  vsbw = 30;//Asv.verticalScrollBar()->width();
  hsbh = 30; //Asv.horizontalScrollBar()->height();
  emit resize(Width + vsbw, Height + hsbh);
  Qp = new QPainter;
  Pen = new QPen;
  astro = new QFont(ASTROFONT);
  CHECK_PTR(astro);
  astro->setCharSet(QFont::Latin1);
  text = new QFont(TEXTFONT);
  CHECK_PTR(text);
  QPict = new QPicture;
  GetTitle(as);
  setCaption(as);
  connect(this, SIGNAL(Close(AstroWindow *)), amw, SLOT(Close(AstroWindow *)));
  Repaint();
  show();
#endif
}

AstroWindow::~AstroWindow()
{
}

void AstroWindow::closeEvent(QCloseEvent* e)
{
  QScrollView::closeEvent(e);
  emit Close(this);
}

#ifdef KDE
void AstroWindow::drawContentsOffset(QPainter *p, int ox, int oy, int cx, int cy, int cw, int ch )
{
  p->translate((float)-ox, (float)-oy);
  Redraw(p);
}

void AstroWindow::resizeEvent(QResizeEvent* qre)
{
  if(Cbp->DynamicResize)
  {
    int nw = (int)((double)OrigWidth * (double)qre->size().width() / (double)qre->oldSize().width()), nh = (int)((double)nw * (double)OrigHeight / (double)OrigWidth);
    Resize(nw, nh);
    int ow = OrigWidth;
    OrigWidth = nw;
    OrigHeight = nh;
    Repaint(TRUE);
    resizeContents(nw, nh);
  }
  QScrollView::resizeEvent(qre);
}
 
void AstroWindow::dragEnterEvent(QDragEnterEvent* event)
{
   event->accept(QTextDrag::canDecode(event));
}

void AstroWindow::dropEvent(QDropEvent* event)
{
  QString text, as("^"), as1;
  ButtonType bt = Undefined_Button;
  if(QTextDrag::decode(event, text))
  {
    int i;
    as += PACKAGE;
    QRegExp test(as);
    if(test.match(text) != -1)
    {
      test = as + " Data:";
      if(test.match(text) != -1) bt = DataButton;
      test = as + " Restrict:";
      if(test.match(text) != -1) bt = RestrictButton;
      test = as + " Subtype:";
      if(test.match(text) != -1) bt = SubTypeButton;
      if(bt != Undefined_Button)
      {
			text = text.right(text.length() - text.find(':') - 1);
			i = text.toInt();
			switch(bt)
			{
     			case DataButton:
	    			SetData(Adl->Get(i), Amw->Ring - 1); break;
	  			case RestrictButton:
	   			SetRestrict(Arl->Get(i), Amw->Ring - 1); break;
	  				case SubTypeButton:
	    			ManageSubTypes(i); break;
			}
			Repaint();
      }
    }
  }
}
#endif

void AstroWindow::ManageSubTypes(int i)
{
  AstroObjs* a;
  AstroString as;
  int r = Amw->Ring -1;
  switch(i)
  {
    case Show_Hide:
      if(!SetSubType(Show_Hide, r))
	AstroMessage(i18n("Sorry, Can't hide this ring !"));
      break;
    case Single:
    case Transit:
    case Direction:
    case Progression:
      if(!SetSubType((enum ChartSubType)i, r))
	AstroMessage(Sprintf(i18n("Sorry, Can't set this subtype ring to %a"), &as, Asr->SubTypesNames[i]));
      break;
    default:;
  }
}

AstroGraphicChart::AstroGraphicChart(int width, int height): AstroGraphics(0, 0, 0, width, height), AstroChart()
{
 Cbp->Ag = Base = this; 
 ct = StdWheels;
 Dm = GraphicMode;
 DblBuf = 0;
 Kp = 0;
 Width  = OrigWidth = width;
 Height = OrigHeight = height;
 PosX = PosY = 0;
 Size = Width / 50;
}

void AstroGraphicChart::SetDisplayMode(enum DisplayMode dm)
{
  Dm = dm;
  switch(Dm)
  {
    case GraphicMode:
      if(DblBuf) { delete DblBuf; DblBuf = 0; }
      if(Kp)     { delete Kp; Kp = 0; }
      break;
    case DblBufMode:
      if(Kp)     { delete Kp; Kp = 0; }
      DblBuf = new QPixmap(Width, Height);
      CHECK_PTR(DblBuf);
      DblBuf->setOptimization(QPixmap::BestOptim);
      break;
    case PrintMode:
      if(DblBuf) { delete DblBuf; DblBuf = 0; }
      Kp = new KPrinter;
      CHECK_PTR(Kp);
      if(!Kp->setup())
      {
	Dm = GraphicMode;
	delete Kp;
	Kp = 0;
      }
  };
}

void AstroGraphicChart::BackgroundColor()
{
#ifdef KDE
  Qw->setBackgroundColor(*COL.Back);
#endif
}

void AstroGraphicChart::Repaint(bool b)
{
  int w, h;
  AstroChart::Repaint(b);
#ifdef KDE
  switch(Dm)
  {
    case  GraphicMode:
      delete QPict;
      QPict = new QPicture;
      Qp->begin(QPict);
      BackgroundColor();
      Display();
      Qp->end();
      break;
    case DblBufMode:
      DblBuf->fill(*COL.Back);
      Qp->begin(DblBuf);
      Display();
      Qp->end();
      break;
    case PrintMode:
      w = GetWidth();
      h = GetHeight();
      Resize(OrigWidth * PRINTZOOMFACT, OrigHeight * PRINTZOOMFACT);
      Qp->begin(Kp);
      QPen& pn = (QPen&)Qp->pen();
      pn.setWidth(BOLDLINE);
      PrinterSize(w, h);
      Qp->setWindow(0, 0, w * PRINTZOOMFACT, h * PRINTZOOMFACT);
      Display();
      Qp->end();
      Resize(OrigWidth, OrigHeight);
      break;
  };
#endif
}

void AstroGraphicChart::PrinterSize(int w, int h)
{
  QPaintDeviceMetrics qpd(Kp);
  int rw = qpd.width();
  Qp->setViewport(0, 0, rw, (int)(((double)rw * (double)h) / (double)w));
}

void AstroWindow::Repaint(bool b)
{
  int w, h, s;
  AstroString as;
  switch(Dm)
  {
    case GraphicMode:
      SetWidth(Asr->Width);
      SetHeight(Asr->Height);
      AstroGraphicChart::Repaint(b);
      resizeContents(GetWidth(), GetHeight());
      viewport()->update();
      GetTitle(as);
      setCaption(as);
      break;
    case DblBufMode:
      AstroGraphicChart::Repaint(b);
      bitBlt(viewport(), 0, 0, DblBuf, 0, 0, viewport()->width(),
	      viewport()->height(), CopyROP, TRUE);
      break;
    case PrintMode:
      AstroGraphicChart::Repaint(TRUE);
  };
}

void AstroWindow::Anim(bool start)
{
  if(start)
  {
    SetDisplayMode(DblBufMode);
    QTim = new QTimer;
    CHECK_PTR(QTim);
    connect(QTim, SIGNAL(timeout()), this, SLOT(TimerSlot()));
    QTim->start(MilliSec, FALSE);
  }
  else
  {
    QTim->stop();
    SetDisplayMode(GraphicMode);
    delete QTim;
    QTim = 0;
  }
}

void AstroWindow::TimerSlot()
{
  int s = Backward ? -1 : 1;
  AstroObjs& ao = *(*this)[Amw->Ring - 1];
  ao.AstroDate::Recompute();
  double j = IncTime(ao.GetJd(), s * Minute, s * Hour, s * Day, s * Month, s * Year);
  ao.SetJd(j);
  ao.Recompute();
  Repaint();
}

void AstroWindow::SetJbt(enum JumpBaseType jbt)
{
  Jbt = jbt;
  Day = Month = Year = Hour = Minute = 0;
  switch(Jbt)
  {
    case Minutes:   Minute = Rate; break;
    case Hours:     Hour   = Rate; break;
    case Days:      Day    = Rate; break;
    case Months:    Month  = Rate; break;
    case Years:     Year   = Rate; break;
    case Centuries: Year   = 100 * Rate; break;
    case Millenia:  Year   = 1000 * Rate; break;
  };
}

void AstroWindow::SetRate(int r)
{
  Rate = r;
  switch(Jbt)
  {
    case Minutes:   Minute = Rate; break;
    case Hours:     Hour   = Rate; break;
    case Days:      Day    = Rate; break;
    case Months:    Month  = Rate; break;
    case Years:     Year   = Rate; break;
    case Centuries: Year   = 100 * Rate; break;
    case Millenia:  Year   = 1000 * Rate; break;
  };
}

AstroIconsList::AstroIconsList()
{
  AstroIcon* ai;
  AstroString name, file;
  AstroRequest Arq;
  int i;
  if(!Arq.Exec("SELECT * FROM TypeIcons"))
  {
    AstroMessage(i18n("Can't load Type icons"));
    exit(1);		 
  }
  
  name = i18n("Now chart");
  ai = new AstroIcon("now.png", &name);
  Append(ai);

  for(i = 0; i < Arq.Number(); i++, Arq.Next())
  {
    name = Arq < "Name";
    file = Arq < "File";
    ai = new AstroIcon(file.Data(), &name);
    Append(ai);
  }
}

AstroIconsWindow::AstroIconsWindow(AstroMainWindow *amw) : QListView(amw->centralWidget())
{
  int i;
  Idx = 0;
  FirstXY = IsPressed = FALSE;
  Aib = 0;
  const AstroPixmap& gp = *Asr->Pixmaps[0];
#ifdef KDE
  setBackgroundMode( QWidget::PaletteBase );
  setRootIsDecorated(TRUE);
  addColumn(i18n("Data"));
  Data = new QListViewItem(this, i18n("Data sets"));
  Restrictions = new QListViewItem(this, i18n("Restrictions sets"));
  SubTypes = new QListViewItem(this, i18n("Subtypes"));
  Data->setOpen(TRUE);
  Data->setPixmap(0, gp);
  Restrictions->setPixmap(0, gp);
  SubTypes->setOpen(TRUE);
  SubTypes->setPixmap(0, gp);
  DataPm = new QPopupMenu(this);
  DataPm->insertItem(i18n("Edit"), 1);
  DataPm->insertItem(i18n("Save"), 2);
  DataPm->insertItem(i18n("Delete"), 3);
  DataPm->insertItem(i18n("Duplicate"), 4);
  RestPm = new QPopupMenu(this);
  RestPm->insertItem(i18n("Edit"), 1);
  RestPm->insertItem(i18n("Save"), 2);
  RestPm->insertItem(i18n("Delete"), 3);
  RestPm->insertItem(i18n("Duplicate"), 4);
  connect(this, SIGNAL(clicked(QListViewItem*)), SLOT(Clicked(QListViewItem*)));
  connect(this, SIGNAL(mouseButtonPressed(int, QListViewItem*, const QPoint&, int)), SLOT(Pressed(int, QListViewItem*, const QPoint&, int)));
  connect(this, SIGNAL(doubleClicked(QListViewItem*)), SLOT(DoubleClicked(QListViewItem*)));
  connect(DataPm, SIGNAL(activated(int)), this, SLOT(Activated(int)));
  connect(RestPm, SIGNAL(activated(int)), this, SLOT(Activated(int)));
#endif
  for(i = Show_Hide; i <= Progression; i++)
    (void)new AstroIconButton(this, i);
  AstroIconButton* ab = new AstroIconButton(this, Adl->First());
  AstroIconButton* ar = new AstroIconButton(this, Drs);
}

void AstroIconsWindow::AddData(const AstroData* ad)
{
  Adl->AddData((AstroData*)ad);
  AstroIconButton* ab = new AstroIconButton(this, ad);
}

void AstroIconsWindow::AddRestrict(const AstroRestrictions* ar)
{
  Arl->AddRestrictions((AstroRestrictions*)ar);
  AstroIconButton* ab = new AstroIconButton(this, ar);
}

void AstroIconsWindow::DeleteCurrent()
{
  switch(Aib->Bt)
  {
    case DataButton:
    	if(Idx != 0)
    	{
      	Data->takeItem(Aib);
      	Adl->RemoveFromDb(Idx);
      	delete Aib;
  			Aib = 0;
      }
      else
      	AstroMessage(i18n("Can't remove this icon"));
      break;
    case RestrictButton:
    	if(Idx != 1)
    	{
      	Restrictions->takeItem(Aib);
      	Arl->RemoveFromDb(Idx);
      	delete Aib;
  			Aib = 0;
      }
      else
			AstroMessage(i18n("Can't remove this icon"));
      break;
    default:
      AstroMessage(i18n("Can't remove this icon."));
  }
}


void AstroIconsWindow::Clicked(QListViewItem *qlvi)
{
  IsPressed = FALSE;
  const AstroIconButton* aib = SC(AstroIconButton *, qlvi);
  if(aib)
    Idx = aib->Idx;
}

AstroIconsWindow::AstroIconButton::AstroIconButton(AstroIconsWindow* parent, const AstroData* ad) : QListViewItem(parent->Data)
{
  Idx = ad->Idx;
  Bt = DataButton;
#ifdef KDE
  setText(0, ad->Name);
  setPixmap(0, *(*Ail)[ad->ChartType]);
#endif
}

AstroIconsWindow::AstroIconButton::AstroIconButton(AstroIconsWindow* parent, const AstroRestrictions* ar) : QListViewItem(parent->Restrictions)
{
  Idx = ar->Idx;
  Bt = RestrictButton;
#ifdef KDE
  setText(0, ar->Name);
  setPixmap(0, *Asr->Pixmaps[1]);
#endif
}

AstroIconsWindow::AstroIconButton::AstroIconButton(AstroIconsWindow* parent, int index, ButtonType bt) : QListViewItem(bt == SubTypeButton ? parent->SubTypes : 0)
{
  Idx = index;
  Bt = bt;
  switch(Bt)
  {
    case SubTypeButton:
#ifdef KDE
      setText(0, (*Asl) << index);
      setPixmap(0, *Asr->Pixmaps[1]);
#endif
      break; 
    default:;
  };
}
void AstroIconsWindow::Pressed(int b, QListViewItem* item, const QPoint& qp, int c)
{
	AstroIconButton* aib = SC(AstroIconButton*, item);
	switch(b)
	{
		case 1: FirstXY = IsPressed = TRUE;
  			Aib = (item != Data && item != SubTypes) ? aib : 0;
  			break;
  		case 2:
			if(aib)
			{
				Idx = aib->Idx;
				switch(aib->Bt)
				{
					case DataButton: DataPm->exec(qp); break;
					case RestrictButton: RestPm->exec(qp); break;
					default:;
				};
			}
			break;
		}
}

void AstroIconsWindow::ChangeCurrent(const AstroData* ad)
{
  AstroIconButton* aib = SC(AstroIconButton*, currentItem());
#ifdef KDE
  aib->setText(0, ad->Name);
  aib->setPixmap(0,*(*Ail)[ad->ChartType]);
#endif 
}

void AstroIconsWindow::ChangeCurrent(const AstroRestrictions* ar)
{
  AstroIconButton* aib = SC(AstroIconButton*, currentItem());
#ifdef KDE
  emit aib->setText(0, ar->Name);
#endif
}

void AstroIconsWindow::contentsMouseMoveEvent(QMouseEvent * e)
{
  if(!Aib || !IsPressed) return;
  if(FirstXY)
  {
    FirstXY = FALSE;
    X = e->x();
    Y = e->y();
  }
  else
  {
    int x = abs(X - e->x()), y = abs(Y - e->y());
    if(x > 5 || y > 5)
    {
      AstroString as(PACKAGE);
      switch(Aib->Bt)
      {
        case DataButton:     as += " Data:";     as += Aib->Idx; break;
        case RestrictButton: as += " Restrict:"; as += Aib->Idx; break;
        case SubTypeButton:  as += " Subtype:";  as += Aib->Idx; break;
        default:;
      };
      Idx = Aib->Idx;
#ifdef KDE
      QDragObject *drobj = new QTextDrag(as, this);
      drobj->dragCopy();
#endif
      IsPressed = FALSE;
    }
  }
} 

void AstroIconsWindow::Activated(int i)
{
	switch(i)
	{
		case 1: Amw->Edit(); break;
		case 2: Amw->Save(); break;
		case 3: Amw->Delete(); break;
		case 4: Amw->Duplicate(); break;
	}
}



void AstroIconsWindow::DoubleClicked(QListViewItem* qlvi)
{
	AstroIconButton* aib = SC(AstroIconButton*, qlvi);
	if(aib)
	{
		Idx = aib->Idx;
		Amw->Edit();
	}
}
