/***************************************************************************
                          astrowindow.h  -  description
                             -------------------
    begin                : Sat Dec 30 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROWINDOW_H
#define ASTROWINDOW_H

#include "../config.h"
#include "astrochart.h"
#include "astrographics.h"
#include "astrodata.h"

#ifdef KDE
#include <qscrollview.h>
#include <qlistview.h>
#include <qpopupmenu.h>

class QPainter;
class QPicture;
class KPrinter;
class QTimer;
class QPaintEvent;
class QMouseEvent;
class QCloseEvent;
#endif

class AstroInterface;
class AstroMainWindow;
class AstroRestrictions;

enum ButtonType { Undefined_Button, DataButton, RestrictButton, SubTypeButton };

/** A portable Icon class */
class AstroIcon: public AstroPixmap
{
    
 public:

  /** Contructor */
  inline AstroIcon(const char* name, const AstroString* as) : AstroPixmap(name) { Name = (AstroString)*as; }

  // Icon name
    AstroString Name;
};

/** A list of icons */
class AstroIconsList: public AstroList(AstroIcon)
{
 public:

  /** Constructor */
  AstroIconsList();

};


/** This class is the standard class foe the graphical charts (that can be displayed in a window) */
class AstroGraphicChart : public AstroGraphics, public AstroChart
{
 public:

  enum DisplayMode { GraphicMode, DblBufMode, PrintMode };

  /** Constructor
   @param width the charts's width
   @param height the chart's height
  */
  AstroGraphicChart(int width, int height);

  /** the overloaded Repaint() method */
  void Repaint(bool b = FALSE);

  /** Sets the chart's background color */
  void BackgroundColor();
  
  /** Select the display mode, graphic, anim or printing mode */
  void SetDisplayMode(enum DisplayMode);

  /** Invoked when printing or window resizing 
      @param x new width
      @param y new height
  */
  inline void Resize(int x, int y) { SetWidth(x); SetHeight(y);  Size = AstroGraphics::Width/ 50; }

#ifdef KDE
  inline void Redraw(QPainter *qp) { qp->drawPicture(*QPict); }
#endif

 protected:

  int OrigWidth, OrigHeight;

  enum DisplayMode Dm;

  void PrinterSize(int w, int h);

#ifdef KDE
  QWidget *Qw;
  QPicture *QPict;
  QPixmap* DblBuf;
  KPrinter* Kp;
#endif
};

/** This class manages the  window in which a chart is displayed */
class AstroWindow:
public QScrollView, 
public AstroGraphicChart
{
  Q_OBJECT

 public:

  friend class AstroMainWindow;

  /** Constructor
  @param ad1 the first data for the chart
  @param ad2 ad3 ad4 possible 2nd 3rd and 4th data for the chart
  */
  AstroWindow(const AstroMainWindow*, AstroData* ad1, AstroData* ad2 = 0, AstroData* ad3 = 0, AstroData* ad4 = 0);

  /** Destructor */
  ~AstroWindow();

  enum JumpBaseType { Minutes, Hours, Days, Months, Years, Centuries, Millenia };
  /** The overloded Repaint method */
  void Repaint(bool b = FALSE);

  /** Displays a chart params dialog box */
  inline bool EditParms() { return Cbp->Edit(); }

  /** Start or stop the animation */
  void Anim(bool start);

  // Used by anims
  int Day, Month, Year, Hour, Minute, Rate;

  // If anim runs backward
  bool Backward;

  // Type of anim jump
  enum JumpBaseType Jbt;

 private:

  void ManageSubTypes(int i);
  void SetJbt(enum JumpBaseType);
  void SetRate(int r);

#ifdef KDE
  QTimer* QTim;
  void drawContentsOffset(QPainter *p, int ox, int oy, int cx, int cy, int cw, int ch );
  void resizeEvent(QResizeEvent*); 
  void dragEnterEvent(QDragEnterEvent*);
  void dropEvent(QDropEvent*);
  void closeEvent(QCloseEvent*);
#endif
  int MilliSec;

  // The window's width and height
  int Width, Height;

#ifdef KDE
 signals:

  void Close(AstroWindow *);

 private slots:

  void TimerSlot();
#endif 
};

  /** The lateral windows diplaying the icons tree */
class AstroIconsWindow:
public QListView
{
 Q_OBJECT

 public:

 friend class AstroMainWindow;

 /** A class for the icon buttons */
 friend class AstroIconButton 
#ifdef KDE
   : public QListViewItem
#endif
 {
   public: 

   /** Constructor
       @param index, ith button of the corresponding subtype branch of the tree
       @param bt subtype branch of the tree
   */
    AstroIconButton(AstroIconsWindow*, int index, ButtonType bt = SubTypeButton);

    /** Constructor that adds a data icon */
    AstroIconButton(AstroIconsWindow*, const AstroData* ad);

    AstroIconButton(AstroIconsWindow*, const AstroRestrictions* ar);

    // index database (if any) of the data
    int Idx;

    // Type of button 
    ButtonType Bt;
 };

 /** Constructor */
 AstroIconsWindow(AstroMainWindow*);

  /** Destructor */
  ~AstroIconsWindow() { delete DataPm; delete RestPm; }

 /** Returns the data of the clicked icon */  
 inline AstroData* GetData() const { return Adl->Get(Idx); }

 /** Returns the restrictions set of the clicked icon */
 inline AstroRestrictions* GetRestrictions() const {  return Arl->Get(Idx); }

   /** Adds a new data icon */
   void AddData(const AstroData*);

   /** Adds a new restrictions set icon */
   void AddRestrict(const AstroRestrictions*);

   /** Delete the clicked data icon and value */
   void DeleteCurrent();

   /** Changes the data of the clicked icon data */
   void ChangeCurrent(const AstroData* ad);

   /** Changes the data of the clicked icon restrictions set */
   void ChangeCurrent(const AstroRestrictions* ar);

 private:

#ifdef KDE
   QListViewItem *Data, *Restrictions, *SubTypes;
   QPixmap* qpm;
   QPopupMenu *DataPm, *RestPm;
   void contentsMouseMoveEvent(QMouseEvent *);
#endif

   // Active button + array for the subtypes icon buttons 
   AstroIconButton *Aib;
   Index Idx;
   bool FirstXY, IsPressed;
   int X, Y;

#ifdef KDE
 private slots:

   void Pressed(int, QListViewItem*, const QPoint&, int);
   void Clicked(QListViewItem*);
	void DoubleClicked(QListViewItem *);
	void Activated(int);
#endif
};

#endif



