#include "editastrodata.h"
#include "astrodata.h"
#include <qdatetime.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <mydatepick.h>
#include <knuminput.h>
#include <qradiobutton.h>
#include <kfiledialog.h>
#include <qlabel.h>
#include "astrowindow.h"
#include <qcheckbox.h>
#include "mylineedit.h"
#include "editplace.h"

extern AstroIconsList* Ail;

/* 
 *  Constructs a EditAstroData which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */

EditAstroData::EditAstroData( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : _EditAstroData( parent, name, modal, fl )
{
}

EditAstroData::EditAstroData(AstroData& ad) : _EditAstroData(0, "EditAstroData", TRUE, 0)
{
  AdBase = new AstroData(ad);
  Ad = &ad;
  int h, m, day, month, year;
  bool corr = AdBase->Corrected;
  AstroIcon* ai;
  AdBase->GetDate(&day, &month, &year, &h, &m);
  const QDate qd(year, month, day);
  AdBase->Corrected = FALSE;
  Date->setDate(qd);
  if(Ad->Idx == 0)
  {
	emit Date->setEnabled(FALSE);
  	emit Minute->setEnabled(FALSE);
  	emit Hour->setEnabled(FALSE);
  	emit Name->setEnabled(FALSE);
  	emit TypeofChart->setEnabled(FALSE);
  }
  for(ai = Ail->First(); ai; ai = Ail->Next())
    TypeofChart->insertItem(ai->Name);
  emit ChartTypeIcon->setPixmap(*(*Ail)[AdBase->ChartType]);
  emit Name->setText(AdBase->Name);
  emit Comment->setText(AdBase->Comment);
  switch(AdBase->Gender)
  {
    case Male:    GenderGroup->setButton(GenderGroup->id(MaleButton)); break;
    case Female:  GenderGroup->setButton(GenderGroup->id(FemaleButton)); break;
    case Neutral: GenderGroup->setButton(GenderGroup->id(NeutralButton)); break;
  };
  TypeofChart->setCurrentItem(AdBase->ChartType);
  emit Hour->setValue(h);
  emit Minute->setValue(m);
  PlaceEdition->Init(AdBase);
  AdBase->Corrected = corr;
  ActiveDaylightButton->setChecked(AdBase->Corrected);
  OffTzRecomp();
  Return = FALSE;
  connect(PlaceEdition, SIGNAL(OffTzRecomp()), this, SLOT(OffTzRecomp()));
}
/*  
 *  Destroys the object and frees any allocated resources
 */
EditAstroData::~EditAstroData()
{
  delete AdBase;
    // no need to delete child widgets, Qt does it all for us
}

/* 
 * public slot
 */
void EditAstroData::CancelSlot()
{
  close();
  Ad->Modified = FALSE;
  Return = FALSE;
}

void EditAstroData::OffTzRecomp()
{
  AstroString as;
  AstroHM hm(AdBase->GetOffset());
  DaylightButton->setChecked(AdBase->GetDaylight());
  hm.Get(as);
  emit TimezoneLine->setText(as);
}

/* 
 * public slot
 */
void EditAstroData::OkSlot()
{
  double d;
  if(Name->text().isEmpty())
  {
  	AstroMessage(i18n("You must provide a name to this data set"));
  	return;
  }
  if(PlaceEdition->Modified)
  {
    AstroMessage(i18n("You have forgotten to validate the modification of the place !"));
    return;
    }
  close();
  AstroHM hm;
  QButton* qbt;
  AdBase->Name = (AstroString)Name->text();
  AdBase->Comment = (AstroString)Comment->text();
  qbt = GenderGroup->selected();
  if(qbt == MaleButton)
    AdBase->Gender = Male;
  else if(qbt == FemaleButton)
    AdBase->Gender = Female;
  else
   AdBase->Gender = Neutral;
  AdBase->ChartType = TypeofChart->currentItem();
  PlaceEdition->UpdateAd(FALSE);
  GetTz(TimezoneLine->text(), d);
  AdBase->SetOffDay(d, AdBase->GetDaylight());
  AdBase->Modified = TRUE;
  *Ad = *AdBase;
  Return = TRUE;
}

void EditAstroData::DateSlot(QDate qd)
{
  int h, m;
  HMR(AdBase->Get(), h, m);
  AdBase->SetDate(qd.day(), qd.month(), qd.year(), h, m);
  if(AdBase->Corrected)
  	RecomputeSlot();
}


void EditAstroData::HourSlot(int hour)
{
  int h, m;
  HMR(AdBase->Get(), h, m);
  h = hour;
  AdBase->Set(h, m);
  if(AdBase->Corrected)
  	RecomputeSlot();
}

void EditAstroData::MinuteSlot(int min)
{
  int h, m;
  HMR(AdBase->Get(), h, m);
  m = min;
  AdBase->Set(h, m);
  if(AdBase->Corrected)
  	RecomputeSlot();
}

void EditAstroData::ChartTypeSlot(int i)
{
  emit ChartTypeIcon->setPixmap(*(*Ail)[i]);
}

void EditAstroData::ActiveDaylightSlot(bool b)
{
  AdBase->Corrected = b;
  AdBase->Modified = TRUE;
  if(AdBase->Modified)
  	RecomputeSlot();
}

void EditAstroData::DaylightSlot(bool b)
{
  AdBase->SetOffDay(AdBase->GetOffset(), b);
  AdBase->Modified = TRUE;
}

void EditAstroData::HMSLongSlot(bool b)
{
  AstroHM hm;
  AstroString as = (AstroString)PlaceEdition->Longitude->text();
  hm.Set(as, AdBase->HMS_Format ? HM_HMS : HM_Longitude);
  AdBase->HMS_Format = b;
  hm.Get(as, AdBase->HMS_Format ?  HM_HMS : HM_Longitude);  
  emit PlaceEdition->Longitude->setText(as);
  AdBase->Modified = TRUE;
}

int EditAstroData::GetTz(QString s, double& d)
{
  bool b = FALSE;
  int i;
  AstroHM hm;
  char c = s.ref(0);
  if(c == '-')
  {
    b = TRUE;
    s.remove(0,1);
  }
  i = hm.Set((AstroString)s);
  d = (b ? -1.0 : 1.0) * hm.Get();
  return i;
}

void EditAstroData::OffsetSlot(const QString& qs)
{
  AstroString r((QString)qs);
  double d;
  AdBase->Modified = TRUE;
  switch(GetTz(r, d))
  {
    case ASTROHM_WRONG_HOUR: 
      r = i18n("Error in offset hour"); break;
    case ASTROHM_WRONG_SEPARATOR:
      r = i18n("Error in hour/minute separator"); break;
    case ASTROHM_WRONG_MINUTE:
      r = i18n("Error in offset minute"); break;
    default: 
      r = "";
  }
  if(!r.isEmpty())
    AstroMessage(r);
  else
    AdBase->SetOffDay(d, AdBase->GetDaylight());
}

void EditAstroData::RecomputeSlot()
{
  AstroHM hm;
  AstroString as;
  bool b = AdBase->Corrected;
  AdBase->Corrected = TRUE;
  PlaceEdition->UpdateAd(TRUE);
  DaylightButton->setChecked(AdBase->GetDaylight());
  hm = AdBase->GetOffset();
  hm.Get(as);
  emit TimezoneLine->setText(as);
  AdBase->Modified = TRUE;
  AdBase->Corrected = b;
}

void EditAstroData::NowSlot()
{
  int d, mo, y, h, m;
  PlaceEdition->UpdateAd(FALSE);
  AdBase->Now();
  AdBase->GetDate(&d, &mo, &y, &h, &m);
  const QDate qd(y, mo, d);
  Date->setDate(qd);
  emit Hour->setValue(h);
  emit Minute->setValue(m);
  if(AdBase->Corrected)
    OffTzRecomp();
  AdBase->Modified = TRUE;
}

void EditAstroData::WriteError(const QString& s)
{
  emit Error->setText(s);
}
