#include "editcountry.h"
#include <qlineedit.h>
#include "mylineedit.h"
#include "astroutils.h"
#include "kfiledialog.h"
#include "astroresources.h"
#include "astrodata.h"
#include "astrochart.h"

/* 
 *  Constructs a EditCountry which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
EditCountry::EditCountry() : _EditCountry(0, 0, TRUE, 0)
{
  Idx = 0;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
EditCountry::~EditCountry()
{
    // no need to delete child widgets, Qt does it all for us
}

/* 
 * public slot
 */
void EditCountry::Clicked(QListViewItem* qlvi)
{
  CountryLine* cl = SC(CountryLine*, qlvi);
  if(!cl) return;
  emit Name->setText(cl->text(0));
  emit ZifName->setText(cl->text(1));
  Idx = cl->Idx;
}
/* 
 * public slot
 */
void EditCountry::DeleteSlot()
{
  if(Adl->CheckIdx(Idx, TRUE) || Ahl->CheckIdx(Idx, TRUE))
  {
    AstroMessage(i18n("Can't delete this country"));
    return;
  }
  if(!Arq.Exec("DELETE FROM Country WHERE Idx = %d", Idx))
  {
    AstroMessage(i18n("Can't delete this country"));
    return;
  }
  CountriesList->clear();
  emit Name->setText("");
  emit ZifName->setText("");
}

/* 
 * public slot
 */
void EditCountry::ModifySlot()
{
  const char *zif = ZifName->text().data();
  if(!Arq.Exec("UPDATE Country SET Name = '%s', TzFile = '%s' WHERE Idx = %d", (const char *)Name->text().utf8(), zif, Idx))
 {
   AstroMessage(i18n("Can't modify this country"));
   return;
 }
  Ahl->UpdateZif(Idx, zif);
}

/* 
 * public slot
 */
void EditCountry::NewSlot()
{
  Index idx;
  if(Name->text().isEmpty() || ZifName->text().isEmpty())
  {
    AstroMessage(i18n("Fields Name and  ZifName can't be empty !"));
    return;
  }
  Arq.Exec("INSERT INTO Country (Name, TzFile) VALUES ('%s', '%s')", (const char *)Name->text().utf8(), ZifName->text().data());
  idx = Arq.GetINSId();
  Arq.Exec("SELECT Idx FROM Country WHERE Oid = %d", idx);
  idx = Arq.GetInt("Idx");
  CountriesList->clear();
  CountriesList->insertItem(new CountryLine(idx, CountriesList, Name->text(), ZifName->text()));
  Idx = idx;
}
/* 
 * public slot
 */
void EditCountry::OkSlot()
{
  close(); 
}
/* 
 * public slot
 */
void EditCountry::SearchSlot()
{
  int i, n;
  CountriesList->clear();
  	if(Name->text().isEmpty())
  		Arq.Exec("SELECT Name, TzFile, Idx FROM Country WHERE Name LIKE '\\%' ORDER BY Name");
  	else
  		Arq.Exec("SELECT Name, TzFile, Idx FROM Country WHERE Name ~* '%s' ORDER BY Name", (const char *)Name->text().utf8());
  n = Arq.Number();
  for(i = 0; i < n; i++, Arq.Next())
    CountriesList->insertItem(new CountryLine(Arq.GetInt("Idx"), CountriesList, Arq < "Name" , Arq < "TzFile"));
}
/* 
 * public slot
 */
void EditCountry::FileListSlot()
{
  QString path(Asr->ZPath), res;
  res = KFileDialog::getOpenFileName(path, QString::null, 0, i18n("Zone info dir"));
  res.remove(0, path.length());
  if(!res.isEmpty())
    emit ZifName->setText(res);
}

