#include "editdefparms.h"
#include "astrohm.h"
#include "mylineedit.h"
#include "astrostring.h"
#include "astroutils.h"
#include "astrographics.h"
#include "qpushbutton.h"
#include <knuminput.h>
#include "editchartparms.h"
#include <kcolorbtn.h>
/*
 *  Constructs a EditDefParms which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
EditDefParms::EditDefParms(AstroParams* ap)
    : _EditDefParms(0, 0, TRUE, 0)
{
  int i;
  AstroString as;
  Ap =ap;
  ApBase = *Ap;
  Chosen = -1;
  Qppm.setCheckable(TRUE); 
  AstroHM hm = ApBase.ExtraTime;
  emit ZoneInfoPath->setText(ApBase.ZPath);
  hm.Get(as);
  for(i = 0; i < 16; i++)
  	Ac[i] = COL[i];
  emit ExtraTime->setText(as);
  emit Width->setValue(ApBase.Width);
  emit Height->setValue(ApBase.Height);
  emit FireColor->setPixmap(COL % ApBase.FireColor);
  emit AirColor->setPixmap(COL % ApBase.AirColor);
  emit WaterColor->setPixmap(COL % ApBase.WaterColor);
  emit EarthColor->setPixmap(COL % ApBase.EarthColor);
  emit AsteroColor->setPixmap(COL % ApBase.AsteroColor);
  emit ExoticColor->setPixmap(COL % ApBase.ExoticColor);
  emit OtherColor->setPixmap(COL % ApBase.OtherColor);
  Col0->setColor(Ac[0]);
  Col1->setColor(Ac[1]);
  Col2->setColor(Ac[2]);
  Col3->setColor(Ac[3]);
  Col4->setColor(Ac[4]);
  Col5->setColor(Ac[5]);
  Col6->setColor(Ac[6]);
  Col7->setColor(Ac[7]);
  Col8->setColor(Ac[8]);
  Col9->setColor(Ac[9]);
  Col10->setColor(Ac[10]);
  Col11->setColor(Ac[11]);
  Col12->setColor(Ac[12]);
  Col13->setColor(Ac[13]);
  Col14->setColor(Ac[14]);
  Col15->setColor(Ac[15]);
  connect(&Qppm, SIGNAL(activated(int)), this, SLOT(Activated(int)));
  for(i = 0; i < 16; i++)
    Qppm.insertItem(COL % i, i);
  Return = FALSE;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
EditDefParms::~EditDefParms()
{
    // no need to delete child widgets, Qt does it all for us
}

/* 
 * public slot
 */
void EditDefParms::AirColorSlot()
{
   ChooseColor(&ApBase.AirColor);
  ChangeButton(AirColor, Ac[ApBase.AirColor]);	
}
/* 
 * public slot
 */
void EditDefParms::AsteroColorSlot()
{
  ChooseColor(&ApBase.AsteroColor);
  ChangeButton(AsteroColor, Ac[ApBase.AsteroColor]);	
}
/* 
 * public slot
 */
void EditDefParms::EarthColorSlot()
{
  ChooseColor(&ApBase.EarthColor);
  ChangeButton(EarthColor, Ac[ApBase.EarthColor]);	
}
/* 
 * public slot
 */
void EditDefParms::ExoticColorSlot()
{
  ChooseColor(&ApBase.ExoticColor);
  ChangeButton(ExoticColor, Ac[ApBase.ExoticColor]);	
}
/* 
 * public slot
 */
void EditDefParms::FireColorSlot()
{
  ChooseColor(&ApBase.FireColor);
  ChangeButton(FireColor, Ac[ApBase.FireColor]);	
}
/* 
 * public slot
 */
void EditDefParms::OkButtonSlot()
{
	int i;
  AstroHM hm = (AstroString)ExtraTime->text();
  strncpy(ApBase.ZPath, ZoneInfoPath->text().data(), BUFSIZE);
  ApBase.ExtraTime = hm.Get();
  ApBase.Width = Width->value();
  ApBase.Height = Height->value();
	for(i = 0; i < 16; i++)
		Acl->SetColor(Ac[i], i);
  *Ap = ApBase;
  close();
  Return = TRUE;
}
/* 
 * public slot
 */
void EditDefParms::OtherColorSlot()
{
  ChooseColor(&ApBase.OtherColor);
  ChangeButton(OtherColor, Ac[ApBase.OtherColor]);
}
/* 
 * public slot
 */
void EditDefParms::WaterColorSlot()
{
  ChooseColor(&ApBase.WaterColor);
  ChangeButton(WaterColor, Ac[ApBase.WaterColor]);	
}
/* 
 * public slot
 */
void EditDefParms::CancelButtonSlot()
{
  close(); 
}

void EditDefParms::ChooseColor(int* ptr)
{
  int i;
  for(i = 0; i < 16; i++)
    Qppm.setItemChecked(i, FALSE);
  Qppm.setItemChecked(*ptr, TRUE);
  Qppm.exec(QCursor::pos());
  *ptr = Chosen;
}

void EditDefParms::Activated(int i)
{
  Chosen = i;
}

void EditDefParms::DefChartParmsSlot()
{
  EditChartParms ecp(&ApBase);
  ecp.show();
}

void EditDefParms::ChangeButton(QPushButton* pb, const QColor& c)
{
	QPixmap* qpm;
	qpm = (QPixmap *)pb->pixmap();
	emit qpm->fill(c);
	emit pb->update();
}

void EditDefParms::ChangeColor(const QColor& c, int i)
{
	AstroColor ac(c.red(), c.green(), c.blue());
	Ac[i] = ac;
	Qppm.pixmap(i)->fill(c);
	if(ApBase.FireColor == i) ChangeButton(FireColor, c);
	if(ApBase.AirColor == i) ChangeButton(AirColor, c);
	if(ApBase.WaterColor == i) ChangeButton(WaterColor, c);
	if(ApBase.EarthColor == i) ChangeButton(EarthColor, c);
	if(ApBase.AsteroColor == i) ChangeButton(AsteroColor, c);
	if(ApBase.ExoticColor == i) ChangeButton(ExoticColor, c);
	if(ApBase.OtherColor == i) ChangeButton(OtherColor, c);
}

void EditDefParms::Col0Slot(const QColor &c)
{
	ChangeColor(c, 0);
}

void EditDefParms::Col10Slot(const QColor &c)
{
	ChangeColor(c, 10);
}

void EditDefParms::Col11Slot(const QColor &c)
{
	ChangeColor(c, 11);
}

void EditDefParms::Col12Slot(const QColor &c)
{
	ChangeColor(c, 12);
}

void EditDefParms::Col13Slot(const QColor &c)
{
	ChangeColor(c, 13);
}

void EditDefParms::Col14Slot(const QColor &c)
{
	ChangeColor(c, 14);
}

void EditDefParms::Col15Slot(const QColor &c)
{
	ChangeColor(c, 15);
}

void EditDefParms::Col1Slot(const QColor &c)
{
	ChangeColor(c, 1);
}

void EditDefParms::Col2Slot(const QColor &c)
{
	ChangeColor(c, 2);
}

void EditDefParms::Col3Slot(const QColor &c)
{
	ChangeColor(c, 3);
}

void EditDefParms::Col4Slot(const QColor &c)
{
	ChangeColor(c, 4);
}

void EditDefParms::Col5Slot(const QColor &c)
{
	ChangeColor(c, 5);
}

void EditDefParms::Col6Slot(const QColor &c)
{
	ChangeColor(c, 6);
}

void EditDefParms::Col7Slot(const QColor &c)
{
	ChangeColor(c, 7);
}

void EditDefParms::Col8Slot(const QColor &c)
{
	ChangeColor(c, 8);
}

void EditDefParms::Col9Slot(const QColor &c)
{
	ChangeColor(c, 9);
}
