#include "editplace.h"
#include <qstring.h>
#include <qcombobox.h>
#include "mylineedit.h"
#include "astrodata.h"
#include "astrohm.h"
#include <qradiobutton.h>
#include <qlineedit.h>
#include "astrowindow.h"

int EditPlace::SearchIdx(Index idx) const
{
  int i;
  for(i = 0; i < Nb; i++)
    if(IdxPtr[i] == idx)
      return i;
  return -1;
}

void EditPlace::UpdateAd(bool b)
{
  AstroHM hm;
  double offset;
  bool daylight;
  if(b)
  {
    if(!Arq.Exec("SELECT TzFile FROM Country WHERE Idx = %d", IdxPtr[Id]))
      return;
    Ad->SetTZ(Arq.Get(Arq.GetIndex("TzFile")), FALSE);
  }
  Ad->PlaceName  = PlaceName->text();
  Ad->PlaceIdx   = Idx;
  Ad->CountryIdx = IdxPtr[Id];
  hm.Set((AstroString)Latitude->text(), HM_Latitude);
  Ad->Latitude   = hm;
  hm.Set((AstroString)Longitude->text(), HM_Longitude);
  Ad->Longitude  = hm;
  Ad->Recompute();
}

void EditPlace::Clicked(QListViewItem* qlvi)
{
  SearchLine* sl = SC(SearchLine *, qlvi);
  if(!sl) return;
  AstroString as;
  Modified = FALSE;
  emit PlaceName->setText(sl->text(0));
  emit Latitude->setText(sl->text(1));
  emit Longitude->setText(sl->text(2));
  Id = SearchIdx(sl->CountryIdx);
  emit Country->setCurrentItem(Id);
  Idx = sl->Idx;
}

void EditPlace::Init(AstroData* ad)
{
  AstroString as;
  int i, n, idx;
  AstroHM hm;
  Ad = ad;
  Idx = Ad->PlaceIdx;
  emit PlaceName->setText(Ad->PlaceName);
  Ad->Latitude.Get(as, HM_Latitude);
  emit Latitude->setText(as);
  Ad->Longitude.Get(as, HM_Longitude);
  emit Longitude->setText(as);
  connect(SearchList, SIGNAL(clicked(QListViewItem*)), this, SLOT(Clicked(QListViewItem*)));
  if(!Arq.Exec("Select Name, Idx from Country"))
  {
    AstroMessage(i18n("Can't fetch countries names !"));
    return;
  }
  Nb = Arq.Number();
  IdxPtr = new Index[Nb];
  for(i = 0; i < Nb; i++, Arq.Next())
  {
    emit Country->insertItem(Arq < "Name", i);
    idx = Arq.GetInt("Idx");
    if(Ad->CountryIdx == idx)
      Id = i;
    IdxPtr[i] = idx;
  }
  Country->setCurrentItem(Id);
}
/* 
 *  Constructs a EditPlace which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
EditPlace::EditPlace( QWidget* parent, const char *c)
    : _EditPlace(parent, c, 0)
{
  Modified = FALSE;
  Id = 1; 
  SearchList->setColumnWidth(0, PlaceName->width());
  SearchList->setColumnWidth(1, Latitude->width());
  SearchList->setColumnWidth(2, Longitude->width());
  SearchList->setColumnWidth(3, Country->width() - 6);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
EditPlace::~EditPlace()
{
  delete IdxPtr;
}

/* 
 * public slot
 */
void EditPlace::DeleteSlot()
{ 
  if(!Arq.Exec("SELECT Data.Idx FROM Data, Place WHERE Place.Idx = %d AND Data.PlaceIdx = Place.Idx", Idx))
  {
    AstroMessage(i18n("Can't delete this place"));
    return;
  }
  if(Arq.Number() != 0 || Adl->CheckIdx(Idx) || Ahl->CheckIdx(Idx))
  {
    AstroMessage(i18n("Can't delete this place"));
    return;
  }
  Arq.Exec("DELETE FROM Place WHERE Idx = %d", Idx);
  delete SC(SearchLine*, SearchList->currentItem());
  CleanSlot();
}

void EditPlace::ModifySlot()
{
  double lat, lon;
  AstroHM hm;
  Index idx, countryidx;
  AstroString as, alat, alon;
  if(PlaceName->text().isEmpty() || Latitude->text().isEmpty() || 
     Longitude->text().isEmpty())
  {
    AstroMessage(i18n("Fields Name, Latitude or Longitude can't be empty !"));
    return;
  }
  alat = Latitude->text();
  hm.Set(alat, HM_Latitude);
  lat = hm.Get();
  alon = Longitude->text();
  hm.Set(alon, HM_Longitude);
  lon = hm.Get();
  as = PlaceName->text();
  countryidx = AstroData::FetchCountryIdx(Country->text(Id).data());
  if(!Arq.Exec("UPDATE Place SET Name = '%s', Latitude = '%f', Longitude = '%f', CountryIdx = %d WHERE Idx = %d", as.Utf8().Data(), lat, lon, countryidx, Idx))
  {
    AstroMessage(i18n("Can't modify new place"));
    return;
  }
  UpdateAd(TRUE);
  Modified = FALSE;
}

/* 
 * public slot
 */
void EditPlace::CountrySlot(int i)
{
  Id = i;
  UpdateAd(TRUE);
  emit OffTzRecomp();
}

void EditPlace::LatitudeSlot(const QString& s)
{
  Modified = TRUE;
  AstroHM hm;
  AstroString r;
  switch(hm.Set((AstroString)s, HM_Latitude))
  {
    case ASTROHM_WRONG_HOUR: 
      r = i18n("Error in latitude hour"); break;
    case ASTROHM_WRONG_SEPARATOR:
      r = i18n("Error in hour/minute separator or trailer"); break;
    case ASTROHM_WRONG_MINUTE:
      r = i18n("Error in latitude minute"); break;
    default: 
      r = "";
  }
  if(!r.isEmpty())
    AstroMessage(r);
}
/* 
 * public slot
 */
void EditPlace::LongitudeSlot(const QString& s)
{
  Modified = TRUE;
  AstroHM hm;
  AstroString r;
  HM_Mode hmm = Ad->HMS_Format ? HM_HMS : HM_Longitude;
  switch(hm.Set((AstroString)s, hmm))
  {
    case ASTROHM_WRONG_HOUR: 
      r = i18n("Error in longitude hour"); break;
    case ASTROHM_WRONG_SEPARATOR:
      r = i18n("Error in hour/minute separator or trailer"); break;
    case ASTROHM_WRONG_MINUTE:
      r = i18n("Error in longitude minute"); break;
    case ASTROHM_WRONG_SECOND:
      r = i18n("Error in longitude second"); break;
    default: 
      r = "";
  }
  if(!r.isEmpty())
    AstroMessage(r);
}
/* 
 * public slot
 */
void EditPlace::NameSlot(const QString&)
{
  Modified = TRUE;
}
/* 
 * public slot
 */
void EditPlace::NewSlot()
{
  double lat, lon;
  AstroHM hm;
  Index idx, countryidx;
  AstroString as, alat, alon;
  if(PlaceName->text().isEmpty() || Latitude->text().isEmpty() || 
     Longitude->text().isEmpty())
  {
    AstroMessage(i18n("Fields Name, Latitude or Longitude can't be empty !"));
    return;
  }
  alat = Latitude->text();
  hm.Set(alat, HM_Latitude);
  lat = hm.Get();
  alon = Longitude->text();
  hm.Set(alon, HM_Longitude);
  lon = hm.Get();
  as =  PlaceName->text();
  countryidx = AstroData::FetchCountryIdx(Country->text(Id).data());
  if(!Arq.Exec("INSERT INTO Place (Name, Latitude, Longitude, CountryIdx) VALUES('%s', %f, %f, %d)", as.Utf8().Data(), lat, lon, countryidx))
  {
    AstroMessage(i18n("Can't insert new place"));
    return;
  }
  idx = Arq.GetINSId();
  Arq.Exec("SELECT Idx FROM Place WHERE Oid = %d", idx);
  Idx = Arq.GetInt("Idx");
  Modified = FALSE;
  SearchList->clear();
  SearchList->insertItem(new SearchLine(Idx, SearchList, as, alat, alon, Country->text(Id), countryidx));
  UpdateAd(TRUE);
}
/* 
 * public slot
 */
void EditPlace::SearchSlot()
{
  int i, n;
  AstroString lat, lon, s;
  AstroHM hm;
  if(PlaceName->text().isEmpty())
  {
    AstroMessage(i18n("Place name must not be empty !"));
    return;
  }
  s = PlaceName->text();
  if(!Arq.Exec("SELECT Place.Idx AS PIdx, Place.Name AS PName, Latitude, Longitude, Country.Name AS CName, Country.Idx AS CIdx FROM Place, Country WHERE Place.Name ~* '%s' AND Place.CountryIdx = Country.Idx ORDER BY Place.Name", s.Utf8().Data()))
  {
    AstroMessage(i18n("Can't fetch places names !"));
    return;
  }
  n = Arq.Number();
  SearchList->clear();
  for(i = 0; i < n; i++, Arq.Next())
  {
    hm = Arq.GetDouble("Latitude");
    hm.Get(lat, HM_Latitude);
    hm = Arq.GetDouble("Longitude");
    hm.Get(lon, HM_Longitude);
    SearchList->insertItem(new SearchLine(Arq.GetInt("PIdx"), SearchList, Arq < "PName", lat, lon, Arq < "CName", Arq.GetInt("Cidx")));
  }
}

void EditPlace::CleanSlot()
{
  emit PlaceName->setText("");
  emit Latitude->setText("");
  emit Longitude->setText("");
  Idx = 0;
  CountrySlot(0);
  emit Country->setCurrentItem(0);
  Modified = TRUE;
}
