#include "editrestrictions.h"
#include "objrestr.h"
#include "asprestr.h"
#include <qheader.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <kcursor.h>
#include <qcombobox.h>
#include <kapp.h>
#include "astroutils.h"

extern KApplication *Kaptr;

/* 
 *  Constructs a EditObjRestr which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
EditRestrictions::EditRestrictions(AstroRestrictions* ar)
    : _EditRestrictions( 0, 0, TRUE, 0 )
{
  int a, b, c, d, e, i;
  Ar = ar;
  ArBase = *Ar;
  Return = FALSE;
  Kaptr->setOverrideCursor(KCursor::waitCursor());
  QString s = i18n("Object");
  Orl->Init(ArBase.Rest);
  Orl->GetLen(&a, &b, &c, &d);
  emit RSName->setText(ArBase.Name);
  if(ArBase.IfHouses())
    emit Toggle_House->setChecked(TRUE);
  else
    ToggleHousesSlot(FALSE);
  if(ArBase.IfExotic())
    emit Toggle_Exo->setChecked(TRUE);
  else
    ToggleExoticSlot(FALSE);
  if(ArBase.IfStars())
    emit Toggle_Stars->setChecked(TRUE);
  else
    ToggleStarsSlot(FALSE);
  Header->addLabel(s, a);
  s = i18n("Use");
   Header->addLabel(s, b);
  s = i18n("Orb Add");
  Header->addLabel(s, c);
  s = i18n("Influence");
  Header->addLabel(s, d);
  Arl->Init(ArBase.Asprst);
  Arl->GetLen(&a, &b, &c, &d, &e);
   if(ArBase.IfMinor())
    emit Toggle_Minor->setChecked(TRUE);
  else
    ToggleMinorSlot(FALSE);
  s = i18n("Aspect");
  AspHeader->addLabel(s, a);
  s = i18n("Use");
  AspHeader->addLabel(s, b);
  s = i18n("Orb");
  AspHeader->addLabel(s, c);
  s = i18n("Influence");
  AspHeader->addLabel(s, d);
  s = i18n("Color");
  AspHeader->addLabel(s, e);
  for(i = Placidus; i <= Alcabitus; i++)
    HousesCombo->insertItem(*Asr->HousesSystemNames[i], i);
  HousesCombo->setCurrentItem(ArBase.GetHouse());
  TrueNodeCheck->setChecked(ArBase.TrueNode);
  for(i = Tropical; i <= B_1950; i++)
    SiderealCombo->insertItem(*Asr->SiderealSystemNames[i + 1], i+ 1);
  SiderealCombo->setCurrentItem(ArBase.Sidereal + 1);
  Kaptr->setOverrideCursor(KCursor::arrowCursor());
  show();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
EditRestrictions::~EditRestrictions()
{
}

void EditRestrictions::OkSlot()
{
  if(RSName->text().isEmpty() /*|| ar.CheckName(as)*/)
  {
  	AstroMessage(i18n("This restrictions set must have a unique name"));
  	return;
  }
  close();
  ArBase.SetHousesSys((enum House)HousesCombo->currentItem());
  ArBase.TrueNode = TrueNodeCheck->isChecked();
  ArBase.Sidereal = (enum Sidereal)(SiderealCombo->currentItem() - 1);
  *Ar = ArBase;
  Ar->Name = RSName->text();
  Ar->Modified = TRUE;
  Return = TRUE;
}

void EditRestrictions::CancelSlot()
{
  close();
}

void  EditRestrictions::ToggleExoticSlot(bool b)
{
  int i;
  for(i = First_Exotic; i <= Last_Exotic; i++)
    Orl->Inhibit(i, !b);
  ArBase.RestExotic(b);
}

void EditRestrictions::ToggleHousesSlot(bool b)
{
  int i;
  for(i = Ascendant; i <= House_12; i++)
    Orl->Inhibit(i, !b);
   ArBase.RestHouses(b);
}

void EditRestrictions::ToggleStarsSlot(bool b)
{
  int i;
  for(i = First_Star; i <= Last_Star; i++)
    Orl->Inhibit(i, !b);
   ArBase.RestStars(b);
}

void EditRestrictions::RestrictAllSlot()
{
  int i;
  for(i = Sun; i <= Last_Planet; i++)
  {
    Orl->Check(i, FALSE);
    ArBase.Restrict(i, TRUE);
  }
  for(i = Ascendant; i <= House_12; i++)
  {
    Orl->Check(i, FALSE);
    ArBase.Restrict(i, TRUE);
  }
  for(i = First_Exotic; i <= Last_Exotic; i++)
  {
    Orl->Check(i, FALSE);
    ArBase.Restrict(i, TRUE);
  }
  for(i = First_Star; i <= Last_Star; i++)
  {
    Orl->Check(i, FALSE);
    ArBase.Restrict(i, TRUE);
  }
}

void EditRestrictions::ReleaseAllSlot()
{
  int i;
  for(i = Sun; i <= Last_Planet; i++)
  {
    Orl->Check(i, TRUE);
    ArBase.Restrict(i, FALSE);
  }
  for(i = Ascendant; i <= House_12; i++)
  {
    Orl->Check(i, TRUE);
    ArBase.Restrict(i, FALSE);
  }   
  for(i = First_Exotic; i <= Last_Exotic; i++)
  {
    Orl->Check(i, TRUE);
    ArBase.Restrict(i, FALSE);
  }
  for(i = First_Star; i <= Last_Star; i++)
  {
    Orl->Check(i, TRUE);
    ArBase.Restrict(i, FALSE);
  }
}

void EditRestrictions::InverseSlot()
{
  int i;
  for(i = Sun; i <= Last_Planet; i++)
  {
    Orl->Check(i, !Orl->IsChecked(i));
    ArBase.Restrict(i, ArBase == i);
  }
  for(i = Ascendant; i <= House_12; i++)
  {
    Orl->Check(i, !Orl->IsChecked(i));
    ArBase.Restrict(i, ArBase == i);
  }
  for(i = First_Exotic; i <= Last_Exotic; i++)
  {
    Orl->Check(i, !Orl->IsChecked(i));
    ArBase.Restrict(i, ArBase == i);
  }    
  for(i = First_Star; i <= Last_Star; i++)
  {
    Orl->Check(i, !Orl->IsChecked(i));
    ArBase.Restrict(i, ArBase == i);
  }
}

void EditRestrictions::ReleaseAllAspSlot()
{
  int i;
  for(i = Conjunction; i < Unknown_Aspect; i++)
  {
    Arl->Check(i, TRUE);
    ArBase.AspRestrict(i, FALSE);
  }
}

void EditRestrictions::RestrictAllAspSlot()
{
  int i;
  for(i = Conjunction; i < Unknown_Aspect; i++)
  {
    Arl->Check(i, FALSE);
    ArBase.AspRestrict(i, TRUE);
  }
}

void EditRestrictions::InverseAspSlot()
{
  int i;
  for(i = Conjunction; i < Unknown_Aspect; i++)
  {
    Arl->Check(i, !Arl->IsChecked(i));
    ArBase.AspRestrict(i, !ArBase.Asp(i));
  }
}

void EditRestrictions::ToggleMinorSlot(bool b)
{
  int i;
  for(i = First_Minor; i < Unknown_Aspect; i++)
    Arl->Inhibit(i, !b);
  ArBase.RestMinor(b);
}
