#include "objrestr.h"
#include "astrorestrictions.h"
#include "astroresources.h"
#include <qlabel.h>
#include "mylineedit.h"
#include "astroutils.h"
#include <cstdlib>

/* 
 *  Constructs a ObjRestr which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
ObjRestr::ObjRestr(struct Rest* rest, int i)
    : _ObjRestr(0, 0, 0 )
{
  char buf[7];
  AstroString as, *ptr = 0;
  Rest = rest;
  if(i >= First_Star && i <= Last_Star)
    ptr = Asr->StarsNames[i - First_Star];
  else if(i >= First_Exotic && i <= Last_Exotic)
    i = i - First_Exotic + 23;
  else if(i >= Ascendant && i <= House_12)
    i = i - Ascendant + 31;
  if(!ptr)
    ptr =  Asr->ObjNames[i];
  emit Name->setText(*ptr);
  emit IsRestricted->setChecked(!rest->Rest);
  sprintf(buf, "%.2f", rest->OrbAdd);
  as = buf;
  emit OrbAdd->setText(as);
  OrbAdd->setAlignment(Qt::AlignRight);
  //sprintf(buf, "%d", rest->Influence);
  //as = buf;
  emit Influence->setText(Sprintf("%d", &as,  rest->Influence));
  Influence->setAlignment(Qt::AlignRight);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
ObjRestr::~ObjRestr()
{
    // no need to delete child widgets, Qt does it all for us
}

ObjRestList::ObjRestList(QWidget* parent, const char*) : QScrollView(parent)
{
}

void ObjRestList::FillList(const struct Rest* rest, int first, int last)
{
  int i;
  for(i = first; i <= last; i++)
  {
    Or[i] = new ObjRestr((struct Rest*)&rest[i], i);
    addChild(Or[i], 0, Y);
    Y += H;
  }
}

void ObjRestList::Init(struct Rest* rest)
{
  int i, w;
  Rest = rest;
  Or[Sun] = new ObjRestr(&rest[Sun], Sun);
  addChild(Or[Sun], 0, 0);
  w = Or[Sun]->width();
  Y = H = Or[Sun]->height();
  FillList(rest, Moon, Last_Planet);
  FillList(rest, Ascendant, House_12);
  FillList(rest, First_Exotic, Last_Exotic);
  FillList(rest, First_Star, Last_Star);
  emit resizeContents(w, Y);
}

ObjRestList::~ObjRestList()
{
}

void ObjRestr::RestrictSlot(bool b)
{
  Rest->Rest = !b;
}

void ObjRestr::OrbAddSlot(const QString& s)
{
  double d = atof(s.data());
  if(d >= -10.0 && d <= 10.0) 
    Rest->OrbAdd = d;
  else
    AstroMessage(i18n("The orb add value must be defined within the interval -10.0 ... 10.0"));
}

void ObjRestr::InfluenceSlot(const QString& s)
{
  int i = atoi(s.data());
  if(i >= 0 && i <= 50)
    Rest->Influence = i;
  else
    AstroMessage(i18n("The influence value must be defined within the interval 0 ... 50"));
}

void ObjRestList::GetLen(int* a, int* b, int* c, int *d) const
{
  const ObjRestr& ors = *Or[Sun];
  *a = ors.IsRestricted->x();
  *b = ors.OrbAdd->x() - *a;
  *c = ors.Influence->x() - ors.OrbAdd->x();
  *d = width() - ors.Influence->x();
}

void ObjRestList::Inhibit(int i, bool b)
{
  ObjRestr& ori = *Or[i];
  emit ori.Name->setEnabled(!b);
  emit ori.IsRestricted->setEnabled(!b);
  emit ori.Influence->setEnabled(!b);
  emit ori.OrbAdd->setEnabled(!b);
}

void ObjRestList::Check(int i, bool b)
{
  emit Or[i]->IsRestricted->setChecked(b);
  Rest->Rest = !Rest->Rest;
}
