#include "searchdata.h"
#include "astroparams.h"
#include <qlineedit.h>
#include "astroutils.h"
#include <qbuttongroup.h>
#include "astromainwindow.h"

extern AstroMainWindow* Amw;

/* 
 *  Constructs a SearchData which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
SearchData::SearchData()
    : _SearchData(0, 0, TRUE, 0)
{
  SQLReqType = 0;
  RequestType->setButton(SQLReqType);
  DataLIST->setMultiSelection(TRUE);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
SearchData::~SearchData()
{
    // no need to delete child widgets, Qt does it all for us
}

/* 
 * public slot
 */
void SearchData::ClickedSlot(QListViewItem*)
{ 
}
/* 
 * public slot
 */
void SearchData::OkSlot()
{
  SearchItem* sa = SC(SearchItem*, DataLIST->firstChild());
  for(; sa; sa = SC(SearchItem*, sa->nextSibling()))
  {
    if(sa->isSelected() && !Adl->Get(sa->Idx))
    {
      AstroData* ad = new AstroData;
      CHECK_PTR(ad);
      ad->DbFetch(sa->Idx);
      Amw->Aiw->AddData(ad);
    }
  }
  close();
}
/* 
 * public slot
 */
void SearchData::ReqTypeSlot(int i)
{
  SQLReqType = i; 
}
/* 
 * public slot
 */
void SearchData::SearchSlot()
{
  int i;
  char buf[3 * BUFSIZE];
  if(SQLReqType == 0)
  {
  	if(ReqLine->text().isEmpty())
  	 strcpy(buf,  "Data.Name LIKE '\\%' ");
  	else
    snprintf(buf, 3 * BUFSIZE, "Data.Name ~* '%s'", (const char *)ReqLine->text().utf8());
  }
  else
    strncpy(buf, (const char *)ReqLine->text().utf8(), 3 * BUFSIZE);
  if(!Arq.Exec("SELECT Data.Name AS DName, Data.Idx AS Dix, BirthDay, ChartType, Place.Name AS PName FROM Data, Place WHERE Data.PlaceIdx = Place.Idx AND %s ORDER BY Data.Name", buf))
  {
    AstroMessage(i18n("Can't fetch data !"));
    return;
  }
  DataLIST->clear();
  for(i = 0; i < Arq.Number(); i++, Arq.Next())
    (void)new SearchItem(DataLIST, Arq.GetInt("Dix"), Arq < "Dname", Arq < "BirthDay", Arq < "PName", *(*Ail)[Arq.GetInt("ChartType")]);
}
/* 
 * public slot
 */
void SearchData::CancelSlot()
{
  close();
}

SearchItem::SearchItem(QListView* parent, Index idx, const AstroString& name, const AstroString& date, const AstroString& place, const AstroIcon& icon) : QListViewItem (parent, name, date, place)
{
  Idx = idx;
  setPixmap(0, icon);
}
