/***************************************************************************
                          stdwheelchart.cpp  -  description
                             -------------------
    begin                : Sat Dec 30 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stdwheelchart.h"
#include "astrorings.h"
#include "astrodefs.h"
#include "astroresources.h"

StdWheelChart::StdWheelChart(ChartBaseParams *cbp, ObjsVector* ov) : GraphicChartBase(cbp, ov, cbp->Ag)
{
  Ar = 0;
  Asb = Asb1 = 0;
  ct = StdWheels;
  cm = GraphicMode;
  DualChart = FALSE;
  DrawChart();
}

StdWheelChart::~StdWheelChart()
{
  if(Ar)   delete Ar;
  if(Asb)  delete Asb;
  if(Asb1) delete Asb1;
}

void StdWheelChart::DrawChart()
{
  int f, s, c, size = Ag->GetSize(), h = size * 37;
  AstroString as;
  f = Search(1, Any, TRUE);
  s = Search(2, Any, TRUE);
  const AstroObjs& a = *(*this)[f];
  if(Ar)   delete Ar;
  if(Asb)  delete Asb;
  if(Asb1) delete Asb1;
  if(Cbp->Sidebars)
  {
    if(Count() > 1)
    {
      Asb = new AstroSideBar(Ag, 0, size , size * 10, h, Cbp, this, s);
      CHECK_PTR(Asb);
      c = Asb->GetWidth();
      Ar = new AstroRings(Ag, c, size, h, Cbp, this);
      CHECK_PTR(Ar);
      c += Ar->GetWidth();
      Asb1 = new AstroSideBar(Ag, c, size, size * 10, h, Cbp, this, f);
      CHECK_PTR(Asb1);
    }
    else
    {
      Ar = new AstroRings(Ag, 0, size, h, Cbp, this);
      CHECK_PTR(Ar);
      Asb = new AstroSideBar(Ag, Ar->GetWidth(), size, size * 10, h, Cbp, this, a.Show ? f : s);
      CHECK_PTR(Asb);
      Asb1 = 0;
    }
  }
  else
  {
    Ar =  new AstroRings(Ag, 0, size, size * 37, Cbp, this);
    CHECK_PTR(Ar);
    Asb = Asb1 = 0;
  }
}

bool StdWheelChart::Redisplay()
{
  if(IfRedrawChart)
  {
    DrawChart();
    IfRedrawChart = FALSE;
  }
  Ar->Repaint();
  if(Asb)  Asb->Repaint();
  if(Asb1) Asb1->Repaint();
  return TRUE;
}
